c       If Fortran had data structures and pointers they would be used here.
C       ================================================================
C       A common block of information about the location of the cursor.
C       When each locator event (keystroke in crosshair mode or mouse click)
C       is received, the elements of this common block are updated to reflect
C       the location.
C       ================================================================
c                       the Echelle wavelength nearest to the cursor [Aangstrom]
	real            clambda
c                       the Echelle order which was nearest to the cursor
	integer         corder
c                       the blaze (central) wavelength of the order [Aangstrom]
	real            clambdab
c                       mean dispersion in this order [Aangstroms pixel**-1]
	real            dispapx
c                       mean dispersion in this order [Aangstroms mm**-1]
	real            dispamm
c                       separation between this order and next (last?) [arcsec]
	real            cseps
c                       separation between this order and the next (last?) [mm]
	real            csepm
c                       separation between this order and the next (last?) [pix]
	real            csepp
c                       position of center of the order [mm]
	real            cloc
c                       length of the order [mm]
	real            clen
c                       index of detector which contains this point
c                       this is < 0 if no detector contains the point
	integer         idet
c                       returned value of detector pixel under the point
c                       undefined if no detector contains the point
	real            dpx, dpy
c                       slit width in velocity [m/s]
	real            cslwid
c                       decker height in pixels
	real            cdkpix
C       ================================================================
c       Extreme care must be taken with these pointers to insure that
c       the data in the equivalence do not overlap.
	integer         i_curspsn
	integer         i_clambda
	integer         i_corder
	integer         i_clambdab
	integer         i_dispapx
	integer         i_dispamm
	integer         i_cseps
	integer         i_csepm
	integer         i_csepp
	integer         i_cloc
	integer         i_clen
	integer         i_idet
	integer         i_dpx
	integer         i_dpy
	integer         i_cslwid
	integer         i_cdkpix
	parameter       (i_curspsn  =  0)
	parameter       (i_clambda  =  i_curspsn  + 1)
	parameter       (i_corder   =  i_clambda  + 1)
	parameter       (i_clambdab =  i_corder   + 1)
	parameter       (i_dispapx  =  i_clambdab + 1)
	parameter       (i_dispamm  =  i_dispapx  + 1)
	parameter       (i_cseps    =  i_dispamm  + 1)
	parameter       (i_csepm    =  i_cseps    + 1)
	parameter       (i_csepp    =  i_csepm    + 1)
	parameter       (i_cloc     =  i_csepp    + 1)
	parameter       (i_clen     =  i_cloc     + 1)
	parameter       (i_idet     =  i_clen     + 1)
	parameter       (i_dpx      =  i_idet     + 1)
	parameter       (i_dpy      =  i_dpx      + 1)
	parameter       (i_cslwid   =  i_dpy      + 1)
	parameter       (i_cdkpix   =  i_cslwid   + 1)
C       ----------------------------------------------------------------
c                       lower limit of array of menu entries
	integer         MINCDATA
	parameter       (MINCDATA = i_curspsn)
c                       upper limit of array of menu entries
	integer         MAXCDATA
	parameter       (MAXCDATA = i_cdkpix)
c                       bogus pointer value
	integer         NULCDATA
	parameter       (NULCDATA = MINCDATA - 1)
C       ================================================================
c                       equivalence to all of the above
	real            cdatar(MINCDATA:MAXCDATA)
	integer         cdatai(MINCDATA:MAXCDATA)
	equivalence     (cdatar, cdatai)
c                       some items come with accelerator keys
c                       that allow their values to be changed.
	character*1     cdataa(MINCDATA:MAXCDATA)
c                       pointers telling which ones to actually display
c                       and in what order to display them
c                       if the pointer value is NULCDATA, it is skipped
	integer         cdatap(MINCDATA:MAXCDATA)
c                       format to use to display each value
	character       cdataf(MINCDATA:MAXCDATA)*(14)
c                       character strings to use to display label strings
	character       cdatal(MINCDATA:MAXCDATA)*(LLABSTR)
c                       character strings to use to display value strings
	character       cdatav(SVOLD:SVNEW,MINCDATA:MAXCDATA)*(LVALSTR)
c                       positions of the labels in Normalized Device Coords
	real            cdatax(Xp:Yp,MINCDATA:MAXCDATA,2)
C       ----------------------------------------------------------------
	equivalence     (clambda , cdatar(i_clambda))
	equivalence     (corder  , cdatai(i_corder))
	equivalence     (clambdab, cdatar(i_clambdab))
	equivalence     (dispapx , cdatar(i_dispapx))
	equivalence     (dispamm , cdatar(i_dispamm))
	equivalence     (cseps   , cdatar(i_cseps))
	equivalence     (csepm   , cdatar(i_csepm))
	equivalence     (csepp   , cdatar(i_csepp))
	equivalence     (cloc    , cdatar(i_cloc))
	equivalence     (clen    , cdatar(i_clen))
	equivalence     (idet    , cdatai(i_idet))
	equivalence     (dpx     , cdatar(i_dpx))
	equivalence     (dpy     , cdatar(i_dpy))
	equivalence     (cslwid  , cdatar(i_cslwid))
	equivalence     (cdkpix  , cdatar(i_cdkpix))
C       ----------------------------------------------------------------
c                       is this menu currently displayed?
	logical         cdataup
c                       is this menu currently displayed?
	logical         cmeninit
c                       NDC y limits of this menu
	real            cminndc
	real            cmaxndc
c                       bottom entry to be displayed
	integer         botcdata
C       ----------------------------------------------------------------
c                       the cross-disperser wavelength nearest to the cursor
c                       [Aangstrom]
	real            cxlambda
c                       position of cursor on echelle format [mm]
	real            xclast, yclast
C       ----------------------------------------------------------------
	common  /cdata/
     &          cdatar, cdatap, cminndc, cmaxndc, cdatax,
     &          cxlambda, xclast, yclast,
     &          botcdata,
     &          cdataup, cmeninit
	common  /cdatastr/      cdataf, cdatal, cdatav, cdataa
