#include "include/port.h"
C==============================================================================
C==============================================================================
	subroutine pixpos(imos,idet,cx,ry,ex,ey)
c       determine the position of a pixel measured in the echelle format coords
	IMPLICIT_NONE
#       include "echelle.par"
#       include "include/units.par"
c       External Variables
c                       which of the stored mosaic positions is this?
	integer         imos
c                       which of the detectors in the mosaic is this?
	integer         idet
c                       row and column number of the pixel on the detector
	real            ry, cx
c                       position of this pixel on the echelle format [mm]
	real            ex, ey
c       Internal Variables
c                       position of this pixel on the detector mosaic [m]
	real            mx, my
C       Common Block
	include 'detmos.inc'
	include 'setup.inc'
C       Executable code
c       first convert from pixel coords to detector mosaic coords
c       write(STDERR,*) 'cx, ry', cx, ry
c       write(STDERR,*) 'pixpos,posmos',imos,posmos(xp,imos),posmos(yp,imos)
c       mx = tmc(Xp,Xo,C2M,idet) * cx + tmc(Xp,Yo,C2M,idet) * ry +
c    &  tmc(Xp,To,C2M,idet)
c       my = tmc(Yp,Xo,C2M,idet) * cx + tmc(Yp,Yo,C2M,idet) * ry +
c    &  tmc(Yp,To,C2M,idet)
	mx = cx
	my = ry
	call translate(mx, my, tmc(Xp,Xo,C2M,idet), 1)
c       write(STDERR,*) 'mx, my', mx, my
c       then convert from detector mosaic coords to display coords
	ex = (tmd(Xp,Xo,M2D) * mx + tmd(Xp,Yo,M2D) * my) * mmPERm +
     &  posmos(Xp,imos)
	ey = (tmd(Yp,Xo,M2D) * mx + tmd(Yp,Yo,M2D) * my) * mmPERm +
     &  posmos(Yp,imos)
c       write(STDERR,*) 'ex, ey', ex, ey
	return
	end
C==============================================================================
C==============================================================================
	subroutine mosize()
c       Calculate the size of a rectangle completely enclosing the mosaic
c       These coordinates are in the coordinate system of the echelle
c       focal plane.
c       This defines the contents of the cormos array
c       cormos(Xp,0) is the X location of the geographic center of the mosaic
c       cormos(Yp,0) is the Y location of the geographic center of the mosaic
c       cormos(Xp,1) is the X location of the leftmost    pixel in the mosaic
c       cormos(Yp,1) is the Y location of the bottommost  pixel in the mosaic
c       cormos(Xp,2) is the X location of the rightmost   pixel in the mosaic
c       cormos(Yp,2) is the Y location of the topmost     pixel in the mosaic
c       As the cormos positions are computed using subroutine pixpos,
c       all cormos entries have the dimension [mm].
	IMPLICIT_NONE
#       include "echelle.par"
#       include "include/units.par"
C       External Variables
C       Internal Variables
	real            px, py
	integer         i
C       Common Blocks
	include 'detmos.inc'
	include 'setup.inc'
C       Executable Code
	posmos(Xp,0) = 0.
	posmos(Yp,0) = 0.
	call distran
	call pixpos(0,1,0.,0.,px,py)
	cormos(Yp,1) = py
	cormos(Yp,2) = py
	cormos(Xp,1) = px
	cormos(Xp,2) = px
	do 10 i = 1,nchips
	  call pixpos(0,i,0.,0.,px,py)
	  cormos(Yp,1) = min(cormos(Yp,1),py)
	  cormos(Yp,2) = max(cormos(Yp,2),py)
	  cormos(Xp,1) = min(cormos(Xp,1),px)
	  cormos(Xp,2) = max(cormos(Xp,2),px)
	  call pixpos(0,i,npix(i,Xp)+1.,npix(i,Yp)+1.,px,py)
	  cormos(Yp,1) = min(cormos(Yp,1),py)
	  cormos(Yp,2) = max(cormos(Yp,2),py)
	  cormos(Xp,1) = min(cormos(Xp,1),px)
	  cormos(Xp,2) = max(cormos(Xp,2),px)
	  call pixpos(0,i,npix(i,Xp)+1.,0.,px,py)
	  cormos(Yp,1) = min(cormos(Yp,1),py)
	  cormos(Yp,2) = max(cormos(Yp,2),py)
	  cormos(Xp,1) = min(cormos(Xp,1),px)
	  cormos(Xp,2) = max(cormos(Xp,2),px)
	  call pixpos(0,i,0.,npix(i,Yp)+1.,px,py)
	  cormos(Yp,1) = min(cormos(Yp,1),py)
	  cormos(Yp,2) = max(cormos(Yp,2),py)
	  cormos(Xp,1) = min(cormos(Xp,1),px)
	  cormos(Xp,2) = max(cormos(Xp,2),px)
10      continue
c       find the central location of the mosaic
	cormos(Xp,0) = 0.5 * (cormos(Xp,1) + cormos(Xp,2))
	cormos(Yp,0) = 0.5 * (cormos(Yp,1) + cormos(Yp,2))
c       move this mosaic completely out of the picture
	posmos(Xp,0) = 1.e9
	posmos(Yp,0) = 1.e9
	return
	end
C==============================================================================
C==============================================================================
