C       Common block carrying parameters describing the detectors and mosaics
c
c
c
c
c
c
c       Assumptions:
c       detectors are identical (this is easy to relax)
c       detectors are rectangular
COMMENT         ========================================================
COMMENT                 The Coordinate System of a Detector
COMMENT         Each CCD chip has its own coordinate system.  This is to be
COMMENT         distinguished from the coordinate system of the focal plane
COMMENT         which is handled much further below.
COMMENT
COMMENT         The terminology used to describe this coordinate system is
COMMENT         drawn from the early CCDs which were intended for use in
COMMENT         standard video applications.  Such CCDs had a single
COMMENT         readout amplifier located in a position such that the chip
COMMENT         could be used to produce a signal quite similar to that of
COMMENT         a vidicon.  They were designed to be illuminated from one
COMMENT         particular side so that the handedness of the output signal
COMMENT         would be proper for video applications.
COMMENT
COMMENT         Newer CCDs may have been thinned to allow illumination from
COMMENT         the "back" side.  They may also have multiple readout
COMMENT         amplifiers.  These new capabilities require the precise
COMMENT         specification of the orientation and/or handedness of a
COMMENT         particular detector.
COMMENT
COMMENT         Devices other than CCDs which have rectangular layouts
COMMENT         should use the same terms and coordinate orientation.
COMMENT         --------------------------------------------------------
COMMENT                 Definition of Detector Layout Terminology
COMMENT         This section describes a standard video CCD for reference.
COMMENT
COMMENT         The "x", "horizontal", & "serial" directions are parallel.
COMMENT         The "y", "vertical", & "parallel" directions are parallel.
COMMENT         A serial shift carries charge in the "minus x" direction.
COMMENT         A parallel shift carries charge in the "minus y" direction.
COMMENT         The set of all pixels with the same "y" coordinate is a "row"
COMMENT         The set of all pixels with the same "x" coordinate is a "column"
COMMENT         The words "height" and "width" are associated with
COMMENT         "vertical" and "horizontal", respectively.
COMMENT
COMMENT         In most CCDs, the serial shift register has "pixels" which
COMMENT         must be read out before the charge from the actual photo-
COMMENT         sensitive region of the chip is read.  These are known as
COMMENT         underscan pixels.  Most CCD controllers will also continue
COMMENT         to do serial shifts after all the charge from the photo-
COMMENT         sensitive region has been read.  These are known as overscan,
COMMENT         bias, or baseline pixels.
COMMENT
COMMENT         Q: This document does not treat the existence of these
COMMENT         Q: underscan and overscan pixels, it refers only to the
COMMENT         Q: "real" photosensitive pixels in the array.
COMMENT         Q: For engineering purposes it will be necessary to store
COMMENT         Q: some images with their under- and over-scan pixels.
COMMENT         Q: FITS keywords will be needed which define the number
COMMENT         Q: of these under/overscan pixels.
COMMENT
COMMENT         Assume an observer who is looking at the light-sensitive
COMMENT         surface of the CCD from a viewpoint which would obscure the
COMMENT         the incoming light.  As seen by this observer, the CCD's
COMMENT         coordinate system (x,y) is a right-handed coordinate system.
COMMENT         The readout amplifier is located at the lower left corner
COMMENT         of the chip.  This amplifier is closest to the pixel
COMMENT         identified as (0,0).
COMMENT
COMMENT         A diagram of a detector intended for video applications when
COMMENT         viewed as specified above is presented here:
COMMENT                      ^  (0,YPIX-1)  (1,YPIX-1)  ...  (XPIX-1,YPIX-1)
COMMENT                      |  (0,YPIX-2)  (1,YPIX-2)  ...  (XPIX-1,YPIX-2)
COMMENT     y_axis_direction |   ...         ...        ...        ...
COMMENT                      |  (0,1)       (1,1)       ...  (XPIX-1,1)
COMMENT  amplifier_is_here-> a  (0,0)       (1,0)       ...  (XPIX-1,0)
COMMENT                          ----x_axis_direction------->
COMMENT         parallel shifts move the charge "down" in this picture
COMMENT         serial shifts move the charge "left" in this picture
COMMENT         The above diagram does not show underscan or overscan pixels
COMMENT         in the serial shifts.  Underscan pixels would be to the
COMMENT         left of the (0,Y) pixels and overscan pixels would be to the
COMMENT         right of the (XPIX-1,Y) pixels.
COMMENT         The shift directions used here are those that would be used
COMMENT         when a single amplifier is used to readout the chip.
COMMENT         With a single amplifier the order of readout is
COMMENT         (0,0), (1,0), (2,0), ... (XPIX-1,0), (0,1), ... etc.
COMMENT         Thus, row 0 is read, left to right, then row 1, etc.
COMMENT         Note again that this is a right-handed coordinate system.
COMMENT
COMMENT         Assume that a simple lens makes an image of an object on a
COMMENT         video detector as described above.  When the image is read
COMMENT         in the described order and displayed on a video display in
COMMENT         the standard fashion, the image on the video display will
COMMENT         appear upright and not reversed.  The layout of a standard
COMMENT         video display is shown below, for reference:
COMMENT                          ----x_coord_direction------->
COMMENT     amp_seems_here-> a  (0,0)     , (1,0)     , ..., (XPIX-1,0)     ,
COMMENT                      |  (0,1)     , (1,1)     , ..., (XPIX-1,1)     ,
COMMENT    y_coord_direction |   ...      ,  ...      , ...,       ...      ,
COMMENT                      |  (0,YPIX-2), (1,YPIX-2), ..., (XPIX-1,YPIX-2),
COMMENT                      v  (0,YPIX-1), (1,YPIX-1), ..., (XPIX-1,YPIX-1)
COMMENT         parallel shifts appear to move charge "up" in this picture
COMMENT         serial shifts appear to move charge "left" in this picture
COMMENT         Note that this is a LEFT-handed coordinate system, which is
COMMENT         also known as a video-coordinate system.
COMMENT
COMMENT         --------------------------------------------------------
COMMENT                 "Left-handed" and Multi-amplifier Detectors
COMMENT         A CCD designed for video use may be thinned and illuminated
COMMENT         from the "back" to improve its quantum efficiency.  Such a
COMMENT         detector will still have its (0,0) pixel as the pixel closest
COMMENT         to the amplifier.  However in such usage the handedness of
COMMENT         an image produced by this CCD will be the opposite of standard.
COMMENT         In this case, the coordinate system of the CCD (as seen by the
COMMENT         observer looking at its light-sensitive surface) is a
COMMENT         left-handed coordinate system.  This must be indicated in
COMMENT         the DETPOS card for this detector (see below).
COMMENT
COMMENT         A CCD with multiple amplifiers is a simple extension of the
COMMENT         above rules.  One amplifier should be chosen as the reference
COMMENT         amplifier, and the pixel nearest that will be designated (0,0).
COMMENT         All pixels will be numbered from the designated origin as
COMMENT         if they were being read through the reference amplifier.
COMMENT         The handedness of the detector will be determined by
COMMENT         comparing the handedness of readout through the reference
COMMENT         amplifier with the handedness of a standard video detector.
COMMENT         Q: How is the (baseline, amplifier bias, overscan)
COMMENT         Q: going to be represented in data where multiple amplifiers
COMMENT         Q: are used to read a single chip's imaging area?
COMMENT         Q: (There are no immediate plans to use multi-amplifier
COMMENT         Q: readout, but the Ford (Loral?) chips could do it.)
COMMENT         Q: Might considerations of the baseline measurement for a
COMMENT         Q: multi-amp chip make it necessary to treat it as
COMMENT         Q: several separate chips each making separate images?
COMMENT         Q: If this is the case, the current DETPOS cards will be
COMMENT         Q: rather awkward for specifying the positions.
COMMENT         ========================================================
COMMENT                 Sites which use non-standard video display orientation
COMMENT         It is recognized that there are sites which for one reason
COMMENT         or another display their video images in a manner which
COMMENT         is not standard.  For this reason, the Echelle Simulator
COMMENT         defines three keywords which can be used to flip the
COMMENT         orientation of everything which it displays.  This makes
COMMENT         it easy to match the Echelle Simulator graphics with the
COMMENT         images produced by the actual detector.
COMMENT
COMMENT         Note that the orientation of the video display is actually
COMMENT         an artifact of the video display and of the program which
COMMENT         is being used to drive the video display.  Thus, these
COMMENT         keywords are not strictly associated with the detector,
COMMENT         but rather with the displayer.  Nonetheless, they are
COMMENT         included in the detector file for convenience.
	logical         stdvidv
	logical         stdvidh
	logical         stdvidr
COMMENT         ========================================================
COMMENT                 The Definition of a Pixel
COMMENT         When real-valued coordinates are used, the middle of a
COMMENT         pixel is at a half-integral location.  The real limits of
COMMENT         pixel 0 run from 0. to 1.  To convert from real coordinate
COMMENT         values to integer coordinate values, use the "floor()" function.
COMMENT         To convert from integer coordinate values to real coordinate
COMMENT         values, add 0.5 to the integer value.  These conventions are
COMMENT         generally accepted by the computer graphics community.
COMMENT         They also make it easier to compute, because operations which
COMMENT         deal with entire pixels should be possible using only integer
COMMENT         arithmetic.
COMMENT         ========================================================
COMMENT                 The Detector Identification
COMMENT         Every individual CCD chip (or other similar detector)
COMMENT         is intended to have a unique number associated with it.
COMMENT         This unique number should serve to allow lookup of the
COMMENT         defects and blemishes which are peculiar to that chip.
COMMENT         We expect that some kind of defect and blemish database
COMMENT         will be established which will identify the nature of
COMMENT         the known problems and even tell about problems that
COMMENT         change with time (i.e., new defects arise or old ones vanish).
COMMENT         Note that it has been determined recently that ChipIDs will
COMMENT         be character strings.
	character*80    detector
	integer         nchips
	character*80    chipid(MAXDETMOS)
c
c
c
COMMENT         --------------------------------------------------------
COMMENT                 The number of Pixels
COMMENT         If XPIXn and YPIXn are specified, they apply to detector "n".
COMMENT         If XPIX and YPIX are specified, they apply to all detectors
COMMENT         which were not explicitly specified with XPIXn and YPIXn.
	integer         npix(0:MAXDETMOS, Xp:Yp)
c
COMMENT         Q: How is the (baseline, amplifier bias, overscan)
COMMENT         Q: going to be represented in Keck CCD data?
COMMENT         Q: Will it be represented in extra pixels which are not a
COMMENT         Q: part of the image data?  If so, what keyword will
COMMENT         Q: describe this in the final FITS header for the image?
COMMENT         --------------------------------------------------------
COMMENT                 The spacing of Pixels
COMMENT         Note that PIXXSIZ has become PIXXSZ and likewise for PIXYSIZ.
COMMENT         This was in response to changes made after the 1991 May
COMMENT         DAWG meeting.
COMMENT         The pixel separation in meters is given by PIXXSZn and PIXYSZn.
COMMENT         Al Conrad and Steve Allen agreed on this change 1991 March 22.
COMMENT         If PIXXSIZn and PIXYSIZn are given, they apply to detector "n".
COMMENT         If PIXXSIZ and PIXYSIZ are given, they apply to all detectors
COMMENT         which were not explicitly specified with PIXXSIZn and PIXYSIZn.
	real            pixsiz(0:MAXDETMOS, Xp:Yp)
c
COMMENT         --------------------------------------------------------
COMMENT                 Blemish Information
COMMENT         Bad spots (dead pixels, hot pixels, etc.) on the CCD chips
COMMENT         are described by BSPOT arrays.  Bad spots are assumed to
COMMENT         describable as rectangular regions on the chip.  Bad regions
COMMENT         with more complex shapes can be decomposed into rectangles.
COMMENT         It is not yet clear whether BSPOTs may overlap.
COMMENT         The FITS cards for a given image should simply list the
COMMENT         current bad spots of a detector.  This should be looked up
COMMENT         from a badspot database which may contain a time history
COMMENT         of the bad spots and further information on whether particular
COMMENT         badspots are hot, dead, have poor CTE, etc.
COMMENT
COMMENT         Q: Everybody agrees that this badspot database is a good idea,
COMMENT         Q: but nobody seems to want to define it.
COMMENT
COMMENT         BSPOT arrays consist of 5 integer elements
COMMENT         Note that the order of the positions has been swapped from
COMMENT         that in the CDR document.  This was to put the elements in
COMMENT         the order (x,y) as opposed to the original specs (y,x).
COMMENT         Al Conrad and Steve Allen agreed on this change 1991 Mar 26.
COMMENT         Note also that BADSPOT has become BSPOTnnn.  This was in
COMMENT         response to the 1991 May DAWG meeting which decided that
COMMENT         no keywords should be repeated.  This also introduced NBSPOT.
COMMENT             1) The index indicating which chip has this spot.
COMMENT                If this is n, it refers to CHIPIDn in DETPOSn.
COMMENT             2,3) The column and row (x,y) of the pixel in this
COMMENT                  bad rectangle which is closest to the origin.
COMMENT             4,5) the width and height (dx,dy) of this rectangle.
COMMENT         Note that it is permissible for the height and width of
COMMENT         the bad spot to extend far off the chip.  The software will
COMMENT         clip the badspot to the actual pixel limits.
	integer         nbspot
	integer         bspot(cID:dY, MAXBADSPOT)
c
c
c
c
c
COMMENT         ========================================================
COMMENT                 The Detector Focal Plane Coordinate System
COMMENT         Assume an observer who is looking at the light-sensitive
COMMENT         surface of the detector mosaic from a viewpoint which would
COMMENT         obscure the incoming light.  The coordinate system for the
COMMENT         detector mosaic as seen by that observer is left-handed.
COMMENT         The origin (0.,0.) of the Focal Plane system is some
COMMENT         well-defined point near the center of the mosaic.
COMMENT         The x-axis of the Focal Plane system may be chosen to
COMMENT         point in any direction which is convenient.
COMMENT         "Convenient" should be defined with consideration of the
COMMENT         orientations of the detector and the image or spectrum which
COMMENT         will be formed on the detector.  It will be convenient for
COMMENT         the simulator graphics software if all the coordinate systems
COMMENT         of chips and mosaic are aligned with the 4 cardinal
COMMENT         directions of compass points.
COMMENT         Thus, the x-axis might point "right" and the
COMMENT         y-axis point "down" as seen by the observer described above.
COMMENT         Alternatively, the x-axis might point "down" and y point "left".
COMMENT         In any case, the instrument simulator should provide another
COMMENT         FITS card which gives the relative orientation of the
COMMENT         Focal Plane system and the simulator coordinate system.
COMMENT         --------------------------------------------------------
COMMENT                 The Detector Position Cards
COMMENT         The content of the array is [ x0, y0, Rot, Orient]
COMMENT
COMMENT         x0, and y0 are the position of the (0,0) pixel of the chip
COMMENT         measured in meters in the mosaic coordinate system.
COMMENT         For precision of documentation, and because the software
COMMENT         must know, the precise reference point of the (0,0) pixel is
COMMENT         the corner of the pixel that would be directly between
COMMENT         (0,0) and (-1,-1), as if pixel (-1,-1) actually existed.
COMMENT         This is location (0.,0.) in the floating point system.
COMMENT         (We do not expect that the actual measurements of the location
COMMENT         of this pixel will be precise enough to justify this subtlety.)
COMMENT         When there is only 1 detector in the mosaic, the values of
COMMENT         x0 and y0 may be ignored by the software; the center of
COMMENT         the detector may be placed at the center of the focal plane.
COMMENT
COMMENT         Rot is measured in degrees.  Rot is the angle measured from
COMMENT         the x-axis of the focal plane coordinate system to the
COMMENT         x-axis of the detector.  This angle is measured in a
COMMENT         clockwise sense as seen by an observer who is blocking the
COMMENT         incoming light.
COMMENT
COMMENT         Orient is either +1 or -1, depending on the handedness of
COMMENT         the detector.  Orient should be +1 for a detector which is
COMMENT         being readout with the same handedness as a device intended
COMMENT         for standard video applications.  Orient should be -1 for
COMMENT         a detector with handedness opposite of standard video.
COMMENT
c                       stored in the same order as found in the CHIPIDS card
	real            detpos(Xp:RpY, 1:MAXDETMOS)
COMMENT ----------------------------------------------------------------
c                       Transformation matrices which convert from mosaic coords
c                       to individual chip coords and back again.
	real            tmc(Xp:Yp, Xo:To, M2C:C2M, 1:MAXDETMOS)
COMMENT ----------------------------------------------------------------
c                       The silicon of a CCD detector and the nature of
c                       the readout hardware may force the existence of
c                       a certain number of prescan pixels which will be
c                       read out by each serial amplifier at the beginning
c                       of each row.  This indicates how many such pixels
c                       there are and it is used in subroutine ratwindow
c                       to calculate the actual readout regions in the
c                       presence of various values of WINDOW and BINNING.
	integer         phprepix(1:MAXDETMOS)
COMMENT ================================================================
	integer         i_DETECTOR
	integer         i_STDVIDV
	integer         i_STDVIDH
	integer         i_STDVIDR
	integer         i_NCHIPS
	integer         i_CHIPID
	integer         i_NPIX
	integer         i_PIXSIZ
	integer         i_PIXXSIZ
	integer         i_PIXYSIZ
	integer         i_NBSPOT
	integer         i_BSPOT
c       Extreme care must be taken with these pointers to insure that
c       the data in the equivalence do not overlap.
	parameter       (i_DETECTOR = 0)
	parameter       (i_STDVIDV  = i_DETECTOR + 1)
	parameter       (i_STDVIDH  = i_STDVIDV  + 1)
	parameter       (i_STDVIDR  = i_STDVIDH  + 1)
	parameter       (i_NCHIPS   = i_STDVIDR  + 1)
	parameter       (i_CHIPID   = i_NCHIPS   + 1)
	parameter       (i_NPIX     = i_CHIPID   + MAXDETMOS)
	parameter       (i_PIXSIZ   = i_NPIX  + (MAXDETMOS+1)*(Yp-Xp+1))
	parameter       (i_PIXXSIZ  = i_PIXSIZ      )
	parameter       (i_PIXYSIZ  = i_PIXXSIZ  + 1)
	parameter       (i_NBSPOT   = i_PIXSIZ +(MAXDETMOS+1)*(Yp-Xp+1))
	parameter       (i_BSPOT    = i_NBSPOT   + 1)
C       ----------------------------------------------------------------
c                       these are not as crucial if they get set wrong
	integer         i_CHIPID1
	parameter       (i_CHIPID1  = i_CHIPID + 0)
	integer         i_PIXXSIZ0
	parameter       (i_PIXXSIZ0 = i_PIXSIZ + 0)
	integer         i_PIXYSIZ0
	parameter       (i_PIXYSIZ0 = i_PIXSIZ + 1)
	integer         i_NPIXX0
	parameter       (i_NPIXX0   = i_NPIX   + 0)
	integer         i_NPIXY0
	parameter       (i_NPIXY0   = i_NPIX   + 1)
C       ----------------------------------------------------------------
c                       lower limit of array of menu entries
	integer         MINDDATA
	parameter       (MINDDATA = i_DETECTOR)
c                       upper limit of array of menu entries
	integer         MAXDDATA
	parameter       (MAXDDATA = i_BSPOT  + (dY-cID+1)*MAXBADSPOT)
c                       bogus pointer value
	integer         NULDDATA
	parameter       (NULDDATA = MINDDATA - 1)
C       ================================================================
c                       equivalence to all of the above
	real            ddatar(MINDDATA:MAXDDATA)
	integer         ddatai(MINDDATA:MAXDDATA)
	logical         ddatab(MINDDATA:MAXDDATA)
	equivalence     (ddatar, ddatai, ddatab)
c                       some items come with accelerator keys
c                       that allow their values to be changed.
	character*1     ddataa(MINDDATA:MAXDDATA)
c                       pointers telling which ones to actually display
c                       and in what order to display them
c                       if the pointer value is NULDDATA, it is skipped
	integer         ddatap(MINDDATA:MAXDDATA)
c                       format to use to display each value
	character       ddataf(MINDDATA:MAXDDATA)*(LENFMT)
c                       character strings to use to display label strings
	character       ddatal(MINDDATA:MAXDDATA)*(LLABSTR)
c                       character strings to use to display value strings
	character   ddatav(SVOLD:SVNEW,MINDDATA:MAXDDATA)*(MAXFITSTR)
c                       positions of the labels in Normalized Device Coords
	real            ddatax(Xp:Yp,MINDDATA:MAXDDATA,2)
C       ----------------------------------------------------------------
	equivalence     (detector, ddatav(SVNEW,i_DETECTOR))
	equivalence     (nchips  , ddatai(i_NCHIPS  ))
	equivalence     (chipid  , ddatav(SVNEW,i_CHIPID  ))
	equivalence     (npix    , ddatai(i_NPIX    ))
	equivalence     (pixsiz  , ddatar(i_PIXSIZ  ))
	equivalence     (nbspot  , ddatai(i_NBSPOT  ))
	equivalence     (bspot   , ddatai(i_BSPOT   ))
	equivalence     (stdvidv , ddatab(i_STDVIDV ))
	equivalence     (stdvidh , ddatab(i_STDVIDH ))
	equivalence     (stdvidr , ddatab(i_STDVIDR ))
C       ----------------------------------------------------------------
c                       is this menu currently displayed?
	logical         ddataup
c                       is this menu currently initialized?
	logical         dmeninit
c                       NDC y limits of this menu
	real            dminndc
	real            dmaxndc
c                       bottom entry to be displayed
	integer         botddata
C-----------------------------------------------------------------------
	common  /detectcom/
     &  ddatar, ddatap, dminndc, dmaxndc, ddatax,
     &  ddataup, dmeninit, botddata,
     &  tmc, detpos, phprepix
c       characters are stored in a separate common, per ANSI requirements
	common  /detmostr/      ddataf, ddatal, ddatav, ddataa
