#include "include/port.h"
C==============================================================================
C==============================================================================
	program echellemain
c       (from dan schroeder, april 85)
c       (revised for vms vax by c. pilachowski, april 85)
c       (prism cross dispersion added sept. 85)
c       (original Lick Mongo implementation by A.P. Hatzes)
c       (further work by S.L. Allen for Keck HIRES)
c
c       this program gives the format of an echellogram with a
c       grating or prism cross dispersion over a user-specified
c       range of wavelengths.  input required included telescope
c       and collimator diameters, camera focal length, echelle
c       grating parameters, wavelength range, cross-disperser
c       type, cross disperser grating order and number of
c       grooves/mm or prism glass type, angle, and number of prisms.
c
c       grating parameters can be iterated until a final solution,
c       which is written out to a file called format.out.
c
c       a subroutine of glass types and parameters for the index of
c       refraction as in the Schott glass catalog is required.
c       the subroutine is the file glass.for, which should be
c       linked with this program.
	IMPLICIT_NONE
C       Parameters      ------------------------------------------------
#       include "echelle.par"
#       include "include/stdio.par"
#       include "include/units.par"
c                       Lick Mongo LVISIBILITY mode
	integer         INVERT
	parameter       (INVERT = 2)
C       Local Variables ------------------------------------------------
c                       loop
	integer         i
c                       a file containing detector or spectrograph description
	character       setfilnm*(MXPATH)
c                       prompt information for the user
	character*72    prompts(7)
C       External Functions----------------------------------------------
	integer         keymouse
	integer         comloop
	integer         readmen
	integer         readspc, readdet, readset
	integer         writspc, writdet, writset, wkics
	integer         PMGO(lenc)
C       Common Blocks   ------------------------------------------------
	include 'wizard.cmn'
	include 'spgraf.inc'
	include 'setup.inc'
	include 'schott.cmn'
	include 'echord.cmn'
	include 'cdata.inc'
	include 'lims.cmn'
	include 'linpos.cmn'
	include 'lam.cmn'
	include 'detmos.inc'
C       Data initialization---------------------------------------------
	data    nlam    /0/
	data    wizard  /.false./
	data    (prompts(i),i=1,7) /
     1  ' some Lick Mongo term types are:',
     2  '     0:  Don''t do screen graphics',
     3  '     1:  DEI Retrographics VT-100',
     4  '     7:  Workstation',
     5  '    11:  X11 Window',
     6  '    13:  GraphOn-2xx',
     7  '    14:  GraphOn-140' /
C       Executable Code=================================================
c       Fortran sucks

c       Begin a long segment of interactive I/O

c       Get device for graphical output
	call getenv('DISPLAY',setfilnm)
	if (PMGO(lenc)(setfilnm) .gt. 0) then
c           assume user wants X11 display if DISPLAY is set
	    idev = 11
	else
	    write(STDOUT,ALINE) prompts
92          write(STDOUT,PROMPT) ' Enter Lick Mongo terminal type:  '
	    read(STDIN,*,err=92) idev
	endif
c       jump here for complete restart of program
1       continue
	defdir(1) = DWD
	defdir(2) = DSETUPDIR
	call canondir(ENVARDIR,defdir(2))
	fixspace = .false.
	detfilnm = ' '
	spcfilnm = ' '
	observer = ' '
c
	write(STDOUT,*)
	write(STDOUT,*) 'Which Setup?'
	i = readmen(NDEFDIR,defdir,'set',setfilnm)
	i = readset(setfilnm,NDEFDIR,defdir)
	i = writset(SETBAKF)
c
	if (detfilnm .eq. ' ') then
c           we did not get a file while in readset
	    write(STDOUT,*)
	    write(STDOUT,*) 'Which Detector?'
	    i = readmen(NDEFDIR,defdir,'det',detfilnm)
	    i = readdet(detfilnm)
	endif
	i = writdet(DETBAKF)
c
	if (spcfilnm .eq. ' ') then
c           we did not get a file while in readset
	    write(STDOUT,*)
	    write(STDOUT,*) 'Which Spectrograph?'
	    i = readmen(NDEFDIR,defdir,'spc',spcfilnm)
	    i = readspc(spcfilnm)
	endif
	i = writspc(SPCBAKF)
	fulform = .true.
	call mosize()
	colldiam = (mmPERm/asecPERrad) * (colfocln*primdiam*focscale)
	if (nxdprism .gt. 0) then
	    call glassname(prglas(0))
	endif
	if (nxdgrat .gt. 0) then
	    mci(1) = mc - 1
	    mci(2) = mc + 2
	else
	    mc = 0
	    mci(1) = 0
	    mci(2) = 0
	endif
	call getwave()
	boxfl = .true.
	fulform = .true.
c       jump here to replot existing configuration
100     continue
C       ----------------------------------------------------------------
	call calcform(fixspace)
	tilt = .true.
c       we write out the setup regularly
	i = writset(SETBAKF)
C       ----------------------------------------------------------------
C       Plot the Spectrograph format on the graphics device
	if (idev .ne. 0) then
	    if (fulform) call getclim
c           fulform = .true.
	    if (boxfl) then
		call initscrn
c               write the names of the numeric and text at edge of the picture
		call upbutton
		call upcdata
		call rasetup(0)
		call upsetup
		call udsetup
	    endif
c           plot all the orders
	    call plotmord(0,mci)
c           plot detector arrays at the permanently stored locations
	    do 585 i = 1,nmos
	      call mosaic(i,INVERT)
	      call badcol(i,INVERT)
585         continue
c           plot locations of spectral lines on the Echelle format
	    if (nlam .gt. 0) call marklines
c           plot moveable detector array at the initial location
	    call spc2det
	    call mosaic(0,INVERT)
	    call badcol(0,INVERT)
	    call drowins(INVERT)
	endif
	i = writset(SETBAKF)
	i = writdet(DETBAKF)
	i = writspc(SPCBAKF)
1000    continue
	if (idev .ne. 0) then
c           enter the event processing loop looking for keyboard & mouse input
	    if (keymouse() .eq. -1) goto 100
	endif
C       ----------------------------------------------------------------
	i = comloop()
	if (i .eq. 1) then
	    goto 1
	elseif (i .eq. 100) then
	    goto 100
	elseif (i .eq. 1000) then
	    goto 1000
	endif
C       ----------------------------------------------------------------
c       always write out the final setup before exit
	i = writset(SETBAKF)
	i = wkics(' ')
	end
C==============================================================================
C==============================================================================
