#include "include/port.h"
C==============================================================================
C==============================================================================
	subroutine initscrn
c       initialize Lick Mongo for plotting on the currently selected device
	IMPLICIT_NONE
#       include "echelle.par"
C       Internal Variable
c                       dummy
	integer         i
c                       fractional sizes of Lick Mongo subwindows
	real            fracs(3)
C       External Function
	integer         PTUV(regmot)
	external        xtrackcur
C       Common Block
	include 'lims.cmn'
C       Static Variable
	save            fracs
C       Data
c       data    fracs   /1., 10., 2./
	data    fracs   /2., 10., 0./
C       Executable Code
	call device(idev)
	call tsetup
c       --------
	if (idev .eq. 7 .or. idev .eq. 11) then
c           make the biggest window allowed on the workstation
	    call setphysical(0.,0.,10000.,10000.)
c           wait a little for the window to be mapped
	    call sleep(2)
	    call erase
	    call getphysical(pxmin,pymin,pxmax,pymax)
	    pxmax = pxmax * .9
	    pymax = pymax * .9
	    call setphysical(pxmin,pymin,pxmax,pymax)
	endif
	call erase
	call getphysical(pxmin,pymin,pxmax,pymax)
c       --------
	call submargins(0.,0.)
	call winfrac(fracs,3,1)
	call window(3,1,1)
c       if (idev .eq. 11) call abox(5,5,5,5)
c       call window(3,1,3)
c       if (idev .eq. 11) call abox(5,5,5,5)
	call window(3,1,2)
c       --------
	call setlim(xmin,ymin,xmax,ymax)
	call rect(-1,-1,-1,-1)
c       --------
	if (idev .eq. 11) then
	    i = PTUV(regmot)(1,xtrackcur)
	endif
c       --------
	call makecolor(2,.6,.6,.6)
	call makecolor(3,1.,1.,0.)
	call makecolor(4,0.,1.,1.)
	call makecolor(5,1.,0.,0.)
	return
	end
C==============================================================================
C==============================================================================
	subroutine plotmord(ivis,cdord)
c       Plot all of the echelle orders for each order of cross dispersion
	IMPLICIT_NONE
C       Parameter
#       include "echelle.par"
C       External Variables
c                       which Lick Mongo SETLVIS mode should be used?
	integer         ivis
c                       the interesting range of cross disperser orders
	integer         cdord(2)
C       Internal Variables
c                       loop over all desired cross disperser orders
	integer         j
C       Common Blocks
	include 'lims.cmn'
	include 'echord.cmn'
	include 'setup.inc'
C       Executable Code
c       define Lick Mongo color 2 to be gray
	call setcolor(2)
c       Step through the entire range of cross-dispersion orders
	do 20 j=cdord(1),cdord(2)
c         we do not do the desired cross-dispersion order here
	  if (j .ne. mc) then
	      call plotord(ivis,j,.false.)
	  endif
20      continue
c       Finally, plot the echelle orders for the desired cross-disp. order
	if (idev .eq. 7 .or. idev .eq. 11) then
c           on a workstation device, plot the wings of the orders
	    call plotord(ivis,mc,.true.)
	endif
	call setcolor(1)
	call plotord(ivis,mc,.false.)
	return
	end
C==============================================================================
C==============================================================================
	subroutine plotord(ivis,mord,wing)
c       Plot all of the echelle orders from a single order of the
c       cross disperser.
c       Optionally decorate it with wavelength and/or order # information
	IMPLICIT_NONE
C       Parameter
#       include "echelle.par"
c                       length in characters of an order label
	integer         LOL
	parameter       (LOL = 7)
c                       symbolic names for bluer and redder ends of an order
	integer         BLU, RED
	parameter       (BLU = 1, RED = 2)
c                       how many?
	integer         NRB
	parameter       (NRB = RED - BLU + 1)
C       External Variables
c                       which Lick Mongo SETLVIS mode should be used?
	integer         ivis
c                       which order of cross dispersion to plot?
	integer         mord
c                       If wing is false, only the FSR of each Echelle order
c                       is plotted.
c                       If wing is true, the next FSR-worth of of each
c                       Echelle order outside the FSR is plotted.
	logical         wing
C       Internal Variables
	real            xord(BLU:RED)
	real            yord(BLU:RED)
c                       string used for labelling wavelengths and order numbers
	character       corder*(LOL)
	integer         iord, iop1, iom1, i
C       Common Blocks
	include 'setup.inc'
	include 'echord.cmn'
	include 'lims.cmn'
C       Executable Code
c       All of the real variables used in this routine have the dimensions of
c       millimeters [mm].
	call setlvis(ivis)
c       keep in mind here that x(1) is the bluest echelle order
c       and x(norders) is the reddest echelle order.  This ordering
c       thus runs backwards from the actual echelle order numbers.
	do 10 iord = 1,norders
c         calculate iop1 and iom1 which cannot exceed array bounds
	  iop1 = iord + 1
	  iom1 = iord - 1
c         Calculate position of left and right ends of this order (xord).
c         The xord calculations may be too much of an approximation,
c         but the cursor position finding algorithm uses this same approx.
c         Calculate height of the ends of the FSR of this order (yord).
	  if (wing) then
c             wing implies tilt, this is evident in the yord calculation
c             first, calculate the bluer wing
	      xord(BLU) = -f2db(iord)
	      xord(RED) = -f2db(iord) * 0.5
	      yord(BLU) = x(iom1,mord)
	      yord(RED) = 0.5 * (x(iord,mord) + x(iom1,mord))
c             draw the bluer wing
	      call translate(yord, xord, tde(Xp,Xo,E2D), NRB)
	      call connect(yord,xord,NRB)
c             now, calculate for the redder wing
	      xord(BLU) =  f2db(iord)*0.5
	      xord(RED) =  f2db(iord)
	      yord(BLU) = 0.5 * (x(iord,mord) + x(iop1,mord))
	      yord(RED) = x(iop1,mord)
	  else
	      xord(BLU) = -f2db(iord) * 0.5
	      xord(RED) =  f2db(iord) * 0.5
	      if (tilt) then
		  yord(BLU) = 0.5 * (x(iord,mord) + x(iom1,mord))
		  yord(RED) = 0.5 * (x(iord,mord) + x(iop1,mord))
	      else
		  yord(BLU) = x(iord,mord)
		  yord(RED) = x(iord,mord)
	      endif
	  endif
c         draw this order
	  call translate(yord, xord, tde(Xp,Xo,E2D), NRB)
	  call connect(yord,xord,NRB)
c         find order # 120
	  if (morder(iord) .eq. 120) i120 = iord
c         label the edges of the orders if desired
	  i = mod(morder(iord),5)
	  if (wing) then
c             do not label the wing
	  elseif (ordnum .and. i .eq. 0) then
c             order number on the left side of the order
	      write(corder(1:5),'(a1,i3,a1)') ' ',morder(iord),' '
	      call relocate(yord(BLU),xord(BLU))
	      call putlabel(5,corder,4)
	      if (.not. lamnum) then
c                 order number on the right side of the order
		  call relocate(yord(RED),xord(RED))
		  call putlabel(5,corder,6)
	      endif
	  elseif (lamnum .and. i .eq. 1) then
c             wavelength on the right side of the order
	      write(corder(:LOL),'(f7.1)') wv(iord) + fsr(iord) * 0.5
	      call relocate(yord(RED),xord(RED))
	      call putlabel(LOL,corder,6)
	  elseif (lamnum .and. i .eq. 0) then
c             wavelength on the left side of the order
	      write(corder(:LOL),'(f7.1)') wv(iord) - fsr(iord) * 0.5
	      call relocate(yord(BLU),xord(BLU))
	      call putlabel(LOL,corder,4)
	  endif
10      continue
	call setlvis(0)
	return
	end
C==============================================================================
C==============================================================================
	subroutine getclim
c       this subroutine computes reasonable values for world coordinate limits
	IMPLICIT_NONE
C       Parameter
#       include "echelle.par"
#       include "include/units.par"
C       Internal Variables
c                       height between reddest and bluest order [mm]
	real            xb
c                       position of a "corner", or size of Echelle format [mm]
	real            xcs, ycs
c                       position of middle of the Echelle format [mm]
	real            xmid, ymid
C       Common Blocks
	include 'echord.cmn'
	include 'lims.cmn'
	include 'spgraf.inc'
	include 'detmos.inc'
	include 'setup.inc'
C       Executable Code
c       figure the size of the graphics region leaving a little margin
c       --------    red end of reddest order    ----------------
	ycs = f2db(norders)
	xcs = x(norders,mc)
	call translate(xcs, ycs, tde(Xp,Xo,E2D), 1)
	ymax = ycs
	ymin = ycs
	xmax = xcs
	xmin = xcs
c       --------   blue end of reddest order    ----------------
	ycs = -f2db(norders)
	xcs = x(norders,mc)
	call translate(xcs, ycs, tde(Xp,Xo,E2D), 1)
	ymax = max(ymax, ycs)
	ymin = min(ymin, ycs)
	xmax = max(xmax, xcs)
	xmin = min(xmin, xcs)
c       --------    red end of bluest  order    ----------------
	ycs = f2db(1)
	xcs = x(1,mc)
	call translate(xcs, ycs, tde(Xp,Xo,E2D), 1)
	ymax = max(ymax, ycs)
	ymin = min(ymin, ycs)
	xmax = max(xmax, xcs)
	xmin = min(xmin, xcs)
c       --------   blue end of bluest  order    ----------------
	ycs = -f2db(1)
	xcs = x(1,mc)
	call translate(xcs, ycs, tde(Xp,Xo,E2D), 1)
	ymax = max(ymax, ycs)
	ymin = min(ymin, ycs)
	xmax = max(xmax, xcs)
	xmin = min(xmin, xcs)
c       --------   find middle of Echelle format   -------------
c       write(*,*) 'maxes1', xmin, xmax, ymin, ymax
	xmid = (xmax + xmin) * 0.5
	ymid = (ymax + ymin) * 0.5
c       --------   leave a little room around the edges --------
	xcs = max(xmax - xmin,cormos(Xp,2)-cormos(Xp,1)) * PMARGIN * 0.5
	ycs = max(ymax - ymin,cormos(Yp,2)-cormos(Yp,1)) * PMARGIN * 0.5
	xmax = xmid + xcs
	xmin = xmid - xcs
	ymax = ymid + ycs
	ymin = ymid - ycs
c       --------    swap things as a temporary hack  -----------
c       write(*,*) 'maxes2', xmin, xmax, ymin, ymax
c       xb = xmin
c       xmin = ymin
c       ymin = xb
c       xb = xmax
c       xmax = ymax
c       ymax = xb
c       --------------------------------------------------------
c       If the observing site uses non-standard video orientation,
c       exchange the Lick Mongo coordinates to flip the entire picture.
	if (.not. stdvidv) then
	    xb = ymin
	    ymin = ymax
	    ymax = xb
	endif
	if (.not. stdvidh) then
	    xb = xmin
	    xmin = xmax
	    xmax = xb
	endif
	return
	end
C==============================================================================
C==============================================================================
	subroutine drowins(ivis)
c       plot outlines of all of the readout windows in the mosaic
	IMPLICIT_NONE
#       include "echelle.par"
C       External Variables
c                       Lick Mongo line-visibility
	integer         ivis
C       Local Variables
c                       loop
	integer         i
C       Common Block
	include 'detmos.inc'
C       Executable Code
	call setcolor(3)
	do 10 i = 1,nchips
	  call drowin(i,ivis)
10      continue
	call setcolor(1)
	return
	end
C==============================================================================
C==============================================================================
	subroutine drowin(ndet,ivis)
c       draw the outline of a readout window on the display
	IMPLICIT_NONE
C       Parameters
c                       how many points?
	integer         HMPT
	parameter       (HMPT = 5)
#       include "echelle.par"
#       include "include/units.par"
C       External Variables
c                       which detector in the mosaic
	integer         ndet
c                       Lick Mongo line-visibility
	integer         ivis
C       Local Variables
c                       positions of corners of detectors on format [mm]
	real            xy(HMPT,Xp:Yp)
C       Common Block
c       include 'detmos.inc'
	include 'setup.inc'
C       Executable Code
	call pixpos(0,ndet,real(window(Xp,ndet)),
     &  real(window(Yp,ndet)),              xy(1,Xp),xy(1,Yp))
	call pixpos(0,ndet,window(Xp,ndet)+window(dX,ndet)+1.,
     &  real(window(Yp,ndet)),              xy(2,Xp),xy(2,Yp))
	call pixpos(0,ndet,window(Xp,ndet)+window(dX,ndet)+1.,
     &  window(Yp,ndet)+window(dY,ndet)+1., xy(3,Xp),xy(3,Yp))
	call pixpos(0,ndet,real(window(Xp,ndet)),
     &  window(Yp,ndet)+window(dY,ndet)+1., xy(4,Xp),xy(4,Yp))
	xy(5,Xp) = xy(1,Xp)
	xy(5,Yp) = xy(1,Yp)
c       write(*,*) xy(1,Xp), xy(1,Yp), xy(3,Xp), xy(3,Yp)
	call setlvis(ivis)
	call setltype(1)
c       call translate(xy(1,Xp), xy(1,Yp), tde(Xp,Xo,E2D), HMPT)
	call connect(xy(1,Xp), xy(1,Yp), HMPT)
	call setltype(0)
	call setlvis(0)
	return
	end
C==============================================================================
C==============================================================================
	subroutine mosaic(mos,ivis)
c       plot outlines of all of the detectors in the mosaic
	IMPLICIT_NONE
#       include "echelle.par"
C       External Variables
c                       which mosaic position
	integer         mos
c                       Lick Mongo line-visibility
	integer         ivis
C       Local Variables
c                       loop
	integer         i
C       Common Block
	include 'detmos.inc'
C       Executable Code
	call setcolor(4)
	do 10 i = 1,nchips
	  call ccdbox(mos,i,ivis)
10      continue
	call setcolor(1)
	return
	end
C==============================================================================
C==============================================================================
	subroutine ccdbox(mos,ndet,ivis)
c       draw the outline of a detector on the display
	IMPLICIT_NONE
C       Parameters
c                       how many points?
	integer         HMPT
	parameter       (HMPT = 5)
#       include "echelle.par"
#       include "include/units.par"
C       External Variables
c                       which mosaic position
	integer         mos
c                       which detector in the mosaic
	integer         ndet
c                       Lick Mongo line-visibility
	integer         ivis
C       Local Variables
c                       positions of corners of detectors on format [mm]
	real            xy(HMPT,Xp:Yp)
C       Common Block
	include 'detmos.inc'
	include 'setup.inc'
C       Executable Code
	call pixpos(mos,ndet,0.               ,0.     ,
     &  xy(1,Xp),xy(1,Yp))
	call pixpos(mos,ndet,npix(ndet,Xp)+1.,0.     ,
     &  xy(2,Xp),xy(2,Yp))
	call pixpos(mos,ndet,npix(ndet,Xp)+1.,npix(ndet,Yp)+1.,
     &  xy(3,Xp),xy(3,Yp))
	call pixpos(mos,ndet,0.               ,npix(ndet,Yp)+1.,
     &  xy(4,Xp),xy(4,Yp))
	xy(5,Xp) = xy(1,Xp)
	xy(5,Yp) = xy(1,Yp)
c       write(*,*) xy(1,Xp), xy(1,Yp), xy(3,Xp), xy(3,Yp)
	call setlvis(ivis)
c       call translate(xy(1,Xp), xy(1,Yp), tde(Xp,Xo,E2D), HMPT)
	call connect(xy(1,Xp), xy(1,Yp), HMPT)
	call setlvis(0)
	return
	end
C==============================================================================
C==============================================================================
	subroutine badcol(mos,ivis)
c       draw all the bad spots of a given detector mosaic
	IMPLICIT_NONE
#       include "echelle.par"
#       include "include/units.par"
#       include "include/stdio.par"
C       Parameter
c                       how many points?
	integer         HMPT
	parameter       (HMPT = 2)
c                       we draw using the default foreground color
	integer         FGCLR
	parameter       (FGCLR = 1)
C       External Variables
c                       which mosaic position
	integer         mos
c                       Lick Mongo line-visibility
	integer         ivis
C       Local Variables
	integer         i, j
c                       positions of ends of bad columns on format [mm]
	real            xy(HMPT,Xp:Yp)
C       Common Block
	include 'detmos.inc'
	include 'setup.inc'
C       Executable Code
	call setlvis(ivis)
c       note that the binning factor of the chip is considered when drawing
	do 10 i = 1,nbspot
c         which chip does this bad spot live on
	  j = bspot(cID,i)
c         position of badspot corner nearest to readout amp
	  call pixpos(mos, j,
     &    real( (bspot(Xp,i)/xbin(j))*xbin(j) ),
     &    real( (bspot(Yp,i)/ybin(j))*ybin(j) ),
     &    xy(1,Xp), xy(1,Yp))
c         position of badspot corner farthest from readout amp
	  call pixpos(mos, j,
     &    real(((bspot(Xp,i)+bspot(dX,i) + xbin(j)-1)/xbin(j))*xbin(j)),
     &    real(((bspot(Yp,i)+bspot(dY,i) + ybin(j)-1)/ybin(j))*ybin(j)),
     &    xy(2,Xp), xy(2,Yp))
c         draw the sucker
c         call translate(xy(1,Xp), xy(1,Yp), tde(Xp,Xo,E2D), HMPT)
	  call mrectang(xy(1,Xp), xy(1,Yp), xy(2,Xp), xy(2,Yp), 5)
10      continue
	call setlvis(0)
	return
	end
C==============================================================================
C==============================================================================
	subroutine marklines
c       For every wavelength in the array lambda, plot a dot on the
c       Echelle format which marks that spectral line.
	IMPLICIT_NONE
C       Parameters
#       include "echelle.par"
C       Local Variables
C       Common Blocks
	include 'linpos.cmn'
	include 'lam.cmn'
	include 'cdata.inc'
c       include 'setup.inc'
C       Executable Code
c       calculate the position of each line on the echelle format
	call calclines
c       plot the lines which are inside the FSR
	call points(83.,1,yline(MINLIN,1),xline(MINLIN,1),nlam)
c       plot the lines which are outside the FSR
	call points(80.,1,yline(MINLIN,2),xline(MINLIN,2),nlam)
c       plot the last point where the cursor was known to be
	call points(41.,1,xclast,yclast,1)
	return
	end
C==============================================================================
C==============================================================================
	subroutine calclines
c       Calculate the position on the echelle format for every line in array
c       lambda.
	IMPLICIT_NONE
C       Parameters
#       include "echelle.par"
C       Local Variables
c                       loop
	integer         i, j
c                       the redshifted wavelength
	real            redlam
c                       temporary location to store position
	real            xyl(2,Xp:Yp)
C       Common Blocks
	include 'linpos.cmn'
	include 'lam.cmn'
	include 'setup.inc'
C       Executable Code
	do 50 i = MINLIN,MINLIN-1+nlam
	  redlam = lambda(i)
c         if this is not a terrestrial line, redshift it
	  if (.not. earth(i)) redlam = redlam * (1 + redshift)
c         Take this wavelength and find out where on the Echelle format
c         it would fall.  Return with the position and index of order number.
	  call wav2pos(redlam, xyl, ilval(1,i))
	  do 40 j=1,2
	    xline(i,j) = xyl(j,Xp)
	    yline(i,j) = xyl(j,Yp)
40        continue
50      continue
c       Now must convert from Echelle format coords to Lick Mongo world coords
c       positions of lines which are inside the FSR
	call translate
     &  (yline(MINLIN,1), xline(MINLIN,1), tde(Xp,Xo,E2D), nlam)
c       positions of lines which are outside the FSR
	call translate
     &  (yline(MINLIN,2), xline(MINLIN,2), tde(Xp,Xo,E2D), nlam)
	return
	end
C==============================================================================
C==============================================================================
	subroutine linepos(outfile)
c       For every wavelength in the array lambda, find that position of
c       that line on the detector and write it to an output file.
	IMPLICIT_NONE
C       Parameters
#       include "echelle.par"
#       include "include/stdio.par"
C       External Variable
c                       name of the file to write output
	character*(*)   outfile
C       Local Variables
c                       loop
	integer         i, j
c                       I/O status
	integer         ios
c                       index of detector in which line falls
	integer         idet
c                       pixel coordinates of line
	real            xpix, ypix
C       Common Blocks
	include 'linpos.cmn'
	include 'lam.cmn'
	include 'echord.cmn'
	include 'setup.inc'
C       Executable Code
c       see if we can open the output file
	open(EFOLUN,file=outfile,status='unknown',iostat=ios)
	if (ios .ne. SUCCESS) then
	    write(STDERR,*)
     &      'linepos: could not open output line position file'
	    write(STDERR,*) 'name of file was ',outfile
	    return
	endif
c       paranoia of an experienced Fortran programmer
	rewind(EFOLUN)
c       write all the "header" info to the file
	write(EFOLUN,*)
     &  '# Echelle simulator: approximate positions of marked lines'
	write(EFOLUN,*) 'DetFilNm: ', detfilnm
	write(EFOLUN,*) 'SpcFilNm: ', spcfilnm
	write(EFOLUN,*) 'WaveFile: ', wavefile
	write(EFOLUN,*) 'ECHANGL: ' , ecangle
	write(EFOLUN,*) 'XDANGL: '  , xdangle
	write(EFOLUN,*)
     &  '# lambda[A] x[pixel] y[pixel] order detID telluric?'
	write(EFOLUN,*) 'DataBegin:'
c       calculate the position of each line on the echelle format
	call calclines
c       loop over every defined spectral line
	do 500 i = MINLIN,MINLIN-1+nlam
c         loop over the nearest to blaze and next nearest to blaze
	  do 100 j =1,2
c           convert from Lick Mongo world coords to detector coords
	    call detcoord(yline(i,j),xline(i,j),idet,xpix,ypix)
	    if (idet .ne. -1) then
		write(EFOLUN,'(f9.2,2f9.1,i5,i4,3x,l1)')
     &          lambda(i),xpix,ypix,morder(ilval(j,i)),idet,earth(i)
	    endif
100       continue
500     continue
	close(EFOLUN)
	return
	end
C==============================================================================
C==============================================================================
