#include "include/port.h"
C==============================================================================
C==============================================================================
	subroutine cursdata(xcurs,ycurs)
c       Given a position on the Echelle format, calculate all the
c       interesting facts about that location.
	IMPLICIT_NONE
C       Parameter
#       include "echelle.par"
#       include "include/units.par"
C       External Variables
c                       Focal Plane coordinates of the cursor [mm]
	real            xcurs, ycurs
C       Local Variables
	integer         nord, n1, n2, npoint
	real            y1, y2, dy1, dy2
C       Common Blocks   -------------------------------------------------
	include 'echord.cmn'
	include 'cdata.inc'
	include 'lims.cmn'
	include 'spgraf.inc'
	include 'detmos.inc'
	include 'setup.inc'
C       Executable Code
	if (ycurs .le. (0.5*(x(0,mc) + x(1,mc))) ) then
c           cursor below all the orders
	    nord = 1
	elseif (ycurs .ge. (0.5*(x(norders,mc)+x(norders+1,mc))) ) then
c           cursor above all the orders
	    nord = norders
	else
c           start a binary search to find out what order cursor was on
	    n1 = 1
	    n2 = norders
50          continue
	    if (n2 .eq. n1) then
c               we have found the correct order
		nord = n1
	    elseif ( abs(n2-n1) .eq. 1) then
c               we are within 1 of the correct order
		y1 = x(n1,mc)
		if (tilt) y1 = y1 + delx(n1)*xcurs/f2db(n1)
		y2 = x(n2,mc)
		if (tilt) y2 = y2 + delx(n2)*xcurs/f2db(n2)
		dy1 = abs(ycurs - y1)
		dy2 = abs(ycurs - y2)
		if (dy2 .lt. dy1) then
		    nord = n2
		else
		    nord = n1
		endif
	    else
c               binary search
		npoint = (n2 + n1)/2
		y1 = x(npoint,mc)
		if (tilt)
     &          y1 = y1 + delx(npoint)*xcurs/f2db(npoint)
		if (ycurs .gt. y1) then
		    n1 = npoint
		elseif (ycurs .lt. y1) then
		    n2 = npoint
		elseif (.not. tilt .and. ycurs .eq. x(npoint,mc)) then
		    nord = npoint
		    goto 100
		endif
		goto 50
	    endif
	endif
c       The search is done, we know which order the cursor was near
100     continue
	corder = morder(nord)
	clambdab = wv(nord)
	dispamm = fsr(nord) / f2db(nord)
	dispapx = dispamm * pixsiz(ONLY1,Xp) * mmPERm
	clambda = clambdab + xcurs*dispamm
	csepm = delx(nord)
	csepp = delx(nord) * mPERmm / pixsiz(ONLY1,Xp)
	cseps = phi(nord)
	clen = f2db(nord)
	cloc = x(nord,mc)
	cdkpix = dckpix(nord)
c       new calculation for getting the cross disperser wavelength
	y1 = x(nord,mc)
	if (tilt) y1 = y1 + delx(nord)*xcurs/f2db(nord)
	dy1 = (ycurs - y1) / delx(nord)
	if (dy1 .gt. 0.) then
	    cxlambda = clambda + (wv(nord+1)-wv(nord))*dy1
	else
	    cxlambda = clambda + (wv(nord)-wv(nord-1))*dy1
	endif
c       write(STDERR,*) 'xcurs,ycurs,cxlambda',xcurs,ycurs,cxlambda
	return
	end
C==============================================================================
C==============================================================================
	subroutine detcoord(xc,yc,idet,dpx,dpy)
c       Find out which detector a point is inside, if any, and where
c       Most often the point of interest will be the cursor, but it
c       may be a particular spectral line.
	IMPLICIT_NONE
C       Parameters
#       include "echelle.par"
#       include "include/units.par"
C       External Variables
c                       input:  Focal Plane coordinates of a point [mm]
	real            xc, yc
c                       output: index of the detector containing the point
c                       If not inside any detector, returns -1
	integer         idet
c                       output: pixel coordinates of the point, if inside
	real            dpx, dpy
C       Internal Variables
	real            cx, cy
C       Common Blocks
	include 'setup.inc'
	include 'detmos.inc'
c       include 'cdata.inc'
C       Executable Code
c       Convert to Detector Mosaic coordinates
	cx = (xc - posmos(Xp,0)) * mPERmm
	cy = (yc - posmos(Yp,0)) * mPERmm
c       search all chips for insidedness
	do 1000 idet=1,nchips
	  dpx = tmc(Xp,Xo,M2C,idet)*cx + tmc(Xp,Yo,M2C,idet)*cy +
     &    tmc(Xp,To,M2C,idet)
	  dpy = tmc(Yp,Xo,M2C,idet)*cx + tmc(Yp,Yo,M2C,idet)*cy +
     &    tmc(Yp,To,M2C,idet)
c         if we are inside this chip, jump out with no further checking
	  if (  0 .le. dpx .and. dpx .le. (npix(idet,Xp)+1)
     &    .and. 0 .le. dpy .and. dpy .le. (npix(idet,Yp)+1) ) goto 1001
1000    continue
1001    continue
	if (idet .gt. nchips) idet = -1
	return
	end
C==============================================================================
C==============================================================================
	subroutine wav2pos(waveln,xyl,numord)
c       Given a wavelength waveln this routine finds the positions on the
c       plotted Echelle format at which that wavelength will be found.
c       Two positions are returned; these are in two adjacent Echelle orders.
c       The first position is for the Echelle order where waveln is
c       closest to the Echelle blaze wavelength; the second position is for
c       the next closest occurrance of waveln.
	IMPLICIT_NONE
C       Parameters
#       include "echelle.par"
C       External Variables
c                       wavelength of interest [Aangstrom]
	real            waveln
c                       positions of waveln on focal plane [mm]
	real            xyl(2,Xp:Yp)
c                       index of order number in which this line best appears
	integer         numord(2)
C       Local Variables
c                       current limits of binary search
	integer         n1, n2, npoint
c                       (waveln - (Echelle blaze wavelength)) [Aangstrom]
	real            dellam
C       Common Blocks
	include 'echord.cmn'
	include 'lims.cmn'
	include 'spgraf.inc'
	include 'setup.inc'
C       Executable Code
	if (waveln .ge. wv(1) .and. waveln .le. wv(norders))then
c           this line is inside the specified wavelength range
	    n1 = 1
	    n2 = norders
20          continue
	    npoint = (n2 + n1) / 2
	    if (n2 .eq. n1) then
		    npoint = n1
	    elseif ( abs(n2-n1) .eq. 1) then
c               we are within one of the correct order
		if ((waveln-wv(n1))/fsr(n1) .le. 0.5) then
		    npoint = n1
		else
		    npoint = n2
		endif
	    else
c               this is a binary search
		if (waveln .gt. wv(npoint)) then
		    n1 = npoint
		    goto 20
		elseif (waveln .lt. wv(npoint))then
		    n2 = npoint
		    goto 20
		else
c                   they are equal, fall thru the if blocks to the calc
		endif
	    endif
	    do 40 n1=1,2
	      numord(n1) = npoint
	      dellam = (waveln - wv(npoint)) / fsr(npoint)
	      xyl(n1,Xp) = dellam * f2db(npoint)
	      xyl(n1,Yp) = x(npoint,mc)
	      if (tilt) xyl(n1,Yp) = xyl(n1,Yp) + dellam * delx(npoint)
	      npoint = npoint + sign(1.,dellam)
40          continue
	else
c           this line is outside the specified wavelength range
	    do 140 n1=1,2
	      numord(n1) = 0
	      xyl(n1,Xp) = 1.e9
140         continue
	endif
	return
	end
C==============================================================================
C==============================================================================
	subroutine getwave()
c       Open and read a wavelength file (if any)
	IMPLICIT_NONE
C       Parameters
#       include "echelle.par"
#       include "include/stdio.par"
C       Internal Variables
c                       loop
	integer         k
c                       I/O status
	integer         ios
	character       fname*(MXPATH)
C       External function
c       integer         PMGO(lenc)
C       Common Block
	include 'lam.cmn'
	include 'lims.cmn'
	include 'setup.inc'
C       Executable Code
	if (wavefile .eq. ' ') return
	fname = wavefile
	call canonfile(NDEFDIR,defdir,fname)
c       if (PMGO(lenc)(fname) .gt. 0) then
	if (fname .ne. ' ') then
c           the filename is not null
	    open(unit=WAVLUN,file=fname,status='OLD',
     &      form='FORMATTED',iostat=ios)
	    if (ios .ne. 0) then
		write(STDERR,*) ' Error opening file ',fname
	    else
		k = MINLIN
		ios = SUCCESS
7               continue
		read(WAVLUN,*,iostat=ios) lambda(k), earth(k), wt(k)
		if (ios .eq. EOF) then
c                   normal termination, fall off ifblock
		elseif (ios .ne. SUCCESS) then
c                   some kind of error while reading file
		    write(STDERR,*) 'Error reading file ',fname
		    write(STDERR,*) 'Check format and protection.'
		elseif (ios .eq. SUCCESS) then
c                   successful read
		    k = k + 1
		    if (k .le. MAXLIN) then
c                       there is still room, go back for more
			goto 7
		    else
c                       full table of lines, report and fall through
			write(STDERR,*)
     &                  'Not enough room for all the lines in ',fname
			write(STDERR,*)
     &                  'Increase parameter NUMLIN for more.'
		    endif
		endif
		nlam = k - MINLIN
		close(WAVLUN)
	    endif
	else
c           the filename is null
	    write(STDERR,*) ' Cannot find a file called ',wavefile
	endif
	return
	end
C==============================================================================
C==============================================================================
	subroutine calcform(fixspace)
c       This is the subroutine which actually calculates the format of the
c       echelle spectrum on the detector.  The original author of this code
c       (Schroeder) did not include any references to published texts.
c       (Presumably because his own text was not yet published.)
c       References have been added, and they are to the text of the author, viz.
c       Astronomical Optics; Daniel J. Schroeder; Academic Press, Inc.; 1987
c       For prism dispersion, new references have been added to this text:
c       Optics; E. Hecht & A. Zajac; Addison-Wesley; 1974
	IMPLICIT_NONE
C       Parameters      ------------------------------------------------
#       include "echelle.par"
#       include "include/units.par"
#       include "include/stdio.par"
C       External Variables
c                       When a grating is being ruled, it is usually best if
c                       the spacing between the grooves is an integral multiple
c                       of the wavelength of a reference beam.
c                       A commonly used wavelength is that produced by
c                       a Helium-Neon laser.
c                       If this program is being used to design a spectrograph,
c                       the designer may wish to enforce such a spacing.
c                       fixspace determines whether such spacing is enforced.
	logical         fixspace
C       Internal Variables
c                       the blaze wavelength of an echelle order
c                       sometimes [um], sometimes [mm]
	real            bwav
c                       the echelle order number of the bluest (shortest) order
	integer         ms
c                       the echelle order number of the reddest (longest) order
	integer         ml
c                       loops
	integer         i, j, mi
c                       temporary variable
	real            temp
c               For Prism Cross Dispersion
c                       incident angle from surface normal for a prism [radian]
	real            thetain
c                       outbound angle from surface normal for a prism [radian]
	real            thetout
c                       {the index of refraction of the prism cross disperser}
c                       evaluated at blaze wavelength of one order
c                       [dimensionless]
	real            avn
c               For Grating Cross Dispersion
c                       sine of incident angle (alpha)
	real            sa
C       External Function
	real            glref
C       Common Blocks   ------------------------------------------------
	include 'schott.cmn'
	include 'echord.cmn'
	include 'spgraf.inc'
	include 'setup.inc'
C       Executable Code ------------------------------------------------
c       compute echelle blaze angle in radians
	ecdelta = ecdeltad * radPERdeg
c       compute theta (half angle) in radians
	ectheta = ecthetad * radPERdeg
	if (fixspace) then
C           Convert spacing to integral number of HeNeWAV wavelengths
	    ecsigma = AangPERmm / ecsigmai
	    ecsigma = (HeNeWAV * nint(ecsigma / HeNeWAV)) * mmPERAang
	    ecsigmai = umPERmm / ecsigma
	else
	    ecsigma = umPERmm / ecsigmai
	endif
	f2dbdl = camfocln*mmPERm / (ecsigma * cos(ecdelta - ectheta))
c       base is the echelle grating constant m*lambda [micrometers]
c       see Shroeder Table 15.3
	base = 2.*ecsigma*sin(ecdelta)*cos(ectheta)
c       this order is the next bluer order than the one containing ws
	ms = int((AangPERum * base / ws) + 0.5) + 1
c       this order is the next redder order than the one containing wl
	ml = int((AangPERum * base / wl) - 0.5)
	norders = (ms - ml) - 1
	if (norders .ge. MAXORD) then
	    write(STDERR,*)
     &      'There are too many echelle orders in the wavelength range.'
	    write(STDERR,*) norders,' >= ',MAXORD
	    write(STDERR,*) ' Increase parameter MAXORD and try again.'
	    stop
	endif
c       The collimator/echelle/cross disperser/camera combination reimages
c       the slit onto the detector and shrinks it in the process.
c       Temporarily, calculate magnification if there were no dispersers.
	demag = camfocln / colfocln
c       write(92,*) 'demag', demag
c       N.B.  This calculation assumes that theta is a constant.  Many
c       echelle gratings can be rotated to "tune" them to look at wavelengths
c       slightly off the blaze.  Such a rotation changes theta, and thus
c       changes the blaze wavelengths, FSRs, &c. of the echelle.
	i = -1
	do 222 mi = ms,ml,-1
	  i = i + 1
c         note that the order number (morder) counts DOWN as i counts UP
	  morder(i) = mi
c         blaze wavelength of echelle order morder(i) [micrometers]
	  bwav = base / morder(i)
c         blaze wavelength of echelle order morder(i) [Aangstroms]
	  wv(i) = AangPERum * bwav
c         the Free Spectral Range of order morder(i) [Aangstroms]
c         this is denoted "DELTA lambda" in Schroeder Table 15.3
	  fsr(i) = wv(i) / morder(i)
c         the length of the FSR of morder(i) on the detector [mm]
c         this is f2 * "DELTA beta" in Schroeder Table 15.3
	  f2db(i) = bwav * f2dbdl
c         initialize decker magnification for each order
	  deckmag(i) = demag
222     continue
C       ----------------------------------------------------------------
c       to get the format, compute the effects of the cross dispersers
c       compute the cross dispersion of any prisms
c       see H&Z eqs. 5.52 and 5.53
	if (nxdprism .gt. 0) then
	    do 511 i = 0,norders+1
	      bwav = wv(i) * umPERAang
	      avn = glref(iglass,bwav)
	      x(i,mc) = 0.
	      thetain = prangind * radPERdeg
	      do 500 j = 1,nxdprism
		temp = sin(thetain)
		thetout = asin(sin(prapexd(j)*radPERdeg) *
     &          sqrt(avn**2 - temp**2) -
     &          temp * cos(prapexd(j)*radPERdeg) )
		deckmag(i) = deckmag(i) * cos(thetain) / cos(thetout)
		thetain = prface(j+1)*radPERdeg - thetout
500           continue
	      x(i,mc) = -thetout
511         continue
	endif
C       ---------
c       compute the cross dispersion of any gratings
	if (nxdgrat .gt. 0) then
c           Compute the incident angle (alpha).
c           This uses the given value of xdangle as the tilt of the
c           cross-disperser off-blaze.
	    xdalphad = xdangle + xddeltad + xdalfbet*0.5
	    sa = sin(xdalphad * radPERdeg)
c           For blaze wavelength of each Echelle order, calculate the
c           outgoing angle (beta) from the cross dispersing grating.
c           See Schroeder Eq. 13.2.1, Fig. 13.2, and associated text.
	    do 275 j = mci(1), mci(2)
	      temp = j * xdsigmai * mmPERAang
	      do 225 i = 0,norders+1
c               this is the diffracted angle (beta) for this wavelength
		if (nxdprism .gt. 0) then
		    x(i,j) = x(i,mc) + asin(temp * wv(i) - sa)
		else
		    x(i,j) = asin(temp * wv(i) - sa)
		endif
		if (j .eq. mc) then
		    deckmag(i) = deckmag(i) *
     &              sqrt( (1. - sa*sa) / (1. - (temp * wv(i) - sa)**2))
		endif
225           continue
275         continue
c           for reference, save the outgoing angle of two orders
	    xdbetad(1) =asin(mc*xdsigmai*mmPERAang*wv(1) - sa)*degPERrad
	    xdbetad(2) =asin(mc*xdsigmai*mmPERAang*wv(norders) - sa) *
     &      degPERrad
c           and for display, save the outgoing central angle
	    xdbetad(3) = xdalphad - xdalfbet
	endif
C       ---------
c       The dispersers have an anamorphic effect which is different in
c       the two different directions of dispersion.
c       See Shroeder eqs. 12.2.1a, 13.1.1, 13.2.3 and the related text.
c       For purposes of variable naming in an Echelle spectrograph, the
c       two different magnifications are here called slitmag and deckmag.
c       These are in the Echelle and cross dispersing directions, respectively.
c       deckmag has already been calculated for each order.
c       Slitmag also varies slightly along each order, but here we simply
c       calculate the magnification when the Echelle is on-blaze.
c       magnification of slit on detector [dimensionless]
	slitmag = demag * cos(ecdelta + ectheta)/cos(ecdelta - ectheta)
C       ---------
c       Convert the angular deviations caused by the cross-disperser into
c       linear displacement at the camera focus.
c       up to this point, x has been the angular deviation [radians]
c       this loop converts each x into linear deviation [mm]
	if (nxdprism .gt. 0) then
c           there are prisms in use...
c           put x=0 in the middle of the format
	    temp = ( x(1,mc) + x(norders,mc) ) * 0.5
	else
c           no prisms in use
c           put x=0 at the point which is directly on the optical axis
c           of the beam as it leaves the cross disperser headed toward camera
	    temp = (xdangle + xddeltad - xdalfbet*0.5) * radPERdeg
	endif
	do 720 j = mci(1), mci(2)
	  do 700 i = 0,norders+1
	    x(i,j) = (x(i,j) - temp) * camfocln * mmPERm
700       continue
720     continue
c       using the previous calculations, take their "derivative"
	do 750 i = 1,norders
	  delx(i) = 0.5 * (x(i+1,mc) - x(i-1,mc))
	  phi(i) = focscale * delx(i) / deckmag(i)
750     continue
c       finally, calculate where the detector is, and how big slit is
	call spc2det
	call calcdeck
	call calcslit
	call echtran
	return
	end
C==============================================================================
C==============================================================================
	subroutine det2spc(egtilt)
c       Given information about the position of the detector on the Echelle
c       format, calculate the configuration of the spectrograph.
c       COMPARE THIS ROUTINE WITH spc2det, WHICH DOES THE OPPOSITE.
c       Inputs are the last position of the cursor clambda and
c       cxlambda or the position of the mosaic.
c       Results are new values of ecangle xdangle xdalphad hamhgd egtilt.
	IMPLICIT_NONE
C       Parameters
#       include "include/units.par"
#       include "echelle.par"
C       External Variables
c                       this might have been in the setup.inc common block
c                       but is is not.  It is the tilt of the Hamilton
c                       Echelle grating measured in encoder units.
	real            egtilt
C       Internal Variables
C       Common Block
	include 'spgraf.inc'
	include 'setup.inc'
	include 'echord.cmn'
	include 'lims.cmn'
	include 'cdata.inc'
C       Executable Code
c       use the position of the mosaic to determine which wavelengths at origin
c       Note that this corrupts the values of everything in the cdata common
c       such that it thinks the cursor was at the origin of the mosaic,
c       but that the cursor may, in fact, have been at an edge or corner
c       after a drag of the detector.
c       Note that we take care to find the position of the center of
c       the mosaic, not the position of the origin (which may not be at center).
	call cursdata(posmos(Xp,0)+cormos(Xp,0),
     &  posmos(Yp,0)+cormos(Yp,0))
c       calculate the Echelle rotation angle off blaze
	ecangle = asin( corder * clambda /
     &  (2 * AangPERum * ecsigma * cos(ectheta)))
     &  * degPERrad - ecdeltad
c       The following calc does not handle a spectrograph with both
c       prisms and gratings for cross dispersion.
	if (nxdgrat .gt. 0) then
	    xdangle = asin( mc * cxlambda /
     &      (2 * AangPERum * xdsigma * cos(xdalfbet*0.5*radPERdeg)))
     &      * degPERrad - xddeltad
	    xdalphad = xdangle + xddeltad + xdalfbet*0.5
	else
c           Hamilton Spectrograph with TI 800x800 CCD
c           Hamilton Dewar stepper motor height in device units
c           when order 120 is at center of chip, we are at hgt(0)
	    hamhgt = hgt(0) + hgt(1) *
     &      (posmos(Yp,0) + cormos(Yp,0) - x(i120,mc))
c           Hamilton Echelle grating tilt in device units
c           when center of chip is on blaze, we are at egt(0)
	    egtilt = egt(0) + egt(1) *
     &      (posmos(Xp,0) + cormos(Xp,0))
	endif
	return
	end
C==============================================================================
C==============================================================================
	subroutine spc2det
c       Given information about the state of the spectrograph,
c       calculate where on the Echelle format the detector will be.
c       COMPARE THIS ROUTINE WITH det2spc, WHICH DOES THE OPPOSITE.
c       This is intended to be called after calcform.
c       Its typical use is to calculate where the detector array will be
c       on the Echelle format given grating angles and/or stage positions.
c       Inputs are xdangle or hamhgt and ecangle.
c       Results are posmos(Xp,0), posmos(Yp,0), xdalphad, and
c       a new transform matrix.
	IMPLICIT_NONE
C       Parameters
#       include "include/units.par"
#       include "echelle.par"
c                       symbolic to avoid any confusion
	integer         ACPL
	parameter       (ACPL = 2)
C       Internal Variables
c                       cross disperser wavelength produced by xdangle [Aang.]
	real            xlam
c                       Echelle wavelength produced by ecangle [Aangstrom]
	real            elam
c                       position of wavelengths on Echelle format [mm]
	real            xyl(ACPL,Xp:Yp)
c                       indices of Echelle orders nearest to wavelength
	integer         numord(ACPL)
c                       loop
	integer         i
C       Common Block
	include 'spgraf.inc'
	include 'setup.inc'
	include 'echord.cmn'
	include 'lims.cmn'
C       Executable Code
c       This code does not handle the case where there are both prisms and
c       gratings for cross dispersion.
	if (nxdgrat .gt. 0) then
c           cross dispersing grating
	    xlam = (2 * AangPERum * xdsigma *
     &      cos(xdalfbet*0.5*radPERdeg) *
     &      sin((xddeltad + xdangle) * radPERdeg)) / mc
	    call wav2pos(xlam, xyl, numord)
c           (mosaic center) = (position of desired light) - (coord of ctr)
	    posmos(Yp,0) = xyl(1,Yp) - cormos(Yp,0)
	    xdalphad = xdangle + xddeltad + xdalfbet*0.5
	else
c           cross dispersing prism
C           THIS CODE ONLY WORKS FOR HAMILTON SPEC WITH TI 800x800 CCD
	    posmos(Yp,0) = ((hamhgt - hgt(0)) / hgt(1)) +
     &      (x(i120,mc) - cormos(Yp,0))
	endif
c       this is the blaze wavelength times Echelle order number
	elam = 2 * ecsigma * AangPERum * cos(ectheta) *
     &  sin((ecdeltad + ecangle) * radPERdeg)
	do 100 i=1,ACPL
	  icord = numord(i)
c         this is the number of Free Spectral Ranges off blaze
	  xyl(i,Yp) = (elam / morder(icord) - wv(icord)) / fsr(icord)
100     continue
	if (abs(xyl(1,Yp)) .le. abs(xyl(2,Yp))) then
c           choose the first position as closest
	    i = 1
	else
c           choose the second position as closest
	    i = 2
	endif
	icord = numord(i)
c       position of center = (# of mm off blaze) - (coord of center)
	posmos(Xp,0) = xyl(i,Yp) * f2db(icord) - cormos(Xp,0)
c       calculate the new transformation matrix from echelle to mosaic
	call distran
	return
	end
C==============================================================================
C==============================================================================
	subroutine calcdeck
c       Step through all the possible ways in which the decker may be set
c       Find the way with the highest priority and use it to reset all
c       the other values.
C       Parameters
#       include "echelle.par"
#       include "include/stdio.par"
#       include "include/units.par"
C       Internal Variables
c                       geometric mean pixel size
	real            pixmean
c                       loop
	integer         i
C       Common Blocks
	include 'detmos.inc'
	include 'echord.cmn'
	include 'setup.inc'
	include 'spgraf.inc'
C       Executable Code
c       This code does not work for non-square pixels,
c       but we take the geometric mean of the pixels in such a case.
	pixmean = sqrt(pixsiz(ONLY1,Xp) * pixsiz(ONLY1,Yp))
c       The order nearest the center of the detector is icord.
c       The decker size in pixels varies slightly across the Echelle
c       format.  DECKHGT and DECKSIZE are constant, but DECKSIZE depends
c       upon the telescope and not on the spectrograph proper.
c       DECKHGT is therefore regarded as the canonical specification.
c       If the observer specified something else, the order at the center of
c       the detector is used to set DECKHGT from the wishes of the observer.
	if     (deckmeth .eq. i_DECKHGT ) then
c           do nothing
	elseif (deckmeth .eq. i_DECKSIZE) then
	    deckhgt = decksize / (focscale * mmPERm)
	elseif (deckmeth .eq. i_DECKPIX ) then
	    deckhgt = deckpix * pixmean / deckmag(icord)
	else
	    write(STDERR,*) 'calcdeck: no method'
	endif
	decksize = deckhgt * focscale * mmPERm
	do 100 i = 0,MAXORD
	  dckpix(i) = deckhgt * deckmag(i) / pixmean
100     continue
	deckpix  = dckpix(icord)
	return
	end
C==============================================================================
C==============================================================================
	subroutine calcslit
c       Step through all the possible ways in which the slit may be set
c       Find the way with the highest priority and use it to reset all
c       the other values.
C       Parameters
#       include "echelle.par"
#       include "include/stdio.par"
#       include "include/units.par"
c                       number of machine encoder units per meter of slit width
	real            meuPERslitm
	parameter       (meuPERslitm = 1.e6)
C       Internal Variables
c                       geometric mean pixel size
	real            pixmean
C       Common Blocks
	include 'detmos.inc'
	include 'echord.cmn'
	include 'setup.inc'
	include 'spgraf.inc'
C       Executable Code
c       This code does not work for non-square pixels,
c       but we take the geometric mean of the pixels in such a case.
	pixmean = sqrt(pixsiz(ONLY1,Xp) * pixsiz(ONLY1,Yp))
c       The order nearest the center of the detector is icord.
c       The slit size in pixels and velocity varies slightly across the
c       Echelle format.  It actually varies along each Echelle order, but
c       that effect is NOT modelled here.
c       SLITWID and SLITSIZE are constant, but SLITSIZE depends
c       upon the telescope and not on the spectrograph proper.
c       SLITPIX does not change here, but slitmag is not really a
c       constant (this is the unmodelled effect).
c       SLITWID is therefore regarded as the canonical specification.
c       If the observer specified something else, the order at the center of
c       the detector is used to set SLITWID from the wishes of the observer.
	if     (slitmeth .eq. i_SLITWID ) then
c           do nothing
	elseif (slitmeth .eq. i_SLITSIZE) then
/*          slitwid  = slitsize / (focscale * mmPERm)                   */
	    slitwid  = slitsize /  focscale
	elseif (slitmeth .eq. i_SLITPIX ) then
/*          slitwid  = slitpix * pixmean / slitmag                      */
	    slitwid  = mmPERm * slitpix * pixmean / slitmag
	elseif (slitmeth .eq. i_SLITVEL ) then
/*          slitwid  = slitvel * base*f2dbdl / (slitmag*cLIGHT*mmPERm)  */
	    slitwid  = slitvel * base*f2dbdl / (slitmag*cLIGHT)
	elseif (slitmeth .eq. i_SLITRAW ) then
/*          slitwid  = slitraw / meuPERslitm                            */
	    slitwid  = mmPERm * slitraw / meuPERslitm
	else
	    write(STDERR,*) 'calcslit: no method'
	endif
#       ifdef METER
c       this was the old way, slitwid in [meter]
	slitsize = slitwid * focscale * mmPERm
	slitpix  = slitwid * slitmag / pixmean
	slitraw  = slitwid * meuPERslitm
	slitvel  = slitwid * slitmag * cLIGHT * mmPERm / (base * f2dbdl)
#       else /* METER */
c       new way, slitwid now in [mm]
	slitsize = slitwid * focscale
	slitpix  = slitwid * slitmag / pixmean * mPERmm
	slitraw  = slitwid * meuPERslitm * mPERmm
	slitvel  = slitwid * slitmag * cLIGHT / (base * f2dbdl)
#       endif /* METER */
	return
	end
C==============================================================================
C==============================================================================
	integer function findline(x,y,indx)
c       find the marked spectral line closest to point (x,y)
	IMPLICIT_NONE
C       Parameters
#       include "echelle.par"
#       include "include/stdio.par"
c                       default distance greater than which is ridiculous
	real            TOOFAR
	parameter       (TOOFAR = 1.e30)
C       External Variables
c                       x and y location of a point on the echelle format [mm]
	real            x, y
c                       if a line is found, this is the index pointing to it
	integer         indx
C       Internal Variables
c                       loop
	integer         i, j
c                       nearest distance so far
	real            dist, newdist
C       Common Blocks
	include 'lam.cmn'
	include 'linpos.cmn'
C       Executable Code
	if (nlam .le. 0) then
c           we return something other than SUCCESS
c           someday we might want to define other status codes
	    findline = EOF
	    return
	endif
	dist = TOOFAR
	do 20 j=1,2
	  do 10 i=MINLIN,MINLIN-1+nlam
	    newdist = ((x - xline(i,j))**2 + (y - yline(i,j))**2)
	    if (newdist .lt. dist) then
		dist = newdist
		indx = i
	    endif
10        continue
20      continue
	findline = SUCCESS
	return
	end
C==============================================================================
C==============================================================================
