#include "include/port.h"
C==============================================================================
C==============================================================================
	subroutine minimize(sigi,thetad,deltad)
c       Minimize deviation of lines from blaze center by searching
c       through parameter space.  This routine is intended to be used
c       when designing a new spectrograph.
	IMPLICIT_NONE
#       include "include/stdio.par"
#       include "echelle.par"
#       include "include/units.par"
C       External Variables
c                       spacing of grooves on echelle grating [grooves/mm]
	real            sigi
c                       half angle (thetad) [degrees]
	real            thetad
c                       echelle blaze angle [degrees]
	real            deltad
C       Local Variables
	real            theta1, theta2, delta1, delta2
	real            stemp, sig1, sig2
	real            th1, th2, thb1, thb2, dth, dthb
	real            qmin, scheck, q
	integer         nth, nthb, nds, ith, ithb
	integer         isig, isig1, isig2, jsig
	real            sig0, theta0, delta0
	character*80    oneline
C       External Function
	real            func
C       Executable Code
700     write(STDOUT,PROMPT)
     &  ' Enter range of spacing [mm**-1] <min> <max>:  '
	call mx11gets(oneline)
	read(oneline,*,ERR=700) sig1,sig2
725     write(STDOUT,PROMPT)
     &  ' Enter range of theta [degrees] <min> <max>:  '
	call mx11gets(oneline)
	read(oneline,*,ERR=725) theta1,theta2
740     write(STDOUT,PROMPT)
     &  ' Enter range of blaze angle [degrees] <min> <max>:  '
	call mx11gets(oneline)
	read(oneline,*,ERR=740) delta1,delta2
	IF (SIG1 .GT. SIG2) THEN
c           reorder spacing values if they were backward
	    STEMP = SIG2
	    SIG2 = SIG1
	    SIG1 = STEMP
	ENDIF
	THB1 = delta1
	THB2 = delta2
	TH1 = THETA1
	TH2 = THETA2
	dth = min(abs(theta2-theta1)/20.,0.2)
	dthb = min(abs(delta2-delta1)/10.,0.25)
	QMIN = 1.E20
c       Check to see if one of the search parameters is fixed
	IF (THETA2-THETA1 .LE. 0) THEN
	    NTH = 1
	ELSE
	    NTH = IFIX((THETA2-THETA1)/DTH) + 1
	ENDIF
	IF (delta2-delta1 .LE. 0) THEN
	    NTHB = 1
	ELSE
	    NTHB  = IFIX((delta2-delta1)/DTHB) + 1
	ENDIF
	if(sig1 .eq. sig2) then
	    nds = 1
	else
c           Force spacing to be an integral number of HeNeWAV angstroms.
	    SIG1 = AangPERmm / SIG1 + HeNeWAV
	    SIG2 = AangPERmm / SIG2
	    ISIG2 = IFIX(SIG2/HeNeWAV)
	    SIG2 = HeNeWAV*FLOAT(ISIG2)
	    ISIG1 = NINT(SIG1/HeNeWAV)
	    NDS = ISIG1-ISIG2
	endif
	DO 759 ITH = 1,NTH
	  DO 757 ITHB = 1,NTHB
	    DO 755 ISIG = 1,NDS
	      SCHECK=SIG2+HeNeWAV*(ISIG-1)
	      IF(SCHECK .GT. SIG1) GO TO 755
	      SIGI = SCHECK
	      thetad = (ITH-1)*DTH + THETA1
	      deltad = delta1 + (ITHB-1)*DTHB
	      Q = FUNC(SIGI,thetad,deltad)
c             Mimimize Q,the weighted sum square of the line deviations
c             from blaze center
	      IF(Q .LT. QMIN) THEN
		  QMIN = Q
		  SIG0 = SIGI
		  THETA0 = thetad
		  delta0 = deltad
		  JSIG = ISIG
	      ENDIF
755         CONTINUE
757       CONTINUE
759     CONTINUE
	SIG0 = AangPERmm / SIG0
	thetad = THETA0
	deltad = delta0
	SIGI = SIG0
	return
	end
C==============================================================================
C==============================================================================
	real FUNCTION FUNC(SIG,thetad,deltad)
c       Find the weighted sum squared deviations of the line positions from
c       the blaze wavelength (Q-value). Weights range from 1-10.
	IMPLICIT_NONE
C       Parameters
#       include "echelle.par"
#       include "include/units.par"
C       External Variables
c                       spacing of grooves on echelle [grooves/mm]
	real            sig
c                       half angle (theta) [degrees]
	real            thetad
c                       echelle blaze angle [degrees]
	real            deltad
C       Internal Variables
	integer         i, m
	real            b, qtemp, wb, q
C       Common Block
	include 'lam.cmn'
C       Executable Code
c       Grating equation
	b = 2 * sig * sin(deltad*radPERdeg) * cos(thetad*radPERdeg)
	DO 10 I = 1,NLAM
c         Get order (m)
	  M = NINT(B / LAMBDA(I))
C         Get blaze wavelength
	  WB = B / M
	  QTEMP = WT(I)*2*(B/WB)*(LAMBDA(I)-WB)/WB
	  Q = QTEMP**2 + Q
10      continue
C       Normalize to the number of wavelengths
	FUNC = Q / NLAM
	RETURN
	END
C==============================================================================
C==============================================================================
