#include "include/port.h"
C==============================================================================
C==============================================================================
	subroutine findfiles(tailend,numd,defdir,numf,files)
c       given tail end of a file name (the yyyy part of 'xxxx.yyyy')
c       find all files in a set of specified default directories
c       that end with that tail.
c       Return this list of files to the calling routine.
c       It is presumed that the user has the ability to write in the
c       current working directory.  Otherwise this fails.
c       This is, by its nature, very operating system dependent.
c       This should work on most Un*x systems.
	IMPLICIT_NONE
C       Parameters
#       include "include/stdio.par"
c                       logical unit for reading directory file
	integer         DLUN
	parameter       (DLUN = 97)
c                       max size of command line
	integer         COMLEN
	parameter       (COMLEN = 256)
c                       name of temporary file
	character*(*)   TMPFILE
	parameter       (TMPFILE = '.dir.')
c                       command to remove a file
	character*(*)   REMOVE
	parameter       (REMOVE = '/bin/rm -f')
c                       command to create an empty file
	character*(*)   CREATE
	parameter       (CREATE = '/bin/touch')
C       External Variables
c                       The tail end of the files to be matched
c                       This is taken literally on a Un*x system
c                       if value is 'foo.bar' we find *.foo.bar
c                       if value is '.foo.bar' we find *..foo.bar
c                       But we do not find "hidden" files starting with '.'
	character*(*)   tailend
c                       number of directories to search
	integer         numd
c                       default directory to search
	character*(*)   defdir(numd)
c                       On Input:  numf(0) is compiled size of array of strings
c                       On Output: numf(0) is total number of matches
c                       On Output: numf(i) is number of matched in dir i
	integer         numf(0:numd)
c                       list of matching files
	character*(*)   files(numf(0))
C       Internal Variables
c                       save value of numf(0)
	integer         maxf
c                       loops etc.
	integer         i, j
c                       length of the tail end
	integer         lentail
c                       length of default directory
	integer         lendef
c                       length of matching result
	integer         lenr
c                       command sent to OS to find contents of dir
	character       comstr*(COMLEN)
c                       Fortran I/O status
	integer         ios
C       External Functions
	integer         lenc
	integer         system
C       Executable Code
#       define STRIPSPACE
c       find correct length of the tail string
	lentail = lenc(tailend)
#       ifdef STRIPSPACE
	i = index(tailend,' ')
	if (i .ne. 0 .and. i .lt. lentail) lentail = i
#       endif /* STRIPSPACE */
	maxf = numf(0)
	numf(0) = 0
	do 1000 j=1,numd
	  numf(j) = 0
c         get rid of any previous result file
	  comstr = REMOVE//' '//TMPFILE//';'//CREATE//' '//TMPFILE
	  i = system(comstr)
c         find the correct length of the directory
	  lendef = lenc(defdir(j))
c         get all the matching names in this directory
	  if (lendef .gt. 0) then
#             ifdef STRIPSPACE
	      i = index(defdir(j),' ')
	      if (i .ne. 0 .and. i .lt. lendef) lendef = i
#             endif /* STRIPSPACE */
c             Note that this is specific to the Cshell, oops
	      comstr = 'csh -f -c "(/bin/ls '//defdir(j)(:lendef)//'/*.'
     &        //tailend(:lentail)//' >> '//TMPFILE//') >& /dev/null"'
	      i = system(comstr)
	  endif
c         read the matching names
	  open(unit=DLUN,file=TMPFILE,status='old')
	  rewind(DLUN)
	  i = numf(j-1)
	  ios = SUCCESS
100       continue
	  read(DLUN,ALINE,iostat=ios) comstr
	  if (ios .eq. SUCCESS) then
c             read another one successfully
	      lenr = lenc(comstr)
	      if (i .ge. maxf) then
		  write(STDERR,*)
     &            'findfiles: no room for '//comstr(:lenr)
	      elseif (comstr(1:1)       .eq. '#'
     &        .or.    comstr(lenr:lenr) .eq. '~' ) then
c                 ignore files with such names
c                 they are presumed to be editor backups
	      elseif (lenr-(lendef+1) .le. len(files(1))) then
c                 the name of the file will fit in the output string
		  if (comstr(lendef+2:lendef+2) .eq. '#') then
c                     kindly ignore this file
		  else
		      i = i + 1
		      numf(j) = numf(j) + 1
		      files(i) = comstr(lendef+2:)
		  endif
	      else
		  write(STDERR,*) 'findfiles: file name too long'
		  write(STDERR,*) comstr(:lenr)
	      endif
	      goto 100
	  elseif (ios .eq. EOF) then
c             normal completion, fall off if blocks
	  else
	      write(STDERR,*) 'findfiles: something weird'
	  endif
1000    continue
c       assign the proper count
	numf(0) = i
	return
	end
C==============================================================================
C==============================================================================
	subroutine canondir(envar,result)
c       Search the environment for a variable with the name in envar.
c       If one exists, put its value into result.
c       Otherwise, leave result unmodified.
	IMPLICIT_NONE
C       Parameters
#       include "include/stdio.par"
c                       maximum length of value of environment variable
	integer         MAXLEN
	parameter       (MAXLEN = 128)
C       External Variables
c                       If there exists an environment variable with the
c                       name in envar, its value is put into result.
c                       If there is no such environment variable
c                       result is not modified.
	character*(*)   envar
c                       presumably a hard-coded default directory which
c                       may be overridden by the environment.
	character*(*)   result
C       Internal Variables
c                       value of environment variable
	character       enval*(MAXLEN)
c                       length of result
	integer         lres
c                       length of environment variable
	integer         lenv
C       External Functions
	integer         lenc
C       Executable Code
	lenv = lenc(envar)
	if (lenv .eq. 0) return
	call getenv(envar,enval)
	lenv = lenc(enval)
	if (lenv .gt. 0) then
c           there is a translation
	    lres = len(result)
	    if (lenv .gt. lres) then
		write(STDERR,*) 'canondir: result does not fit'
	    else
		result = enval
	    endif
	endif
	return
	end
C=======================================================================
C=======================================================================
	subroutine canonfile(numd,defdir,result)
c       given a file name, find out if it exists and can be read
	IMPLICIT_NONE
C       Parameters
#       include "include/stdio.par"
c                       maximum size of full path name of a file
	integer         MXPATH
	parameter       (MXPATH = 128)
C       External Variables
c                       number of directories to search
	integer         numd
c                       default directory to search
	character*(*)   defdir(numd)
c                       the result file to inquire about
c                       if inquire fails, the result is blanked
	character*(*)   result
C       Internal Variables
c                       loop
	integer         i
c                       does the file exist?
	logical         ex
c                       length of default directory
	integer         lendef
c                       place to hold constructed file name
	character       filenm*(MXPATH)
C       External Functions
	integer         PMGO(lenc)
C       Executable Code
	if (result .eq. ' ') return
c       first try the filename exactly as given
	ex = .false.
	inquire(file=result,exist=ex)
	if (ex) return
c       otherwise, loop over all possible directories
	do 1000 i=1,numd
	  lendef = PMGO(lenc)(defdir(i))
	  filenm = defdir(i)(:lendef)//'/'//result
	  inquire(file=filenm,exist=ex)
	  if (ex) goto 9000
1000    continue
c       done looping
9000    continue
	if (ex) then
	    result = filenm
	else
c           we did not find such a file
	    result = ' '
	endif
	return
	end
C==============================================================================
C==============================================================================
