/*
 *      If not for the history of this program (being that it was
 *      originally written in Fortran, and that a goal
 *      in its design was to use Lick Mongo to insure that it could
 *      operate on a wide variety of graphics devices) this would
 *      consist of macro-replicated C code.
 *
 *      As it is, it consists of macro-replicated Fortran code.
 *      It works, but it would be more transparent in C.
 */

#include "include/port.h"
#ifdef CURSOR
#   define INCFILE 'cdata.inc'
#   define DATFILE "cdata.dta"
#   define TITLE   cdata
#   define PF c
#   define MIN MINCDATA
#   define BOT botcdata
#   define MAX MAXCDATA
#   define NUL NULCDATA
#endif /* CURSOR */
#ifdef SPGRAF
#   define INCFILE 'spgraf.inc'
#   define DATFILE "spgraf.dta"
#   define TITLE   spgraf
#   define PF i       /* "i" is for "instrument", telescope/spect. combo */
#   define MIN MINIDATA
#   define BOT botidata
#   define MAX MAXIDATA
#   define NUL NULIDATA
#endif /* SPGRAF */
#ifdef SETUP
#   define INCFILE 'setup.inc'
#   define DATFILE "setup.dta"
#   define TITLE   setup
#   define PF s
#   define MIN MINSDATA
#   define BOT botsdata
#   define MAX MAXSDATA
#   define NUL NULSDATA
#endif /* SETUP */
#ifdef DETMOS
#   define INCFILE 'detmos.inc'
#   define DATFILE "detmos.dta"
#   define TITLE   detmos
#   define PF d
#   define MIN MINDDATA
#   define BOT botddata
#   define MAX MAXDDATA
#   define NUL NULDDATA
#endif /* DETMOS */
#ifdef BUTTON
#   define INCFILE 'button.inc'
#   define DATFILE "button.dta"
#   define TITLE   button
#   define PF b
#   define MIN MINBDATA
#   define BOT botbdata
#   define MAX MAXBDATA
#   define NUL NULBDATA
#endif /* DETMOS */
C==============================================================================
C==============================================================================
	subroutine ud TITLE
c       The screen displays some data regarding the portion of the
c       echelle format underneath the cursor.  This routine updates
c       all of the information that is expected to change as the cursor moves.
c       This code underwent a severe rewrite as a part of the final
c       polishing.  The final product for Keck does not want to see all the
c       labels on the screen which allow the user to modify the spectrograph.
c       In the original version, this was all done in the nightmare of code
c       in the file echpltxt.F.  It is debatable whether this new code is
c       any less of a nightmare, but at least it breaks up that horrid
c       label code into smaller chunks so that not all of it need be
c       displayed.
	IMPLICIT_NONE
C       Parameters
#       include "echelle.par"
#       include "include/stdio.par"
C       Internal Variables
c                       loop
	integer         i
c                       terseness
	integer         j
c                       I/O status
	integer         ios
C       External Function
	integer         PMGO(lenc)
C       Common Block
	include INCFILE
C       Data
#       include DATFILE
C       Executable Code
	if (.not. PF dataup) return
	do 200 i = MIN,BOT
	  j = PF datap(i)
c         write out the new label into the string for storage
	  if (j .eq. NUL) then
	      goto 199
	  elseif (PF dataf(j) .eq. NoValFmt) then
	      goto 199
	  elseif (index(PF dataf(j),'f') .ne. 0) then
c             presume real data
	      write(PF datav(SVNEW,j),PF dataf(j),iostat=ios)
     &        PF datar(j)
	  elseif (index(PF dataf(j),'i') .ne. 0) then
c             presume integer data
	      write(PF datav(SVNEW,j),PF dataf(j),iostat=ios)
     &        PF datai(j)
	  elseif (index(PF dataf(j),'a') .ne. 0) then
c             presume character data--already written into NEW string
	  else
	      write(STDERR,*) 'ud*:  unknown data format'
	      write(STDERR,*) 'ud*:  ', PF dataf(j),i,j
	  endif
c         erase the label string from last time
	  call PMGO(grelocate)(PF datax(Xp,j,2),PF datax(Yp,j,2))
	  call PMGO(setlvis)(1)
	  call PMGO(label)
     &    (PMGO(lenc)(PF datav(SVOLD,j)),PF datav(SVOLD,j))
c         draw the new label string on the screen
	  call PMGO(grelocate)(PF datax(Xp,j,2),PF datax(Yp,j,2))
	  call PMGO(setlvis)(0)
	  call PMGO(label)
     &    (PMGO(lenc)(PF datav(SVNEW,j)),PF datav(SVNEW,j))
c         copy new into old
	  PF datav(SVOLD,j) = PF datav(SVNEW,j)
199       continue
200     continue
	return
	end
C==============================================================================
C==============================================================================
	subroutine dn TITLE
c       This routine undisplays the cursor data menu.
	IMPLICIT_NONE
C       Parameters
#       include "echelle.par"
C       Internal Variables
c                       loop
	integer         i
c                       terseness
	integer         j
C       External Function
	integer         PMGO(lenc)
C       Common Block
	include INCFILE
C       Executable Code
	if (.not. PF dataup) return
	call PMGO(setlvis)(1)
	do 200 i = MIN,BOT
	  j = PF datap(i)
	  if (j .eq. NUL) goto 199
c         erase the label strings
	  call PMGO(grelocate)(PF datax(Xp,j,1),PF datax(Yp,j,1))
	  call PMGO(label)(PMGO(lenc)(PF datal(j)),PF datal(j))
c         erase the value strings
	  call PMGO(grelocate)(PF datax(Xp,j,2),PF datax(Yp,j,2))
	  call PMGO(label)
     &    (PMGO(lenc)(PF datav(SVOLD,j)),PF datav(SVOLD,j))
c         blank the old strings for next time
	  PF datav(SVOLD,j) = ' '
199       continue
200     continue
	call PMGO(setlvis)(0)
	PF dataup = .false.
	return
	end
C==============================================================================
C==============================================================================
	subroutine up TITLE
c       This routine redisplays the cursor data menu.
c       The positions of each label are always recalculated.
	IMPLICIT_NONE
C       Parameters
#       include "echelle.par"
C       Internal Variables
c                       loop
	integer         i
c                       terseness
	integer         j
c                       is there a Lick Mongo escape character in string?
	logical         mgoesc
C       External Function
	integer         PMGO(lenc)
C       Common Block
	include INCFILE
	include 'lims.cmn'
C       Executable Code
	if (.not. PF meninit) call ra TITLE(0)
	call PMGO(setlvis)(0)
	do 200 i = MIN,BOT
	  j = PF datap(i)
	  if (j .eq. NUL) goto 199
c         calculate where the label should be
	  PF datax(Xp,j,1) = pxmin + (pxmax - pxmin) * 0.03
	  PF datax(Yp,j,1) = pymin + (pymax - pymin) *
     &    ((((PF minndc - PF maxndc) /
     &    (BOT-MIN+1)) *
     &    (i-MIN+0.5)) + PF maxndc)
c         plot the label
	  call PMGO(grelocate)(PF datax(Xp,j,1),PF datax(Yp,j,1))
	  mgoesc = index(PF datal(j),char(92)//'gV') .ne. 0
	  if (mgoesc) call PMGO(setpsfmode)(1)
	  call PMGO(label)(PMGO(lenc)(PF datal(j)),PF datal(j))
	  if (mgoesc) call PMGO(setpsfmode)(2)
c         store the current location for plotting RHS of equations
	  call PMGO(getgxy)(PF datax(Xp,j,2),PF datax(Yp,j,2))
c         call PMGO(label)(PMGO(lenc)(PF datav(j)),PF datav(j))
199       continue
200     continue
	PF dataup = .true.
c       updating the data is always a part of putting up the menu
	call ud TITLE
	return
	end
C==============================================================================
C==============================================================================
	logical function wb TITLE(xw,yw,index)
c       This function determines whether the given position in world
c       coordinates is within the boundaries of one of the labels
c       being displayed in the menu.
c       If it is within a boundary, the index of that label is returned.
	IMPLICIT_NONE
C       Parameters
#       include "echelle.par"
C       External Variables
c                       x and y world coordinates
	real            xw, yw
c                       return value of closest index
c                       if the return value of the function is false,
c                       this is undefined
	integer         index
C       Internal Variables
c                       x and y normalized device coordinates
	real            xnd, ynd
C       Common Block
	include INCFILE
C       Executable Code
c       see if this menu is currently displayed
	wb TITLE = PF dataup
	if (.not. wb TITLE) then
	    return
	endif
c       convert to NDC
	call nc TITLE(xw, yw, xnd, ynd)
	wb TITLE = (ynd .ge. MIN) .and. (ynd .le. BOT+1)
c       if we are outside the Y menu limits, return
	if (.not. wb TITLE) then
	    return
	endif
c       compute which label is closest, and lookup its true id via pointer
	index = PF datap(nint(ynd - 0.5))
c       see if the X menu limits are resonable
	wb TITLE = (xnd .ge. PF datax(Xp,index,1)) .and.
     &             (xnd .le. PF datax(Xp,index,2))
c       and that is all, one way or the other
	return
	end
C==============================================================================
C=======================================================================
	subroutine nc TITLE(wx,wy, gx, gy)
c       given a Lick Mongo world coordinate, find the menu coordinate
	IMPLICIT_NONE
C       Parameters
#       include "echelle.par"
C       External Variables
c                       input world coordinates
	real            wx, wy
c                       output y menu coordinates
c                       output device x coordinate
	real            gx, gy
C       Internal Variables
c                       device coordinate limits in effect when invoked
	real            gx1, gx2, gy1, gy2
c                       world coordinate limits in effect when invoked
	real            wx1, wx2, wy1, wy2
C       Common Block
	include 'lims.cmn'
	include INCFILE
C       Executable Code
c       get the transformation
	call getloc(gx1,gy1,gx2,gy2)
	call getlim(wx1,wy1,wx2,wy2)
c       convert to device coordinates
	gx = gx1 + (gx2-gx1)*(wx-wx1)/(wx2-wx1)
	gy = gy1 + (gy2-gy1)*(wy-wy1)/(wy2-wy1)
c       convert to Normalized device coordinates
	gy = (gy - pymin) / (pymax - pymin)
c       convert to Y menu coordinates
	gy = ((gy - PF maxndc) * (BOT - MIN + 1) /
     &  ( PF minndc - PF maxndc)) + MIN
	return
	end
C=======================================================================
C==============================================================================
	logical function ac TITLE (ascii,index)
c       This routine determines whether the ASCII character in
c       iascii is one of the accelerator keys.
c       The function returns .true. if there is a match,
c       otherwise it returns .false.
	IMPLICIT_NONE
C       Parameters
#       include "echelle.par"
C       External Variables
c                       the ASCII value of some character
	character*(*)   ascii
c                       the index of the matching table entry, if any
c                       if no match, value is undefined
	integer         index
C       Internal Variables
c                       loop
	integer         i
C       Common Block
	include INCFILE
C       Executable Code
	ac TITLE = .false.
	if (ascii .eq. NoAc) return
	do 100 i = MIN,MAX
	  if (ascii .eq. PF dataa(i)) then
	      ac TITLE = .true.
	      index = i
	      return
	  endif
100     continue
	return
	end
C==============================================================================
C==============================================================================
	logical function tr TITLE (string,j)
c       This routine takes the character string in string and
c       reads a value out of it, stuffing it into the
c       real (datar), integer (datai), or character value (datav) arrays.
c       The format is obtained from the dataf array.
c       The function returns .true. if it is successful,
c       otherwise it returns .false.
c       Much of this code is effectively the inverse operation of ud TITLE
	IMPLICIT_NONE
C       Parameters
#       include "echelle.par"
#       include "include/stdio.par"
C       External Variables
c                       the string from which we will read a value
	character*(*)   string
c                       the index of the table entry which will have
c                       its value replaced
	integer         j
C       Internal Variables
c                       Fortran I/O status
	integer         ios
C       Common Block
	include INCFILE
C       Executable Code
c       read in the new label into the string for storage
	if (PF dataf(j) .eq. NoValFmt) then
c           do nothing
	elseif (index(PF dataf(j),'f') .ne. 0) then
c           presume real data
	    read(string,*,iostat=ios) PF datar(j)
	elseif (index(PF dataf(j),'i') .ne. 0) then
c           presume integer data
	    read(string,*,iostat=ios) PF datai(j)
	elseif (index(PF dataf(j),'a') .ne. 0) then
c           presume character data
	    PF datav(SVNEW,j) = string
	else
	    write(STDERR,*) 'tr*:  unknown data format'
	    write(STDERR,*) 'tr*:  ', PF dataf(j),j
	endif
	tr TITLE = (ios .eq. SUCCESS)
	return
	end
C==============================================================================
C==============================================================================
