#include "include/port.h"
	program pdisp
c       calculate the dispersion of a prism as a function of the input angle
	IMPLICIT_NONE
#       include "echelle.par"
#       include "include/units.par"
C       Parameters
C       Internal Variables
c                       wavelength near which we want to look [micrometers]
	real            lambda
c                       vertex angle of the prism
	real            vertex
c                       trig. funcs. of vertex angle
	real            sa, ca
c                       incident angle theta
	real            thetin
c                       trig. func. of incident angle
	real            st
c                       index of refraction and its derivative
	real            refn, drefdl
c                       total angular deviation
	real            delta
c                       derivative of deviation wrt wavelength [radian/um]
	real            dddl
c                       index into glass type table
	integer         igt
c                       outgoing angle
	real            thetout
c                       temporary variables for calcs
	real            zeta
	real            xi
	integer         i
C       External Function
	integer         gldref
C       Executable Code
	write(STDERR,PROMPT)
     &  ' Give the vertex angle of the prism [degrees]:  '
	read(STDIN,*) vertex
	vertex = vertex * radPERdeg
	sa = sin(vertex)
	ca = cos(vertex)
	write(STDERR,PROMPT)
     &  ' Give the index number of the glass type:  '
	read(STDIN,*) igt
	write(STDERR,PROMPT)
     &  ' Give the wavelength of interest [um]:  '
	read(STDIN,*) lambda
	i = gldref(igt,lambda,refn,drefdl)
c       References given in the following code are to the book
c       Optics, E. Hecht & A. Zajac, Addison-Wesley, 1974
	do 10 i = 1,89
	  thetin = i * radPERdeg
	  st = sin(thetin)
c         see H&Z eqs. 5.52 and 5.53
	  zeta = sqrt(refn**2 - st**2)
	  xi = sa * zeta - st * ca
	  thetout = asin(xi)
	  delta = thetin - vertex + thetout
c         this results from differentiation of H&Z eq. 5.53
	  dddl = refn * sa * drefdl
	  dddl = dddl / (zeta * (1. - xi**2) )
c         convert back to degrees
	  delta = delta * degPERrad
	  thetout = thetout * degPERrad
	  write(STDOUT,*) i, thetout, delta, dddl
10      continue
	end
