#include "include/port.h"
C==============================================================================
C==============================================================================
	integer function readdet(detfile)
C       ---------------------
c       Open a file which presumably contains FITS-card-like lines
c       and read the configuration of a detector from that file.
C       ---------------------
	IMPLICIT_NONE
C       ---------------------
C       Parameters
#       include "include/stdio.par"
#       include "token/keyval.par"
#       include "echelle.par"
C       ---------------------
C       External Variables
c                       the almost-FITS file containing the detector info
c                       which we will try to open and try to read
	character*(*)   detfile
C       ---------------------
C       Internal Variables
c                       return status of keyval function
	integer         ts
c                       loop
	integer         i, j
c                       a place to read Conrad FITS string arrays
	character*132   string
c                       iostatus of an internal read
	integer         ios
c                       a place to keep a dynamically created FITS keyword
	character*9     fitskey
c                       a place to keep a dynamically created format string
	character*4     ifmt
#       ifdef RECENTER
c                       temporary position variable
	real            cp(Xo:Yo)
#       endif /* RECENTER */
c                       temporary badspot limits
	integer         badspt(Xo:dY)
C       ---------------------
C       External Function
	integer         figINT
	integer         figREAL
	integer         figQSTR
	integer         kvfile
	integer         kvgQSTR
	integer         kvgINT
	integer         kvgLOG
	integer         lenc
C       ---------------------
C       Common Block
	include 'detmos.inc'
C       ---------------------
C       Executable Code
	ifmt = '(i1)'
c       get the canonical file name
	string = detfile
c       open the file
	call kvverb(.true.)
	ts = kvfile(string)
	if (ts .ne. SUCCESS) return
	write(STDOUT,*) 'Reading detector file ',detfile(:lenc(detfile))
c       ----------------------------------------------------------------
c       there need not be any occurence of these cards
	call kvverb(.false.)
	ts =   kvgLOG( 'STDVIDV ='  ,1,0,stdvidv)
	if (ts .ne. SUCCESS) stdvidv = .true.
	ts =   kvgLOG( 'STDVIDH ='  ,1,0,stdvidh)
	if (ts .ne. SUCCESS) stdvidh = .true.
	ts =   kvgLOG( 'STDVIDR ='  ,1,0,stdvidr)
	if (ts .ne. SUCCESS) stdvidr = .true.
c       grab the results of this for each and every chip
	ts = figINT ('PHPRPX',nchips,phprepix(0))
c       ----------------------------------------------------------------
c       there should be only one occurrence of each of these cards
	call kvverb(.true.)
	call kvrew(.true.)
	ts =   kvgQSTR('DETECTOR='  ,1,0,detector)
	ts =   kvgINT( 'NCHIPS  ='  ,1,0,nchips)
c       ----------------------------------------------------------------
c       grab the results of these for each and every chip
	ts = figQSTR('CHIPID',nchips,chipid(0))
	ts = figINT ('XPIX'  ,nchips,npix(0,Xp))
	ts = figINT ('YPIX'  ,nchips,npix(0,Yp))
	ts = figREAL('PIXXSZ',nchips,pixsiz(0,Xp))
	ts = figREAL('PIXYSZ',nchips,pixsiz(0,Yp))
c       ----------------------------------------------------------------
c       we expect exactly one DETPOSn card for each of NCHIPS
	fitskey = 'DETPOS  ='
	do 2000 i=1,nchips
	  write(ifmt(3:3),'(i1)') int(log10(real(i))+1)
	  write(fitskey(7:8),ifmt) i
	  ts = kvgQSTR(fitskey      ,1,0,string)
	  if (ts .ne. KEYNOT) then
c             a DETPOS card may have 4 or 6 entries
c             first, try reading all 6 possible entries
	      read(string,*,iostat=ios) (detpos(j,i),j=Xp,RpY)
	      if (ios .ne. SUCCESS) then
c                 try reading only 4 entries
		  read(string,*,iostat=ios) (detpos(j,i),j=Xp,Orient)
c                 by default, the reference pixel location is (0.,0.)
		  detpos(RpX,i) = 0.
		  detpos(RpY,i) = 0.
	      endif
	      if (ios .ne. SUCCESS) then
		  write(STDERR,*) 'readdet:  bad format in a DETPOS card'
		  write(STDERR,*) 'string value was ', string
	      else
c                 check orientation value for conformity
		  if (abs(detpos(Orient,i)) .ne. 1.) then
		      write(STDERR,*) 'readdet:  Orient was not +/- 1'
		      write(STDERR,*) 'string value was ', string
		      write(STDERR,*) 'a standard video chip is assumed'
		      detpos(Orient,i) = 1.
		  endif
	      endif
	  endif
2000    continue
c       ----------------------------------------------------------------
c       There may be no BSPOTs on any of the chips in this mosaic
	call kvverb(.false.)
	ts =   kvgINT( 'NBSPOT  ='  ,1,0,nbspot)
	if (ts .ne. SUCCESS) nbspot = 0
	if (nbspot .gt. MAXBADSPOT) then
c           there are too many bad spots for us to remember
	    write(STDERR,*) 'readdet:  too many BSPOT cards'
	    write(STDERR,*) 'used only the first ', MAXBADSPOT
	    nbspot = MAXBADSPOT
	endif
	call kvverb(.true.)
	fitskey = 'BSPOT   ='
	do 3000 i=1,nbspot
	  write(ifmt(3:3),'(i1)') int(log10(real(i))+1)
	  write(fitskey(6:8),ifmt) i
	  ts = kvgQSTR(fitskey      ,1,0,string)
	  if (ts .eq. SUCCESS) then
c             we have another badspot card
	      read(string,*,iostat=ios) (bspot(j,i),j=cID,dY)
	      if (ios .ne. SUCCESS) then
		  write(STDERR,*) 'readdet:  bad format BSPOT card'
		  write(STDERR,*) 'string value was ', string
	      elseif (bspot(cID,i) .lt. 1
     &        .or.    bspot(cID,i) .gt. nchips) then
c                 warn about BSPOTs on chips that do not exist
		  write(STDERR,*)
     &            'readdet:  got BSPOT card for nonexistant chip ',
     &            bspot(cID,i)
	      endif
	  endif
3000    continue
c       ----------------------------------------------------------------
c       clip all badspots so that they do not extend beyond detector edges
	j = 1
	do 3500 i = 1,nbspot
c         clip all corners to limits of chip
	  badspt(Xp) = max(min(bspot(Xp,i), bspot(Xp,i)+bspot(dX,i)), 0)
	  badspt(dX) = min(max(bspot(Xp,i), bspot(Xp,i)+bspot(dX,i)),
     &    npix(bspot(cID, i),Xp))
	  badspt(Yp) = max(min(bspot(Yp,i), bspot(Yp,i)+bspot(dY,i)), 0)
	  badspt(dY) = min(max(bspot(Yp,i), bspot(Yp,i)+bspot(dY,i)),
     &    npix(bspot(cID, i),Yp))
c         find out new extent of the badspot
	  bspot(dX,j) = badspt(dX) - badspt(Xp)
	  bspot(dY,j) = badspt(dY) - badspt(Yp)
	  if (bspot(dX,j) .gt. 0 .and. bspot(dY,j) .gt. 0) then
	      bspot(Xp,j) = badspt(Xp)
	      bspot(Yp,j) = badspt(Yp)
	      j = j + 1
	  else
	      write(STDERR,*) 'readdet:  silly limits in BSPOT card'
	  endif
3500    continue
c       set nbspot to a possibly lower value
	nbspot = j - 1
c       ----------------------------------------------------------------
c       create the transformation matrices
	call mostran
#       ifdef RECENTER
c       if there was only one chip, translate its center to
c       the center of the detector mosaic.
	if (nchips .eq. 1) then
c           compute the middle pixel on the chip
	    cp(Xo) = npix(1,Xo) * 0.5
	    cp(Yo) = npix(1,Yo) * 0.5
c           translate middle of chip to mosaic coords (0,0)
	    detpos(Xp,1) = detpos(Xp,1) - (tmc(Xp,To,C2M,1) +
     &      tmc(Xp,Xo,C2M,1)*cp(Xo) + tmc(Xp,Yo,C2M,1)*cp(Yo))
	    detpos(Yp,1) = detpos(Yp,1) - (tmc(Yp,To,C2M,1) +
     &      tmc(Yp,Xo,C2M,1)*cp(Xo) + tmc(Yp,Yo,C2M,1)*cp(Yo))
c           recompute the transformation matrices
	    call mostran
	endif
#       endif /* RECENTER */
c       ----------------------------------------------------------------
c       close the file
	call kvverb(.false.)
	ts = kvfile(' ')
C       ---------------------
	return
	end
C==============================================================================
C==============================================================================
	subroutine mostran
C       ---------------------
c       Calculate the transformation matrix from chip coords to mosaic coords
c       and vice versa.  All chips are calculated together.
c       Alas, all communication is done thru the mosaic common block.
C       ---------------------
	IMPLICIT_NONE
C       ---------------------
C       Parameters
#       include "echelle.par"
#       include "include/units.par"
C       Internal Variables
c                       loop
	integer         i
c                       trig functions
	real            sinrot, cosrot
C       ---------------------
C       Common Block
	include 'detmos.inc'
C       ---------------------
C       Executable Code
c       All coordinates here are taken to be real-valued coordinates.
c       This routine therefore uses the convention that the edges of
c       pixels have integral values and the centers of pixels have
c       half-integral values.
c       To convert from one system to the other use these rules:
c       real_value    = integer_value + 0.5
c       integer_value = floor( real_value )
	do 1000 i=1,nchips
	  sinrot = detpos(Rot,i) * radPERdeg
	  cosrot = cos( sinrot )
	  sinrot = sin( sinrot )
c         --------------------------------------------------------------
c         calculate the transformation from Chip coords to Mosaic coords
	  tmc(Xp, Xo, C2M, i) =  cosrot * pixsiz(i, Xo)
	  tmc(Yp, Xo, C2M, i) =  sinrot * pixsiz(i, Xo)
	  tmc(Xp, Yo, C2M, i) =  sinrot * pixsiz(i, Yo) *
     &    detpos(Orient, i)
	  tmc(Yp, Yo, C2M, i) = -cosrot * pixsiz(i, Yo) *
     &    detpos(Orient, i)
	  tmc(Xp, To, C2M, i) = detpos(Xp, i) -
     &    (detpos(RpX, i) * tmc(Xp, Xo, C2M, i)) -
     &    (detpos(RpY, i) * tmc(Xp, Yo, C2M, i))
	  tmc(Yp, To, C2M, i) = detpos(Yp, i) -
     &    (detpos(RpX, i) * tmc(Yp, Xo, C2M, i)) -
     &    (detpos(RpY, i) * tmc(Yp, Yo, C2M, i))
c         --------------------------------------------------------------
c         calculate the transformation from Mosaic coords to Chip coords
	  tmc(Xp, To, M2C, i) = detpos(RpX,i)
     &    -(detpos(Xp,i)*cosrot+detpos(Yp,i)*sinrot)/pixsiz(i,Xp)
	  tmc(Yp, To, M2C, i) = detpos(RpY,i) + detpos(Orient, i) *
     &     (detpos(Yp,i)*cosrot-detpos(Xp,i)*sinrot)/pixsiz(i,Yp)
	  tmc(Xp, Xo, M2C, i) =  cosrot / pixsiz(i, Xp)
	  tmc(Yp, Xo, M2C, i) =  sinrot / pixsiz(i, Yp) *
     &    detpos(Orient, i)
	  tmc(Xp, Yo, M2C, i) =  sinrot / pixsiz(i, Xp)
	  tmc(Yp, Yo, M2C, i) = -cosrot / pixsiz(i, Yp) *
     &    detpos(Orient, i)
c         --------------------------------------------------------------
1000    continue
C       ---------------------
	return
	end
C==============================================================================
C==============================================================================
