#include "include/port.h"
C==============================================================================
C==============================================================================
	integer function readspc(spcfile)
C       ---------------------
c       Open a file which presumably contains FITS-card-like lines
c       and read the configuration of a spectrograph from that file.
C       ---------------------
	IMPLICIT_NONE
C       ---------------------
C       Parameters
#       include "include/stdio.par"
#       include "token/keyval.par"
#       include "echelle.par"
#       include "include/units.par"
C       ---------------------
C       External Variables
c                       the almost-FITS file containing the detector info
c                       which we will try to open and try to read
	character*(*)   spcfile
C       ---------------------
C       Internal Variables
c                       return status of token function
	integer         tks(0:MAXPRISM)
c                       other names for the above array for convenience
	integer         ts, ts1
	equivalence     (tks(0), ts)
	equivalence     (tks(1), ts1)
c                       loop
	integer         i
c                       a place to keep the mean of the prism info
	real            dum
c                       a place to keep a dynamically created FITS keyword
c       character*9     fitskey
c                       a place to read Conrad FITS string arrays
	character*132   string
C       ---------------------
C       External Function
	integer         figREAL
	integer         figQSTR
	integer         kvfile
	integer         kvgQSTR
	integer         kvgREAL
	integer         kvgINT
	integer         kvgLOG
	integer         lenc
C       ---------------------
C       Common Block
	include 'spgraf.inc'
C       ---------------------
C       Executable Code
c       get the canonical file name
	string = spcfile
c       open the file
	call kvverb(.true.)
	ts = kvfile(string)
	if (ts .ne. SUCCESS) return
	write(STDOUT,*) 'Reading spectrograph file ',
     &  spcfile(:lenc(spcfile))
c       ----------------------------------------------------------------
c       there should be only one occurrance of each of these cards
	call kvrew(.true.)
	ts =     kvgQSTR('TELESCOP=',1,0,telescop)
	ts =     kvgQSTR('INSTRUME=',1,0,instrume)
	ts =     kvgREAL('PRIMDIAM=',1,0,primdiam)
	ts =     kvgREAL('FOCSCALE=',1,0,focscale)
	ts =     kvgREAL('COLFOCLN=',1,0,colfocln)
	collmeth = i_COLFOCLN
	ts =     kvgREAL('CAMFOCLN=',1,0,camfocln)
	ts =     kvgREAL('ECTHETAD=',1,0,ecthetad)
	ectheta = ecthetad * radPERdeg
	ts =     kvgREAL('ECDELTAD=',1,0,ecdeltad)
	ecdelta = ecdeltad * radPERdeg
	ts =     kvgINT ('NXDGRAT =',1,0,nxdgrat)
	ts =     kvgINT ('NXDPRISM=',1,0,nxdprism)
c       ----------------------------------------------------------------
c       these cards are paired, either one may be set
	call kvverb(.false.)
	ts =     kvgREAL('ECSIGMA =',1,0,ecsigma)
	ts1=     kvgREAL('ECSIGMAI=',1,0,ecsigmai)
	call kvverb(.true.)
	if (ts1 .eq. SUCCESS) then
c           we give priority to the existence of ecsigmai
	    ecsigma = umPERmm / ecsigmai
	elseif (ts .eq. SUCCESS) then
	    ecsigmai = umPERmm / ecsigma
	else
	    write(STDERR,*)
     &      'readspc:  Echelle groove spacing was not given.'
	endif
c       ----------------------------------------------------------------
	ts =     kvgQSTR('XDISPERS=',1,0,xdispers)
c       ----------------------------------------------------------------
	if (nxdgrat .gt. MAXGRATX) then
	    write(STDERR,*)
     &      'readspc:  Too many cross gratings, MAXGRATX is ', MAXGRATX
	elseif (nxdgrat .gt. 0) then
	    ts = kvgREAL('XDDELTAD=',1,0,xddeltad)
	    ts = kvgREAL('XDALFBET=',1,0,xdalfbet)
c           these cards are paired, either one may be set
	    call kvverb(.false.)
	    ts = kvgREAL('XDSIGMA =',1,0,xdsigma)
	    ts1= kvgREAL('XDSIGMAI=',1,0,xdsigmai)
	    call kvverb(.true.)
	    if (ts1 .eq. SUCCESS) then
c               we give priority to the existence of xdsigmai
		xdsigma = umPERmm / xdsigmai
	    elseif (ts .eq. SUCCESS) then
		xdsigmai = umPERmm / xdsigma
	    else
		write(STDERR,*)
     &          'readspc:  Cross disperser spacing was not given.'
	    endif
	endif
c       ----------------------------------------------------------------
	if (nxdprism .gt. MAXPRISM) then
	    write(STDERR,*)
     &      'readspc:  Too many prisms, MAXPRISM is ', MAXPRISM
	elseif (nxdprism .gt. 0) then
	    ts = kvgREAL('PRANGIND=',1,0,prangind)
c           --------
c           get the apex angles of the prisms
	    ts = figREAL('PRAPEX',nxdprism,prapexd(0))
	    dum = 0.
	    do 1000 i=1,nxdprism
	      dum = dum + prapexd(i)
1000        continue
	    if (nxdprism .gt. 1) prapexd(0) = dum/nxdprism
c           --------
c           get the angle between adjacent faces of adjacent prisms
	    ts = figREAL('PRFACE',nxdprism-1,prface(1))
	    dum = 0.
	    do 2000 i=2,nxdprism
	      dum = dum + prface(i)
2000        continue
	    if (nxdprism .gt. 2) prface(1) = dum/(nxdprism-1)
c           --------
c           get the type of glass used in each prism
	    ts = figQSTR('PRGLAS',nxdprism,prglas(0))
	endif
c       ----------------------------------------------------------------
	ts =     kvgLOG ('STDECHFH=',1,0,stdechfh)
	ts =     kvgREAL('FPROTANG=',1,0,fprotang)
c       ----------------------------------------------------------------
c       close the file
	call kvverb(.false.)
	ts = kvfile(' ')
C       ---------------------
	return
	end
C==============================================================================
C==============================================================================
	subroutine echtran
C       ---------------------
c       Calculate the transform matrix from echelle coords to display coords
c       and vice versa.
c       Alas, all communication is done thru the spgraf common block.
c       --------
c       NOTE CAREFULLY:
c       Display coords and echelle coords are defined
c       by viewers on the same sides of the focal plane.  Their
c       +x and +y coordinate directions may be coincident, depending on the
c       value of the angle FPROTANG.  Thus, their handednesses will be the
c       same.
c       --------
c       In the case where STDVIDV=F, the +y coordinate directions of the
c       2 systems will be opposite, and the handedness of the two systems
c       will be the same.
c       --------
c       In the case where STDVIDH=F, the +x coordinate directions of the
c       2 systems will be opposite, and the handedness of the two systems
c       will be the same.
c       --------
c       In the case where STDVIDR=F, there will be a 90degree rotation
c       between the two coordinate systems, and their handednesses
c       will be opposite because they are still defined from opposite
c       sides of the focal plane.
c       --------
c       In the case where more than one of the standard video flags is
c       false, further flips and rotations will be in effect.
C       ---------------------
	IMPLICIT_NONE
C       ---------------------
C       Parameters
#       include "echelle.par"
#       include "include/units.par"
C       Internal Variables
c                       trig functions
	real            sinrot, cosrot
C       ---------------------
C       Common Block
	include 'setup.inc'
	include 'spgraf.inc'
C       ---------------------
C       Executable Code
	sinrot = fprotang * radPERdeg
	cosrot = cos( sinrot )
	sinrot = sin( sinrot )
c       --------------------------------------------------------------
c       Note that both the Echelle coords and the Display coords are
c       measured in millimeters.  Also note that their origins are
c       defined such that they coincide.
c       --------------------------------------------------------------
c       calculate the transformation from Echelle coords to Display coords
	tde(Xp, To, E2D) = 0.
	tde(Yp, To, E2D) = 0.
	tde(Xp, Xo, E2D) =  cosrot
	tde(Yp, Xo, E2D) =  sinrot
	tde(Xp, Yo, E2D) = -sinrot
	tde(Yp, Yo, E2D) =  cosrot
c       --------------------------------------------------------------
c       calculate the transformation from Display coords to Echelle coords
c       This is used in the following fashion
c       ex = tde(Xp,Xo,D2E) * dx + tde(Xp,Yo,D2E) * dy + tde(Xp,To,D2E)
c       ey = tde(Yp,Xo,M2E) * dx + tde(Yp,Yo,D2E) * dy + tde(Yp,To,D2E)
	tde(Xp, To, D2E) = 0.
	tde(Yp, To, D2E) = 0.
	tde(Xp, Xo, D2E) =  cosrot
	tde(Yp, Xo, D2E) = -sinrot
	tde(Xp, Yo, D2E) =  sinrot
	tde(Yp, Yo, D2E) =  cosrot
c       --------------------------------------------------------------
	if (.not. stdechfh) then
c           flip y coordinates of Echelle coords
	    tde(Xp, Yo, E2D) = -tde(Xp, Yo, E2D)
	    tde(Yp, Yo, E2D) = -tde(Yp, Yo, E2D)
	    tde(Yp, Xo, D2E) = -tde(Yp, Xo, D2E)
	    tde(Yp, Yo, D2E) = -tde(Yp, Yo, D2E)
	endif
C       ---------------------
	return
	end
C==============================================================================
C==============================================================================
	subroutine translate(xba, yba, tmat, num)
C       ---------------------
c       apply a general 2-d linear transformation matrix to some coordinates
C       ---------------------
	IMPLICIT_NONE
C       ---------------------
C       Parameters
#       include "echelle.par"
C       ---------------------
C       External Variables
c                       how many are there?
	integer         num
c                       input and output value of x coordinate
	real            xba(1:num)
c                       input and output value of y coordinate
	real            yba(1:num)
c                       transformation matrix
	real            tmat(Xp:Yp, Xo:To)
C       ---------------------
C       Internal Variables
c                       values after the transformation
	real            xaft, yaft
c                       loop
	integer         i
C       ---------------------
C       Executable Code
	do 100 i=1,num
	  xaft = tmat(Xp,Xo)*xba(i) + tmat(Xp,Yo)*yba(i) + tmat(Xp,To)
	  yaft = tmat(Yp,Xo)*xba(i) + tmat(Yp,Yo)*yba(i) + tmat(Yp,To)
	  xba(i) = xaft
	  yba(i) = yaft
100     continue
C       ---------------------
	return
	end
C==============================================================================
C==============================================================================
	subroutine distran
C       ---------------------
c       Calculate the transform matrix from mosaic coords to display coords
c       and vice versa.
c       This calculation is made repeatedly every time the user "drags" the
c       detector around the picture of the echelle format.
c       Alas, all communication is done thru the spgraf common block.
c       --------
c       NOTE CAREFULLY:  In the case where STDVIDV=T, STDVIDH=T, and
c       STDVIDR=T, Display coords and mosaic coords are defined
c       by viewers on opposite sides of the focal plane.  Their
c       +x and +y coordinate directions will be coincident, but because they
c       are viewed from opposite sides, that will make their handednesses
c       opposite.
c       --------
c       In the case where STDVIDV=F, the +y coordinate directions of the
c       2 systems will be opposite, and the handedness of the two systems
c       will be the same.
c       --------
c       In the case where STDVIDH=F, the +x coordinate directions of the
c       2 systems will be opposite, and the handedness of the two systems
c       will be the same.
c       --------
c       In the case where STDVIDR=F, there will be a 90degree rotation
c       between the two coordinate systems, and their handednesses
c       will be opposite because they are still defined from opposite
c       sides of the focal plane.
c       --------
c       In the case where more than one of the standard video flags is
c       false, further flips and rotations will be in effect.
C       ---------------------
	IMPLICIT_NONE
C       ---------------------
C       Parameters
#       include "echelle.par"
#       include "include/units.par"
C       ---------------------
C       Internal Variables
c                       trig functions
	real            sinrot, cosrot
C       ---------------------
C       Common Block
	include 'setup.inc'
	include 'spgraf.inc'
C       ---------------------
C       Executable Code
	sinrot = 0
	cosrot = cos( sinrot )
	sinrot = sin( sinrot )
c       --------------------------------------------------------------
c       Note that posmos is expressed in Display coordinates
c       Note also that we only ever use the position numbered "0".
c       --------------------------------------------------------------
c       calculate the transformation from Display coords to Mosaic coords
	tmd(Xp, To, D2M) = -(posmos(Xp,0)*cosrot + posmos(Yp,0)*sinrot)
	tmd(Yp, To, D2M) =   posmos(Xp,0)*sinrot - posmos(Yp,0)*cosrot
	tmd(Xp, Xo, D2M) =  cosrot
	tmd(Yp, Xo, D2M) = -sinrot
	tmd(Xp, Yo, D2M) =  sinrot
	tmd(Yp, Yo, D2M) =  cosrot
c       --------------------------------------------------------------
c       calculate the transformation from Mosaic coords to Display coords
c       This is used in the following fashion
c       dx = (tmd(Xp,Xo,M2D) * mx + tmd(Xp,Yo,M2D) * my) * mmPERm +
c    &  posmos(Xp,imos)
c       dy = (tmd(Yp,Xo,M2D) * mx + tmd(Yp,Yo,M2D) * my) * mmPERm +
c    &  posmos(Yp,imos)
	tmd(Xp, To, M2D) = posmos(Xp,0)
	tmd(Yp, To, M2D) = posmos(Yp,0)
	tmd(Xp, Xo, M2D) =  cosrot
	tmd(Yp, Xo, M2D) =  sinrot
	tmd(Xp, Yo, M2D) = -sinrot
	tmd(Yp, Yo, M2D) =  cosrot
C       ---------------------
	return
	end
C==============================================================================
C==============================================================================
