#include "include/port.h"
C==============================================================================
C==============================================================================
	subroutine ratwindow()
c       Given the physical/electrical properties of the CCD and readout
c       electronics along with the physical readout window and binning
c       desired by the user calculate the value of the WINDOW cards as
c       needed by the Keck instrument control system.
c       This is a rather lengthy process which involves the many
c       definitions of quantities found in the ratwin common block.
	IMPLICIT_NONE
C       Parameters
#       include "include/stdio.par"
#       include "echelle.par"
C       Common Blocks
	include 'setup.inc'
	include 'detmos.inc'
	include 'ratwin.cmn'
C       Internal Variables
c                       we loop over the 2 dimensions of the CCD
	integer         i
c                       how many imaging pixels are there to be
c                       read thru each amplifier in this readout mode
	integer         npread
C       Data
	data    nfcando         /.true./
	data    nfrecord        /.true./
	data    okmixed         /.true./
	data    dpost           / 30, 30 /
	data    dpre            / 10, 10 /
	data    namp            /  2,  2 /
C       Executable Code
c       check to see if the serial readout characteristics prevent us
c       from being able to do what was requested
	do 9000 i=Xp,Yp
c         how many pixels to be read thru each amp?
	  if (namp(i) .eq. 1) then
	      npread = npix(0,i)
	  elseif (namp(i) .eq. 2) then
c             note that we assume here that the CCD design is
c             symmetrical and the number of pixels is even.
	      npread = npix(0,i) / 2
	  endif
c         find the fiducial pixel
	  if ((i .eq. Xp) .and. (.not. nfcando)) then
	      if (namp(i) .eq. 1) then
c                 we can satisfy binning requirements by choice of fidpix
		  fidpix(i) = mod(phprepix(0) , binning(0,i))
		  if (fidpix(i) .ne. 0)
     &            fidpix(i) = binning(0,i) - fidpix(i)
	      elseif (namp(i) .eq. 2) then
c                 see if we can satisfy the binning requirements
		  if ( mod( npread + phprepix(0), binning(0,i))
     &            .ne. 0 ) then
c                     we cannot satisfy binning requirements
c                     so we punt by resetting this binning to 1
		      binning(0,i) = 1
		      fidpix(i) = 0
		  else
c                     we got lucky and we can do this
		      fidpix(i) = mod(phprepix(0) , binning(0,i))
		      if (fidpix(i) .ne. 0)
     &                fidpix(i) = binning(0,i) - fidpix(i)
		  endif
	      endif
	  else /*  ( i .eq. Yp .or. nfcando ) */
c             we really do not care much where the fiducial pixel is
c             Note that here we simply proceed under the assumption that
c             we are doing a multiple amplifier readout, and also that the
c             CCD in question has an even number of imaging pixels in the
c             serial direction.  This code could be generalized to allow
c             for a different fiducial pixel in cases where we know that
c             we will be using only one amplifier.
c             Thus, npostmix = 0 and ipostmix = 0
c             by the above reasoning, the fiducial pixel is
	      fidpix(i) = npix(0,i) / 2
	  endif
c         find the fiducial offset
	  fidoff(i) = mod(fidpix(i),binning(0,i))
c         find the number of postscan mixed pixels
	  if ( mod(npread,binning(0,i)) .eq. fidoff(i) ) then
	      npostmix(i) = 0
	      ipostmix(i) = 0
	  else
	      npostmix(i) = 1
	      ipostmix(i) = mod(binning(0,i) + mod(npread,binning(0,i))
     &        - fidoff(i), binning(0,i))
	  endif
c         the number of pure postscan logical pixels is easy
	  npostlog(i) = (dpost(i) + binning(0,i) - 1)/binning(0,i)
	  if (i .eq. Xp) then
	      ndrop(i) = mod(fidoff(i) + phprepix(0), binning(0,i))
	      if (fidoff(i) .eq. 0) then
		  nprelog(i) = phprepix(0) / binning(0,i)
	      else
		  nprelog(i) =
     &            (fidoff(i) + phprepix(0) - binning(0,i))/binning(0,i)
		  nprelog(i) = max(nprelog(i),0)
	      endif
	  else /* (i .eq. Yp) */
	      ndrop(i) = 0
	      nprelog(i) = (dpre(i) + binning(0,i) - 1)/binning(0,i)
	  endif
c         prescan mixed pixels are also easy
	  if (fidoff(i) .eq. 0) then
	      npremix(i) = 0
	      ipremix(i) = 0
	  else
	      npremix(i) = 1
	      ipremix(i) = fidoff(i)
	  endif
c         find the total number of logical pure image pixels that will be read
c         This assumes we bother to read out the entire image area.
	  nimage(i) = npread - (ipremix(i) + ipostmix(i))
	  if (mod(nimage(i),binning(0,i)) .ne. 0)
     &    write(STDERR,*) 'impossible nimage value', i
	  nimage(i) = nimage(i) / binning(0,i)
c         find the boundaries of the beginning of the desired readout
c         region in physical pixels
	  if (fidoff(i) .le. window(i,1)) then
c             easy case where the readout window is entirely real pixels
	      pindow(i) = ((window(i,1)-fidoff(i))/binning(0,i)) *
     &        binning(0,i) + fidoff(i)
	  else
c             harder case where we have to worry about mixed pixels
	      if (okmixed) then
		  pindow(i) = fidoff(i) - binning(0,i)
c                 physical window cannot extend off the shift register
c                 in the serial direction if we cannot do non-full
		  if ((i .eq. Xp) .and. (.not. nfcando) .and.
     &            ((pindow(i)+phprepix(0)) .lt. 0) )
     &            pindow(i) = pindow(i) + binning(0,i)
	      else
		  pindow(i) = fidoff(i)
	      endif
	  endif
c         find the boundaries of the end of the desired readout
c         region in physical pixels
	  pindow(i+4) = ((window(i,1)+window(i+2,1)-1 -fidoff(i)) /
     &    binning(0,i)) * binning(0,i) + binning(0,i) - 1 +
     &    fidoff(i)
	  if (pindow(i+4) .ge. npix(0,i)) then
c             end of region extends past the imaging pixels
	      if (.not. okmixed) then
		  pindow(i+4) = pindow(i+4) - binning(0,i)
	      endif
	  endif
c         having the beginning and the end of the desired readout region
c         in physical pixels, calculate the size of the region
	  pindow(i+2) = pindow(i+4) - pindow(i) + 1
c         finally, now, calculate the values needed for KICS in the
c         WINDOW card.
c         The size of the logical window is easy
	  if (mod(pindow(i+2),binning(0,i)) .ne. 0)
     &    write(STDERR,*) 'impossible pindow size', i
	  kindow(i+2) = pindow(i+2) / binning(0,i)
c         **************************************************************
c         NOTE:  the current definition of the KICS WINDOW card is not
c         well equipped to handle the kinds of complexities envisioned
c         in the preceding code.  For that reason, it is not clear
c         just what exactly the WINDOW card should be for arbitrary
c         values of the desired readout window and BINNING.  Thus,
c         the following code may not generate correct results for
c         some cases where the current readout code is incapable of
c         doing the desired task.
c
c         The location of the start of the logical window requires
c         carefully adding up preceding pixels
	  if (i .eq. Yp) then
c             start with those pre imaging rows
	      kindow(i) = nprelog(i)
	      if (pindow(i) .ge. 0) then
c                 add in any mixed prescan/image rows and pure image rows
c                 which come before the desired readout region
		  kindow(i) = kindow(i) + npremix(i) +
     &            ( pindow(i) / binning(0,i) )
	      endif
	  else /* (i .eq. Xp) */
	      if ((pindow(i) + phprepix(0)) .ge. 0) then
		  kindow(i) =
     &            ( pindow(i) + phprepix(0) ) / binning(0,i)
		  if (nfcando .and. nfrecord .and.
     &            (ndrop(i) .gt. 0)) kindow(i) = kindow(i) + 1
	      else
		  kindow(i) = 0
	      endif
	  endif
c         **************************************************************
	  write(STDERR,*) dpre(i),dpost(i),namp(i)
	  write(STDERR,*) fidpix(i), fidoff(i)
	  write(STDERR,*) ndrop(i),nprelog(i),npremix(i),ipremix(i)
	  write(STDERR,*) nimage(i),npostmix(i),ipostmix(i),npostlog(i)
9000    continue
	write(STDERR,*) pindow
	write(STDERR,*) kindow
	return
	end
C==============================================================================
C==============================================================================
