#include "include/port.h"
C==============================================================================
C==============================================================================
	integer function readset(setfile,numd,defdir)
C       ---------------------
c       Open a file which presumably contains FITS-card-like lines
c       and read the configuration of a spectrograph from that file.
C       ---------------------
	IMPLICIT_NONE
C       ---------------------
C       Parameters
#       include "include/stdio.par"
#       include "token/keyval.par"
#       include "echelle.par"
C       ---------------------
C       External Variables
c                       the almost-FITS file containing the detector info
c                       which we will try to open and try to read
	character*(*)   setfile
c                       number of directories to search
	integer         numd
c                       default directory to search
	character*(*)   defdir(numd)
C       ---------------------
C       Internal Variables
c                       return value
	integer         ts, ts1, ts2
c                       loop
	integer         i, j
c                       a place to keep a dynamically created FITS keyword
	character*9     fitskey(3)
c                       a place to keep a dynamically created format string
	character*4     ifmt
c                       a place to read Conrad FITS string arrays
	character*132   string
c                       iostatus of an internal read
	integer         ios
c                       temporary window limits
	integer         winlim(Xo:dY)
c                       take default action
	logical         default
C       ---------------------
C       External Function
	integer         kvfile
	integer         kvgQSTR
	integer         kvgINT
	integer         kvgREAL
	integer         kvgDBLE
	integer         readdet, readspc
	integer         lenc
C       ---------------------
C       Common Block
	include 'lam.cmn'
	include 'setup.inc'
	include 'detmos.inc'
C       ---------------------
C       Executable Code
c       get the canonical file name
	string = setfile
c       open the file
	call kvverb(.true.)
	ts = kvfile(string)
	if (ts .ne. SUCCESS) return
	call kvrew(.true.)
	call kvverb(.false.)
	write(STDOUT,*) 'Reading setup file ',setfile(:lenc(setfile))
c       ----------------------------------------------------------------
c       variables associated with the spectrograph hardware
c       there need not be any occurrance of these cards
c       if they do not exist, reasonable defaults will be used
c       ----------------------------------------------------------------
	ts1 =    kvgQSTR('DETFILNM=',1,0,detfilnm)
	ts2 =    kvgQSTR('SPCFILNM=',1,0,spcfilnm)
	if (ts1 .eq. SUCCESS) then
	    call canonfile(numd,defdir,detfilnm)
	    if (detfilnm .ne. ' ') i = readdet(detfilnm)
	endif
	if (ts2 .eq. SUCCESS) then
	    call canonfile(numd,defdir,spcfilnm)
	    if (spcfilnm .ne. ' ') i = readspc(spcfilnm)
	endif
c       if either of the above was read, we have to reopen the setup file
	if ((ts1 .eq. SUCCESS) .or. (ts2 .eq. SUCCESS))
     &  ts = kvfile(string)
c       ----------------------------------------------------------------
	ts =     kvgQSTR('SETUP   =',1,0,setup)
	ts =     kvgQSTR('OBSERVER=',1,0,observer)
	ts =     kvgREAL('ECHANGL =',1,0,ecangle)
	ts =     kvgREAL('XDANGL  =',1,0,xdangle)
c       the next one is for the Hamilton Echelle spectrograph
	ts =     kvgREAL('HAMHGT  =',1,0,hamhgt)
c       ----------------------------------------------------------------
	ts =     kvgINT('DECKRAW =' ,1,0,deckraw)
	ts =     kvgREAL('DECKPOS =',1,0,deckpos)
	ts =     kvgQSTR('DECKNAME=',1,0,decknnam)
c       ----------------
c       look for the decker height in meters, then arcsec, then pixels
c       these are ordered from least relevant to most relevant, i.e. the
c       last one is most canonical
	if (     kvgREAL('DECKPIX =',1,0,deckpix)  .eq. SUCCESS)
     &  deckmeth = i_DECKPIX
	if (     kvgREAL('DECKSIZE=',1,0,decksize) .eq. SUCCESS)
     &  deckmeth = i_DECKSIZE
	if (     kvgREAL('DECKHGT =',1,0,deckhgt)  .eq. SUCCESS)
     &  deckmeth = i_DECKHGT
c       ----------------
c       look for the precise description of all the holes in the decker
	ts =     kvgQSTR('DECKSPEC=',1,0,string)
	if (ts .eq. SUCCESS) then
c           add code do interpret the string
c           need to read ndeckap and deckspec(1:2*ndeckap)
	    read(string,*,iostat=ios) ndeckap
	    if (ios .eq. SUCCESS) then
		read(string,*,iostat=ios) ndeckap,
     &          (deckspec(i),i=1,2*ndeckap)
	    endif
	    if (ios .ne. SUCCESS) then
		write(STDERR,*) 'readset:  bad format in DECKSPEC card'
		write(STDERR,*) 'string value was ', string
	    endif
	endif
c       ----------------------------------------------------------------
c       look for the slit width in meters, arcsec, pixels, velocity, MEU
c       these are ordered from least relevant to most relevant, i.e. the
c       last one is most canonical
	if (     kvgREAL('SLITVEL =',1,0,slitvel)  .eq. SUCCESS)
     &  slitmeth = i_SLITVEL
	if (     kvgREAL('SLITPIX =',1,0,slitpix)  .eq. SUCCESS)
     &  slitmeth = i_SLITPIX
	if (     kvgREAL('SLITSIZE=',1,0,slitsize) .eq. SUCCESS)
     &  slitmeth = i_SLITSIZE
	if (     kvgINT ('SLITRAW =',1,0,slitraw)  .eq. SUCCESS)
     &  slitmeth = i_SLITRAW
	if (     kvgREAL('SLITWID =',1,0,slitwid)  .eq. SUCCESS)
     &  slitmeth = i_SLITWID
c       ----------------------------------------------------------------
	ts =     kvgINT('FIL1POS =' ,1,0,filter)
	ts =     kvgINT('FIL2POS =' ,1,0,filter2)
	ts =     kvgQSTR('FIL1NAME=',1,0,filname)
	ts =     kvgQSTR('FIL2NAME=',1,0,fil2name)
c       ----------------------------------------------------------------
	ts =     kvgQSTR('COLL    =',1,0,coll)
	ts =     kvgREAL('COFOCUS =',1,0,cofocus)
	ts =     kvgQSTR('CAMERA  =',1,0,camera)
	ts =     kvgREAL('CAFOCUS =',1,0,cafocus)
c       ----------------------------------------------------------------
c       variables associated with the detector mosaic
c       ----------------------------------------------------------------
	ifmt = '(i1)'
c       there need not be any occurrence of these cards
	ts =     kvgQSTR('BINNING =',1,0,string)
	if (ts .eq. SUCCESS) read(string,*,iostat=ts) xbin(0),ybin(0)
	if (ts .ne. SUCCESS) then
	    xbin(0) = 1
	    ybin(0) = 1
	endif
	ts =     kvgQSTR('DWINDOW =',1,0,string)
	if (ts .eq. SUCCESS) read(string,*,iostat=ios)
     &  (window(j,0),j=Xo,dY)
	if (ts .ne. SUCCESS) then
	    window(Xo,0) = 0
	    window(Yo,0) = 0
	    window(dX,0) = npix(0,Xo)
	    window(dY,0) = npix(0,Yo)
	endif
	do 1000 i=1,MAXDETMOS
c         set defaults
	  xbin(i) = xbin(0)
	  ybin(i) = ybin(0)
	  window(Xo,i) = 0
	  window(Yo,i) = 0
	  window(dX,i) = npix(i,Xo)
	  window(dY,i) = npix(i,Yo)
1000    continue
	fitskey(1) = 'BINNING ='
	fitskey(2) = 'DWINDOW ='
	do 2000 i=1,MAXDETMOS
c         create the format which will be used for writing keyword to read
	  write(ifmt(3:3),'(i1)') int(log10(real(i))+1)
c         get BINNINGn for this chip
	  write(fitskey(1)(8:8),ifmt) i
	  ts =   kvgQSTR(fitskey(1) ,1,0,string)
	  if (ts .eq. SUCCESS) read(string,*,iostat=ts) xbin(i),ybin(i)
	  if (ts .ne. SUCCESS) then
	      xbin(i) = xbin(0)
	      ybin(i) = ybin(0)
	  endif
c         get WINDOWn for this chip
	  write(fitskey(2)(7:8),ifmt) i
	  ts2 =  kvgQSTR(fitskey(3) ,1,0,string)
	  default = ts2 .ne. SUCCESS
	  if (.not. default) then
	      read(string,*,iostat=ios) (window(j,i),j=Xo,dY)
	      default = ios .ne. SUCCESS
	      if (default) then
		  write(STDERR,*) 'readset:  bad format DWINDOW card'
		  write(STDERR,*) 'string value was ', string
	      endif
	  endif
	  if (default) then
	      window(Xo,i) = 0
	      window(Yo,i) = 0
	      window(dX,i) = npix(i,Xo)
	      window(dY,i) = npix(i,Yo)
	  else
c             clip all corners to limits of chip
	      winlim(Xo) = max(min(window(Xp,i),
     &        window(Xp,i)+window(dX,i)), 0)
	      winlim(dX) = min(max(window(Xp,i),
     &        window(Xp,i)+window(dX,i)), npix(i,Xp))
	      winlim(Yp) = max(min(window(Yp,i),
     &        window(Yp,i)+window(dY,i)), 0)
	      winlim(dY) = min(max(window(Yp,i),
     &        window(Yp,i)+window(dY,i)), npix(i,Yp))
c             find out new extent of the window limits
	      window(dX,i) = (winlim(dX)-winlim(Xp))
	      window(dY,i) = (winlim(dY)-winlim(Yp))
	      window(Xp,i) = (winlim(Xp)           )
	      window(Yp,i) = (winlim(Yp)           )
c             if (window(dX,i) .le. 0 .or. window(dY,i) .le. 0)
c    &        write(STDERR,*) 'readset:  silly limits in DWINDOW card'
	  endif
c         if we miss a chip entirely, assume there are no more
	  if (.not. ((ts  .eq. SUCCESS)
     &    .or.       (ts2 .eq. SUCCESS))) goto 2001
2000    continue
2001    continue
c       ----------------------------------------------------------------
c       variables used only in the instrument simulator
c       ----------------------------------------------------------------
c       there need not be any occurrance of these cards
	ts =     kvgQSTR('WAVEFILE=',1,0,wavefile)
	ts =     kvgDBLE('RADVEL  =',1,0,redshift)
	if (ts .ne. SUCCESS) then
c           try looking for RADVELZ
	    ts = kvgDBLE('RADVELZ =',1,0,redshift)
	    if (ts .ne. SUCCESS) redshift = 0.
	    radvel = cLIGHT *
     &      (((1+redshift)**2 - 1) / ((1+redshift)**2 + 1))
	else
c           we did get the velocity
	    radvel = redshift
c           convert velocity to redshift
c           this ignores the transverse redshift of special relativity
	    redshift = redshift / cLIGHT
	    redshift = sqrt((1 + redshift) / (1 - redshift)) - 1
	endif
	radvelz = redshift
c       ----------------------------------------------------------------
c       there should be one occurrance of each of these cards
	call kvverb(.true.)
	ts =     kvgREAL('WAVLMAX =',1,0,wl)
	ts =     kvgREAL('WAVLMIN =',1,0,ws)
	ts =     kvgINT ('XDORDER =',1,0,mc)
c       ----------------------------------------------------------------
c       close the file
	call kvverb(.false.)
	ts = kvfile(' ')
C       ---------------------
	return
	end
C==============================================================================
C==============================================================================
