C       Common block describing all the changeable settings of a
C       spectrograph/detector combination.
c
c
c
c
c
c
c
c
COMMENT         ========================================================
COMMENT         This file is intended to represent the information that
COMMENT         an observer would store in advance and bring to the
COMMENT         mountain.  This information contains all of the setup
COMMENT         information needed to describe the state of an Echelle
COMMENT         spectrograph and its detector.
COMMENT         ========================================================
COMMENT         The observer can associate a pet name with a given setup.
COMMENT         This name is used to store and retrive the entire setup.
	character       setup*(MAXFITSTR)
COMMENT         --------------------------------------------------------
COMMENT         The Echelle simulator program refers to files which define
COMMENT         the layout of the detector and the spectrograph.  Obviously
COMMENT         a given "setup" is intended to be used with a particular
COMMENT         detector and a particular spectrograph.  These are the names
COMMENT         of those files.
	character       detfilnm*(MAXFITSTR)
	character       spcfilnm*(MAXFITSTR)
COMMENT         --------------------------------------------------------
COMMENT         The observer can give a name to be put in the FITS header.
	character       observer*(MAXFITSTR)
COMMENT         --------------------------------------------------------
COMMENT         The zero point for echelle rotation angle is that angle
COMMENT         at which the incoming and outgoing light are "on blaze".
COMMENT         The pos/neg direction is TBD.  [degree]
	real            ecangle
COMMENT         The zero point for cross disp. rotation angle is that angle
COMMENT         at which the incoming and outgoing light are "on blaze".
COMMENT         The pos/neg direction is TBD.  [degree]
	real            xdangle
COMMENT         With a prism cross disperser there is no XDANGLE.
COMMENT         The Hamilton Echelle spectrograph can move the detector along
COMMENT         the direction of cross dispersion to see different orders.
c                       detector height [machine encoder units]
	real            hamhgt
COMMENT         With the Hamilton Echelle spectrograph the user sees the
COMMENT         echelle tilt angle in encoder units.
	integer         ecangraw
c               With a given value of xdangle there is one Echelle order
c               nearest to the center of the focal plane.  This is its index.
	integer         icord
COMMENT         --------------------------------------------------------
COMMENT         The DECKER keyword is used differently by almost every
COMMENT         data acquisition system in existence.  In the case of
COMMENT         HIRES, DECKER is a string describing the particular metal
COMMENT         plate which is currently installed (by hand) in HIRES.
	character       decker*(MAXFITSTR)
COMMENT         The HIRES decker is a metal plate with machined notches (holes).
COMMENT         This metal plate is moved parallel to the dispersion
COMMENT         until one of the notches lines up with the slit.  The
COMMENT         FITS keywords to describe how far the metal plate has been
COMMENT         moved are DECKPOS and DECKRAW.  The values of these cards
COMMENT         will only be applicable to a particular decker plate.
COMMENT         The observer may associate a pet name with a particular
COMMENT         notch in the decker plate.
c                       decker plate position [MEU]
	integer         deckraw
c                       decker plate position [meter]
	real            deckpos
	character       decknnam*(MAXFITSTR)
COMMENT         The notch (hole) in the decker plate may actually be
COMMENT         several holes.  This will presumably prove useful after the
COMMENT         image rotator is installed.  An observer might want a
COMMENT         decker which rejected the light from the bright nucleus of
COMMENT         an otherwise faint galaxy.  As a result of the possibly
COMMENT         disconnected nature of the decker notch, the decker height
COMMENT         is defined to be the total vertical extent of the holes
COMMENT         in the decker plate.  The next FITS keywords use this height.
COMMENT         Note that if DECKPIX is used the software will have to
COMMENT         make the assumption that pixels are square and all
COMMENT         detectors in the mosaic are identical.
c                       total decker height [meter]
	real            deckhgt
c                       total decker height [arcsec]
	real            decksize
c                       total decker height [pixel]
	real            deckpix
COMMENT         To allow for the documentation of a decker notch (hole) with
COMMENT         multiple apertures, the following keyword has been developed.
COMMENT         The DECKSPEC keyword is a Keck FITS string array.  The
COMMENT         values in the array are
COMMENT         1) an integer describing how many holes there are
COMMENT         2) pairs of real numbers giving the position of top and bottom
COMMENT            of each hole in meters.  The origin is arbitrary.
	integer         ndeckap
	real            deckspec(MAXDECKAP)
COMMENT         Q: What would the observer specify in advance?
COMMENT         A: Most observers would probably want to specify DECKSIZE.
COMMENT         Q: What should the software do if only that is specified?
COMMENT         Q: Will it know how to pick the available decker height which
COMMENT         Q: most nearly matches?
COMMENT         --------------------------------------------------------
COMMENT         The slit width may be specified by the observer using any
COMMENT         of the following FITS cards.  These cards are listed in the
COMMENT         order they will be sought by the HIRES Instrument Simulator.
COMMENT         Note that if SLITPIX is used the software will have to
COMMENT         make the assumption that pixels are square and all
COMMENT         detectors in the mosaic are identical.
COMMENT         Until 1993 March 22, slitwid was [meter] which is in accord
COMMENT         with IAU and FITS recommendations; however, outcry from
COMMENT         astronomers forced it to be changed to [mm].
c                       slit width [mm]
	real            slitwid
c                       slit width [arcsec]
	real            slitsize
c                       slit width [pixel]
	real            slitpix
c                       slit width [meter/second]
	real            slitvel
c                       slit width [machine encoder units]
	integer         slitraw
COMMENT         --------------------------------------------------------
COMMENT         The observer probably will not specify the filter positions.
COMMENT         Rather, the observer will probably specify the filter names.
c                       filter1 position
	integer         filter
c                       filter2 position
	integer         filter2
c                       Observer's pet name for filter1
	character       filname*(MAXFITSTR)
c                       Observer's pet name for filter2
	character       fil2name*(MAXFITSTR)
COMMENT         --------------------------------------------------------
COMMENT         HIRES can select collimator under computer control.
	character       coll*(MAXFITSTR)
COMMENT         The origin for collimator focus is TBD.  [meter]
	real            cofocus
COMMENT         HIRES Camera must be changed manually by telescope techs.
COMMENT         At first light there will be only one camera available.
	character       camera*(MAXFITSTR)
COMMENT         The origin for camera focus is TBD.  [meter]
	real            cafocus
COMMENT         ========================================================
COMMENT         Presumably we may generalize these keywords to XBINn
COMMENT         and YBINn in the unlikely case that an observer wanted
COMMENT         to use different binnings to readout different chips in a
COMMENT         mosaic detector.
	integer         xbin(0:MAXDETMOS)
	integer         ybin(0:MAXDETMOS)
	integer         binning(0:MAXDETMOS,Xp:Yp)
	equivalence     (xbin(0),binning(0,Xp))
	equivalence     (ybin(0),binning(0,Yp))
COMMENT         --------------------------------------------------------
COMMENT         The FITS card WINDOWn refers to the region of readout for
COMMENT         the detector in position "n" of the mosaic.
COMMENT         There will be WINDOWn cards for 1 <= n <= NCHIPS .
COMMENT         The number of rows and columns to be readout may be
COMMENT         a ridiculously large number; the software will clip
COMMENT         the readout regions to be no larger than the detector.
	integer         window(Xo:dY,0:MAXDETMOS)
c
c
c
COMMENT         ========================================================
COMMENT         The following keywords have been invented for the sake of
COMMENT         the instrument simulator.  They are not presently
COMMENT         intended to be used in the actual Keck data acquisition
COMMENT         system.
c                       which method of specifying the slit was used?
c                       this should have a value such as
c                       i_SLITWID, i_SLITVEL, etc.
	integer         slitmeth
c                       which method of specifying the decker was used?
c                       this should have a value such as
c                       i_DECKHGT, i_DECKPIX, or i_DECKSIZE
	integer         deckmeth
c                       radial velocity of the object [m/s]
	real            radvel
c                       redshift of the object [delta lam/lam]
c                       see also the variable redshift in lam.cmn
	real            radvelz
c                       maximum wavelength to display [Aangstrom]
	real            wl
c                       minimum wavelength to display [Aangstrom]
	real            ws
c                       cross disperser order (if using grating)
	integer         mc
c                       range of interesting cross disperser orders
	integer         mci(2)
c                       name of file containing interesting line wavelengths
	character       wavefile*(MAXFITSTR)
C               --------------------------------------------------------
c                       incident angle of beam on cross-dispersing grating
c                       measured w.r.t. the grating normal [degrees]
c                       this variable is calculated, not input
	real            xdalphad
c                       minimum and maximum outgoing angles of beam from
c                       cross-dispersing grating [degrees]
c                       this variable is calculated, not input
	real            xdbetad(3)
COMMENT         ========================================================
	integer         i_SETUPNM
	integer         i_DETFILNM
	integer         i_SPCFILNM
	integer         i_OBSERVER
	integer         i_ECANGLE
	integer         i_XDANGLE
	integer         i_HAMHGT
	integer         i_ECANGRAW
	integer         i_DECKER
	integer         i_DECKRAW
	integer         i_DECKPOS
	integer         i_DECKNNAM
	integer         i_DECKHGT
	integer         i_DECKSIZE
	integer         i_DECKPIX
	integer         i_DECKSPEC
	integer         i_SLITWID
	integer         i_SLITSIZE
	integer         i_SLITPIX
	integer         i_SLITVEL
	integer         i_SLITRAW
	integer         i_FILTER
	integer         i_FILTER2
	integer         i_FILNAME
	integer         i_FIL2NAME
	integer         i_COLL
	integer         i_COFOCUS
	integer         i_CAMERA
	integer         i_CAFOCUS
	integer         i_XDORDER
	integer         i_XDALPHAD
	integer         i_XDBETAD
	integer         i_XBIN
	integer         i_YBIN
	integer         i_RADVEL
	integer         i_RADVELZ
	integer         i_WAVLMIN
	integer         i_WAVLMAX
	integer         i_WAVFILNM
c       Extreme care must be taken with these pointers to insure that
c       the data in the equivalence do not overlap.
	parameter       (i_SETUPNM  = 0)
	parameter       (i_DETFILNM = i_SETUPNM  + 1)
	parameter       (i_SPCFILNM = i_DETFILNM + 1)
	parameter       (i_OBSERVER = i_SPCFILNM + 1)
c
	parameter       (i_ECANGLE  = i_OBSERVER + 1)
	parameter       (i_XDANGLE  = i_ECANGLE  + 1)
	parameter       (i_HAMHGT   = i_XDANGLE  + 1)
	parameter       (i_ECANGRAW = i_HAMHGT   + 1)
c
	parameter       (i_DECKER   = i_ECANGRAW + 1)
	parameter       (i_DECKRAW  = i_DECKER   + 1)
	parameter       (i_DECKPOS  = i_DECKRAW  + 1)
	parameter       (i_DECKNNAM = i_DECKPOS  + 1)
	parameter       (i_DECKHGT  = i_DECKNNAM + 1)
	parameter       (i_DECKSIZE = i_DECKHGT  + 1)
	parameter       (i_DECKPIX  = i_DECKSIZE + 1)
	parameter       (i_DECKSPEC = i_DECKPIX  + 1)
c
	parameter       (i_SLITWID  = i_DECKSPEC + MAXDECKAP)
	parameter       (i_SLITSIZE = i_SLITWID  + 1)
	parameter       (i_SLITPIX  = i_SLITSIZE + 1)
	parameter       (i_SLITVEL  = i_SLITPIX  + 1)
	parameter       (i_SLITRAW  = i_SLITVEL  + 1)
c
	parameter       (i_FILTER   = i_SLITRAW  + 1)
	parameter       (i_FILTER2  = i_FILTER   + 1)
	parameter       (i_FILNAME  = i_FILTER2  + 1)
	parameter       (i_FIL2NAME = i_FILNAME  + 1)
c
	parameter       (i_COLL     = i_FIL2NAME + 1)
	parameter       (i_COFOCUS  = i_COLL     + 1)
	parameter       (i_CAMERA   = i_COFOCUS  + 1)
	parameter       (i_CAFOCUS  = i_CAMERA   + 1)
c
	parameter       (i_XDORDER  = i_CAFOCUS  + 1)
	parameter       (i_XDALPHAD = i_XDORDER  + 1)
	parameter       (i_XDBETAD  = i_XDALPHAD + 1)
c
	parameter       (i_XBIN     = i_XDBETAD  + 3)
	parameter       (i_YBIN     = i_XBIN     + 1 + MAXDETMOS)
	parameter       (i_RADVEL   = i_YBIN     + 1 + MAXDETMOS)
	parameter       (i_RADVELZ  = i_RADVEL   + 1)
	parameter       (i_WAVLMIN  = i_RADVELZ  + 1)
	parameter       (i_WAVLMAX  = i_WAVLMIN  + 1)
	parameter       (i_WAVFILNM = i_WAVLMAX  + 1)
C       ----------------------------------------------------------------
c                       these are not as crucial if they get set wrong
	integer         i_XBIN1
	integer         i_XBIN2
	integer         i_XBIN3
	integer         i_XBIN4
	parameter       (i_XBIN1    = i_XBIN   + 1)
	parameter       (i_XBIN2    = i_XBIN   + 2)
	parameter       (i_XBIN3    = i_XBIN   + 3)
	parameter       (i_XBIN4    = i_XBIN   + 4)
C       ----------------------------------------------------------------
c                       lower limit of array of menu entries
	integer         MINSDATA
	parameter       (MINSDATA = i_SETUPNM )
c                       upper limit of array of menu entries
	integer         MAXSDATA
	parameter       (MAXSDATA = i_WAVFILNM)
c                       bogus pointer value
	integer         NULSDATA
	parameter       (NULSDATA = MINSDATA - 1)
C       ================================================================
c                       equivalence to all of the above
	real            sdatar(MINSDATA:MAXSDATA)
	integer         sdatai(MINSDATA:MAXSDATA)
	equivalence     (sdatar, sdatai)
c                       some items come with accelerator keys
c                       that allow their values to be changed.
	character*1     sdataa(MINSDATA:MAXSDATA)
c                       pointers telling which ones to actually display
c                       and in what order to display them
c                       if the pointer value is NULSDATA, it is skipped
	integer         sdatap(MINSDATA:MAXSDATA)
c                       format to use to display each value
	character       sdataf(MINSDATA:MAXSDATA)*(LENFMT)
c                       character strings to use to display label strings
	character       sdatal(MINSDATA:MAXSDATA)*(LLABSTR)
c                       character strings to use to display value strings
	character   sdatav(SVOLD:SVNEW,MINSDATA:MAXSDATA)*(MAXFITSTR)
c                       positions of the labels in Normalized Device Coords
	real            sdatax(Xp:Yp,MINSDATA:MAXSDATA,2)
C       ----------------------------------------------------------------
	equivalence     (setup   , sdatav(SVNEW,i_SETUPNM ))
	equivalence     (detfilnm, sdatav(SVNEW,i_DETFILNM))
	equivalence     (spcfilnm, sdatav(SVNEW,i_SPCFILNM))
	equivalence     (observer, sdatav(SVNEW,i_OBSERVER))
	equivalence     (ecangle , sdatar(i_ECANGLE ))
	equivalence     (xdangle , sdatar(i_XDANGLE ))
	equivalence     (hamhgt  , sdatar(i_HAMHGT  ))
	equivalence     (ecangraw, sdatai(i_ECANGRAW))
	equivalence     (decker  , sdatav(SVNEW,i_DECKER  ))
	equivalence     (deckraw , sdatai(i_DECKRAW ))
	equivalence     (deckpos , sdatar(i_DECKPOS ))
	equivalence     (decknnam, sdatav(SVNEW,i_DECKNNAM))
	equivalence     (deckhgt , sdatar(i_DECKHGT ))
	equivalence     (decksize, sdatar(i_DECKSIZE))
	equivalence     (deckpix , sdatar(i_DECKPIX ))
	equivalence     (deckspec, sdatar(i_DECKSPEC))
	equivalence     (slitwid , sdatar(i_SLITWID ))
	equivalence     (slitsize, sdatar(i_SLITSIZE))
	equivalence     (slitpix , sdatar(i_SLITPIX ))
	equivalence     (slitvel , sdatar(i_SLITVEL ))
	equivalence     (slitraw , sdatai(i_SLITRAW ))
	equivalence     (filter  , sdatai(i_FILTER  ))
	equivalence     (filter2 , sdatai(i_FILTER2 ))
	equivalence     (filname , sdatav(SVNEW,i_FILNAME ))
	equivalence     (fil2name, sdatav(SVNEW,i_FIL2NAME))
	equivalence     (coll    , sdatav(SVNEW,i_COLL    ))
	equivalence     (cofocus , sdatar(i_COFOCUS ))
	equivalence     (camera  , sdatav(SVNEW,i_CAMERA  ))
	equivalence     (cafocus , sdatar(i_CAFOCUS ))
	equivalence     (mc      , sdatai(i_XDORDER ))
	equivalence     (xdalphad, sdatar(i_XDALPHAD))
	equivalence     (xdbetad , sdatar(i_XDBETAD ))
	equivalence     (xbin    , sdatai(i_XBIN    ))
	equivalence     (ybin    , sdatai(i_YBIN    ))
	equivalence     (radvel  , sdatar(i_RADVEL  ))
	equivalence     (radvelz , sdatar(i_RADVELZ ))
	equivalence     (wl      , sdatar(i_WAVLMAX ))
	equivalence     (ws      , sdatar(i_WAVLMIN ))
	equivalence     (wavefile, sdatav(SVNEW,i_WAVFILNM))
C       ----------------------------------------------------------------
c                       is this menu currently displayed?
	logical         sdataup
c                       is this menu currently initialized?
	logical         smeninit
c                       NDC y limits of this menu
	real            sminndc
	real            smaxndc
c                       bottom entry to be displayed
	integer         botsdata
COMMENT         ========================================================
c               transformation matrices which convert from echelle coords
c               to display coords and back again
	real    tde(Xp:Yp, Xo:To, D2E:E2D)
c               transformation matrices which convert from mosaic coords
c               to display coords and back again
	real    tmd(Xp:Yp, Xo:To, M2D:D2M)
C               --------------------------------------------------------
c               We can store up to MAXMOSPOS positions of the entire mosaic
c               as seen upon the Echelle Format.  What is stored here is
c               the location of the "center" of the mosaic, where the "center"
c               is defined by the positions in the DETPOS cards.
c               Note that these positions are only valid during a single
c               display of the Echelle format.  If the Echelle format is
c               refreshed, these coordinates become meaningless.  For this
c               reason, the actual position of the detector (as stored in
c               posmos(x:y,0)) is really stored in the spectrograph setup
c               structure as ecangle and either xdangle or hamhgt.
c                       x&y positions on echelle format of mosaic centers [mm]
	real            posmos(Xp:Yp, 0:MAXMOSPOS)
c                       number of mosaic positions that have been stored
	integer         nmos
C       ================================================================
c               This rectangle will be displayed when "dragging" the mosaic
c                       This contains the positions of the center and corners
c                       of a rectangle completely enclosing the mosaic [mm]
c                       The coordinate system here is that the defined "center"
c                       of the mosaic is at the origin.  See detector.F for
c                       more details.
	real            cormos(Xp:Yp,0:2)
COMMENT         ========================================================

	common  /setupcom/
     &  sdatar, sdatap, sminndc, smaxndc, sdatax,
     &  sdataup, smeninit,
     &  ndeckap, window, mci, botsdata,
     &  slitmeth, deckmeth, icord,
     &  tde, tmd, posmos, cormos,     nmos
	common  /setupstr/      sdataf, sdatal, sdatav, sdataa
