C       Common block carrying telescope, & spectrograph optical parameters
c
c
C               --------------------------------------------------------
c
c
c
c
c
c
c
c
C               --------------------------------------------------------
c       References below are to these two books:
c       Astronomical Optics; Daniel J. Schroeder; Academic Press, Inc.; 1987
c       Optics; E. Hecht & A. Zajac; Addison-Wesley; 1974
C               --------------------------------------------------------
C               Telescope and Camera Parameters
c                       character string describing this telescope
	character       telescop*(MAXFITSTR)
c                       character string describing this instrument
	character       instrume*(MAXFITSTR)
c                       diameter of telescope primary [meter]
	real            primdiam
c
c                       image scale at this focus of the telescope [arcsec/mm]
c
	real            focscale
c
c
c
c
c                       focal length of collimator [meter] ([m/radian])
	real            colfocln
c                       diameter of spectrograph collimated beam [meter]
c                       this is a derived quantity in this code
	real            colldiam
c                       spectrograph camera focal length [meter] ([m/radian])
	real            camfocln
C               --------------------------------------------------------
C               Echelle Grating Parameters
c                       groove spacing on echelle grating [micrometer/groove]
	real            ecsigma
c                       groove frequency on echelle grating [groove/mm]
	real            ecsigmai
c                       half angle (theta) [degree], [radian] theta
	real            ecthetad, ectheta
c                       echelle blaze angle [degree], [radian] delta
	real            ecdeltad, ecdelta
C               --------------------------------------------------------
C               Generic Cross Disperser Parameters
c                       In an operational instrument it may be possible
c                       to interchange cross dispersers.
c                       Each cross disperser receives a name.
	character       xdispers*(MAXFITSTR)
C               --------------------------------------------------------
C               Grating Cross Disperser Parameters
c                       number of cross dispersing gratings
	integer         nxdgrat
c                       groove spacing on cross dispersing grating [um/groove]
	real            xdsigma
c                       groove frequency on cross dispersing grating [groove/mm]
	real            xdsigmai
c                       cross disperser blaze angle [degree]
	real            xddeltad
c                       Total angle of deviation from the cross disp. grating
c                       This is equivalent to (alpha - beta) in Schroeder.
c                       (see Fig 13.2 and the text following Eq. 13.2.1)
c                       This is the designed angle needed to reflect one
c                       particular wavelength of light from the echelle into
c                       the central axis of the camera.  The actual incident
c                       (alpha) and reflected (beta) angles vary according to
c                       which cross disp. order and wavelength are desired.
c                       [degrees]
	real            xdalfbet
C               --------------------------------------------------------
C               Prism Cross Disperser Parameters
c                       number of prisms used for cross dispersion
	integer         nxdprism
c                       incident angle on the first prism [degrees]
	real            prangind
c                       cross disperser prism apex angles [degrees]
c                       the (0) element is filled with the mean
	real            prapexd(0:MAXPRISM)
c
c
c                       angles between the adjacent faces of prism(i) and
c                       prism(i-1) [degrees]
c                       the (1) element is filled with the mean
	real            prface(MAXPRISM)
c
c
c                       type or types of glass in the various prisms
c                       This should match the name in the Schott Catalog
	character       prglas(0:MAXPRISM)*(MAXFITSTR)
c
c
C               --------------------------------------------------------
COMMENT                 The Echelle Coordinate System
COMMENT         The Echelle Mosaic Coordinate system is the system used by
COMMENT         calculations within the Echelle program.  Assume an observer
COMMENT         who is looking at the focal plane from a viewpoint behind
COMMENT         the focal plane, i.e. such that the observer could only see
COMMENT         the image if the focal plane were a translucent screen.
COMMENT         The echelle coordinate system
COMMENT         as seen by this observer is a right-handed system.
COMMENT         The "x" direction is along the cross dispersion direction.
COMMENT         The "y" direction is along the echelle dispersion direction.
COMMENT         Orders with longer wavelengths have larger "x" coordinates;
COMMENT         thus they are nearer to the "right" of the picture.
COMMENT         Orders with shorter wavelength have smaller "x" coordinates;
COMMENT         thus they are nearer to the "left" of the picture.
COMMENT         The long wavelength end of each order has a larger "y" coord;
COMMENT         thus it is nearer to the "top" of the picture.
COMMENT         The short wavelength end of each order has a smaller "y" coord;
COMMENT         thus it is nearer to the "bottom" of the picture.
COMMENT         Overall, travelling in either the positive "x" or the
COMMENT         positive "y" direction takes you to a longer wavelength.
COMMENT         The y-origin/x-axis of this system is a horizontal line
COMMENT         running along the blaze wavelength of each order.
COMMENT         The x-origin of this system is somewhat arbitrary, and it is
COMMENT         currently set at the x-position of the blaze wavelength of
COMMENT         the bluest (leftmost) order.
COMMENT
COMMENT         The Echelle coordinate system was defined so as to match the
COMMENT         appearance of the coordinate system used by Schroeder in
COMMENT         figure 15.10 of his text.  This definition also matches
COMMENT         that of the original code as written by Schroeder.
COMMENT         Internal calculations about the position of a point in the
COMMENT         Echelle coordinate system are always carried out under the
COMMENT         above definitions of x and y.  It is recognized that there
COMMENT         will be spectrographs where the handedness of the echelle
COMMENT         and cross dispersion directions will be opposite of that
COMMENT         described here.  In order to allow the modelling of such
COMMENT         spectrographs, the following keyword can be used.
COMMENT
COMMENT         If this variable is .false., then the long wavelength end
COMMENT         of each echelle order has a smaller "y" coordinate as seen
COMMENT         by the observer behind the focal plane.  Note that the echelle
COMMENT         coordinate system as described above always holds.
COMMENT         This variable merely introduces a flip in the transformation
COMMENT         matrix between Echelle and Detector coordinate systems.
	logical         stdechfh
COMMENT         Note that this variable should be .false. for the
COMMENT         Hamilton Echelle spectrograph.
COMMENT         --------------------------------------------------------
COMMENT                 Relative Orientation of Coordinate Systems
COMMENT         The format of the echelle spectrum is likely to be larger than
COMMENT         the solid-state detectors currently in use.  Sampling of the
COMMENT         entire spectrum requires that the detector and the light be
COMMENT         able to move with respect to each other.  This may be performed
COMMENT         by translating the detector across the focal plane.  It may
COMMENT         be performed by rotating the dispersing elements.  The echelle
COMMENT         software allows for such relative translation of the echelle
COMMENT         and detector coordinate systems.  It does this assuming that
COMMENT         these coordinate systems do not rotate while translating.
COMMENT         The angle between the x-axis of the Echelle and Detector Focal
COMMENT         coordinate systems is specified here at its constant value.
COMMENT         (It is assumed that the 2 systems are coplanar.)
COMMENT         This angle is measured in the Echelle Coordinate
COMMENT         system.  It is measured from the x-axis of the Detector Focal
COMMENT         Plane system to the x-axis of the Echelle system.
COMMENT         This angle is measured in a counter-clockwise sense.
COMMENT         Note that as seen by the observer who is behind the detector
COMMENT         (i.e., the detector is between the observer and the incoming
COMMENT         light) both of these coordinate systems are right-handed
COMMENT         (when they have standard orientations and handednesses).
	real            fprotang
COMMENT         ========================================================
COMMENT         The following keyword has been invented for the sake of
COMMENT         the instrument simulator.  It is not presently
COMMENT         intended to be used in the actual Keck data acquisition
COMMENT         system.
c                       which method of specifying the collimator was used?
c                       this should have one of two values, either
c                       i_COLFOCLN or i_COLLDIAM
	integer         collmeth
C       ================================================================
	integer         i_XDISPERS
	integer         i_TELESCOP
	integer         i_INSTRUME
	integer         i_PRIMDIAM
	integer         i_COLLDIAM
	integer         i_CAMFOCLN
	integer         i_ECSIGMA
	integer         i_ECSIGMAI
	integer         i_ECTHETAD
	integer         i_ECTHETA
	integer         i_ECDELTAD
	integer         i_ECDELTA
	integer         i_FOCSCALE
	integer         i_COLFOCLN
	integer         i_NXDGRAT
	integer         i_XDSIGMA
	integer         i_XDSIGMAI
	integer         i_XDDELTAD
	integer         i_XDALFBET
	integer         i_STDECHFH
	integer         i_FPROTANG
	integer         i_NXDPRISM
	integer         i_PRANGIND
	integer         i_PRAPEXD
	integer         i_PRFACE
	integer         i_PRGLAS
	integer         i_wizard
c       Extreme care must be taken with these pointers to insure that
c       the data in the equivalence do not overlap.
	parameter       (i_XDISPERS = -3)
	parameter       (i_TELESCOP = -2)
	parameter       (i_INSTRUME = -1)
	parameter       (i_PRIMDIAM =  0)
	parameter       (i_COLLDIAM =  1)
	parameter       (i_CAMFOCLN =  2)
	parameter       (i_ECSIGMA  =  3)
	parameter       (i_ECSIGMAI =  4)
	parameter       (i_ECTHETAD =  5)
	parameter       (i_ECTHETA  =  6)
	parameter       (i_ECDELTAD =  7)
	parameter       (i_ECDELTA  =  8)
	parameter       (i_FOCSCALE =  9)
	parameter       (i_COLFOCLN = 10)

	parameter       (i_NXDGRAT  = i_COLFOCLN + 1)
	parameter       (i_XDSIGMA  = i_NXDGRAT  + 1)
	parameter       (i_XDSIGMAI = i_XDSIGMA  + 1)
	parameter       (i_XDDELTAD = i_XDSIGMAI + 1)
	parameter       (i_XDALFBET = i_XDDELTAD + 1)

	parameter       (i_STDECHFH = i_XDALFBET + 1)
	parameter       (i_FPROTANG = i_STDECHFH + 1)

	parameter       (i_NXDPRISM = i_FPROTANG + 1)
	parameter       (i_PRANGIND = i_NXDPRISM + 1)
	parameter       (i_PRAPEXD  = i_PRANGIND + 1)
	parameter       (i_PRFACE   = i_PRAPEXD + MAXPRISM + 1)
	parameter       (i_PRGLAS   = i_PRFACE  + MAXPRISM)
	parameter       (i_wizard   = i_PRGLAS  + MAXPRISM + 1)
C       ----------------------------------------------------------------
c                       these are not as crucial if they get set wrong
	integer         i_PRAPEX0
	integer         i_PRAPEX1
	integer         i_PRAPEX2
	integer         i_PRAPEX3
	parameter       (i_PRAPEX0 = i_PRAPEXD + 0)
	parameter       (i_PRAPEX1 = i_PRAPEXD + 1)
	parameter       (i_PRAPEX2 = i_PRAPEXD + 2)
	parameter       (i_PRAPEX3 = i_PRAPEXD + 3)
	integer         i_PRFACE1
	integer         i_PRFACE2
	integer         i_PRFACE3
	parameter       (i_PRFACE1 = i_PRFACE + 0)
	parameter       (i_PRFACE2 = i_PRFACE + 1)
	parameter       (i_PRFACE3 = i_PRFACE + 2)
	integer         i_PRGLAS0
	integer         i_PRGLAS1
	integer         i_PRGLAS2
	integer         i_PRGLAS3
	parameter       (i_PRGLAS0 = i_PRGLAS + 0)
	parameter       (i_PRGLAS1 = i_PRGLAS + 1)
	parameter       (i_PRGLAS2 = i_PRGLAS + 2)
	parameter       (i_PRGLAS3 = i_PRGLAS + 3)
C       ----------------------------------------------------------------
c                       lower limit of array of menu entries
	integer         MINIDATA
	parameter       (MINIDATA = i_XDISPERS)
c                       upper limit of array of menu entries
	integer         MAXIDATA
	parameter       (MAXIDATA = i_wizard)
c                       bogus pointer value
	integer         NULIDATA
	parameter       (NULIDATA = MINIDATA - 1)
C       ================================================================
c                       equivalence to all of the above
	real            idatar(MINIDATA:MAXIDATA)
	integer         idatai(MINIDATA:MAXIDATA)
	logical         idatab(MINIDATA:MAXIDATA)
	equivalence     (idatar, idatai, idatab)
c                       some items come with accelerator keys
c                       that allow their values to be changed.
	character*1     idataa(MINIDATA:MAXIDATA)
c                       pointers telling which ones to actually display
c                       and in what order to display them
c                       if the pointer value is NULIDATA, it is skipped
	integer         idatap(MINIDATA:MAXIDATA)
c                       format to use to display each value
	character       idataf(MINIDATA:MAXIDATA)*(LENFMT)
c                       character strings to use to display label strings
	character       idatal(MINIDATA:MAXIDATA)*(LLABSTR)
c                       character strings to use to display value strings
	character   idatav(SVOLD:SVNEW,MINIDATA:MAXIDATA)*(MAXFITSTR)
c                       positions of the labels in Normalized Device Coords
	real            idatax(Xp:Yp,MINIDATA:MAXIDATA,2)
C       ----------------------------------------------------------------
	equivalence     (primdiam, idatar(i_PRIMDIAM))
	equivalence     (colldiam, idatar(i_COLLDIAM))
	equivalence     (camfocln, idatar(i_CAMFOCLN))
	equivalence     (ecsigma , idatar(i_ECSIGMA ))
	equivalence     (ecsigmai, idatar(i_ECSIGMAI))
	equivalence     (ecthetad, idatar(i_ECTHETAD))
	equivalence     (ectheta , idatar(i_ECTHETA ))
	equivalence     (ecdeltad, idatar(i_ECDELTAD))
	equivalence     (ecdelta , idatar(i_ECDELTA ))
	equivalence     (focscale, idatar(i_FOCSCALE))
	equivalence     (colfocln, idatar(i_COLFOCLN))
	equivalence     (nxdgrat , idatai(i_NXDGRAT ))
	equivalence     (xdsigma , idatar(i_XDSIGMA ))
	equivalence     (xdsigmai, idatar(i_XDSIGMAI))
	equivalence     (xddeltad, idatar(i_XDDELTAD))
	equivalence     (xdalfbet, idatar(i_XDALFBET))
	equivalence     (stdechfh, idatab(i_STDECHFH))
	equivalence     (fprotang, idatar(i_FPROTANG))
	equivalence     (nxdprism, idatai(i_NXDPRISM))
	equivalence     (prangind, idatar(i_PRANGIND))
	equivalence     (prapexd , idatar(i_PRAPEXD ))
	equivalence     (prface  , idatar(i_PRFACE  ))
	equivalence     (prglas  , idatav(SVNEW,i_PRGLAS  ))
	equivalence     (telescop, idatav(SVNEW,i_TELESCOP))
	equivalence     (instrume, idatav(SVNEW,i_INSTRUME))
	equivalence     (xdispers, idatav(SVNEW,i_XDISPERS))
C       ----------------------------------------------------------------
c                       is this menu currently displayed?
	logical         idataup
c                       is this menu currently initialized?
	logical         imeninit
c                       NDC y limits of this menu
	real            iminndc
	real            imaxndc
c                       bottom entry to be displayed
	integer         botidata
C       ----------------------------------------------------------------
	common  /instrument/
     &      idatar, idatap, iminndc, imaxndc, idatax,
     &      idataup, botidata, imeninit, collmeth
	common  /inststr/       idataf, idatal, idatav, idataa
