;+ 
; NAME:
; x_apall   
;   Version 1.0
;
; PURPOSE:
;    Plots any array interactively
;
; CALLING SEQUENCE:
;   
;   x_apall, ydat, [head]
;
; INPUTS:
;   ydat       - Values 
;   [head]     - Header
;
; RETURNS:
;
; OUTPUTS:
;
; OPTIONAL KEYWORDS:
;   wave       - wavelength array
;   DISPLAY    - Display the sky subtracted image with xatv
;
; OPTIONAL OUTPUTS:
;
; COMMENTS:
;
; EXAMPLES:
;   x_apall, 'spec.fits'
;
;
; PROCEDURES/FUNCTIONS CALLED:
;
; REVISION HISTORY:
;   19-Nov-2001 Written by JXP
;-
;------------------------------------------------------------------------------

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

function x_apall, file, OVR=ovr, CLINE=cline, APER=aper, $
                  SKYFUNC=skyfunc, SKYNORD=skynord, SKYREG=skyreg, $
                  DISPLAY=DISPLAY, NOREJ=norej


;
  if  N_params() LT 1  then begin 
    print,'Syntax - ' + $
             'x_apall, file, OVR=, CLINE=, APER= ] (V1.0))'
    print, '          SKYFUNC=, SKYNORD=, SKYREG=, /DISPLAY, /NOREJ'
    return, -1
  endif 

;  Optional Keywords


; Read in ydat 

  dat = mrdfits(file, /fscale)
  sz = size(dat, /dimensions)

; Overscan

  if not keyword_set( OVR ) then begin
      ovr = [sz[0]-50, sz[0]-1]
  endif

  ov = djs_median(dat[ovr[0]:ovr[1],*],1)
  ovdat = fltarr(ovr[1], sz[1])
  for i=0L,ovr[1]-1 do ovdat[i,*] = dat[i,*] - ov

; CLINE

  if not keyword_set( CLINE ) then  cline = ovr[1]/2

; Set Aperture

  if not keyword_set( APER ) then $
    aper = x_setaper(ovdat, CLINE=cline)

; Set Sky Region

  if not keyword_set( SKYFUNC ) then skyfunc = 'POLY'
  if not keyword_set( SKYNORD ) then skynord = 2

  if not keyword_set( NOREJ ) then $
    fit = x1dfit(findgen(sz[1]), ovdat[cline,*], /inter, FUNC=skyfunc, $
                 NORD=skynord, REG=skyreg, LSIG=2., HSIG=2.) $
  else $
    fit = x1dfit(findgen(sz[1]), ovdat[cline,*], /inter, FUNC=skyfunc, $
                 NORD=skynord, REG=skyreg) 

; Subtract Sky
  
  sky = fltarr(ovr[1], aper[1]-aper[0]+1)
  for j=0L,ovr[1]-1 do begin 
      ; FIT
      if not keyword_set( NOREJ ) then $
        fit = x_fitrej(findgen(sz[1]), ovdat[j,*], skyfunc, skynord, $
                       REG=skyreg, LSIGMA=2., HSIGMA=2., NITER=1) $
      else fit = x_fit(findgen(sz[1]), ovdat[j,*], skyfunc, skynord, $
                       REG=skyreg) 
      ; SAVE to SKY
      sky[j,*] = fit[aper[0]:aper[1]]
  endfor
  fimg = temporary(ovdat[*,aper[0]:aper[1]]) - temporary(sky)

;
  if keyword_set( DISPLAY ) then xatv, fimg, /block

; Extract Box Car

  spec = total(fimg, 2)

return, spec
end
