;+ 
; NAME:
; x_setaper   
;       Version 1.0
;
; PURPOSE:
;    Plots any array interactively
;
; CALLING SEQUENCE:
;   
;   x_setaper, img, [head]
;
; INPUTS:
;   img       - Values 
;
; RETURNS:
;
; OUTPUTS:
;
; OPTIONAL KEYWORDS:
;   xsize      - Draw window xsize (pixels)
;   ysize      - Draw window ysize (pixels)
;   CLINE      - Center line for 2D image
;
; OPTIONAL OUTPUTS:
;
; COMMENTS:
;
; EXAMPLES:
;   x_setaper, 'spec.fits'
;
;
; PROCEDURES/FUNCTIONS CALLED:
;  XGETX_PLT
;  XGETY_PLT
;  XGETXPIX_PLT
;  XGETYPIX_PLT
;
; REVISION HISTORY:
;   17-Nov-2001 Written by JXP
;-
;------------------------------------------------------------------------------

;;;;
; Events
;;;;

pro x_setaper_event, ev

  WIDGET_CONTROL, ev.top, get_uvalue = state, /no_copy
  WIDGET_CONTROL, ev.id, get_uvalue = uval

  case uval of
      'ERRORB' : widget_control, state.error_msg_id, set_value=''
      'DRAW_BASE' : begin
          if ev.enter EQ 0 then begin ; Turn off keyboard
              widget_control, state.text_id, sensitive = 0
          endif
          WIDGET_CONTROL, state.base_id, set_uvalue = state, /no_copy
          return
      end
      'DRAW' : begin
          widget_control, state.text_id, /sensitive, /input_focus ; Turn on keyb
          case ev.type of
              0 : begin ; Button press
                  case ev.press of
                      1 : begin   ; Left Aperture
                          state.aper[0] = xgetx_plt(state.xcurs,$
                                                    state.pos,state.xymnx,$
                                                    state.size) 
                          if state.flg_aper MOD 2 NE 1 then $
                            state.flg_aper = state.flg_aper + 1
                      end 
                      4 : begin
                          state.aper[1] = xgetx_plt(state.xcurs,$
                                                    state.pos,state.xymnx,$
                                                    state.size) 
                          if state.flg_aper MOD 4 LT 2 then $
                            state.flg_aper = state.flg_aper + 2
                      end 
                      else:
                  endcase
              end
              1 : ; Button release
              2 : begin ; Motion event
                  state.xcurs = ev.x
                  state.ycurs = ev.y
                  WIDGET_CONTROL, state.base_id, set_uvalue = state, /no_copy
                  return
              end
          endcase
      end
      'TEXT' : begin
          eventch = string(ev.ch)
          if (state.flg_reg EQ 1 AND eventch NE 's') then begin
              widget_control, state.error_msg_id, $
                set_value='Expecting another s !!'
              WIDGET_CONTROL, state.base_id, set_uvalue = state, /no_copy
              return
          endif
          case eventch of
              'b': state.xymnx[1] = xgety_plt(state.ycurs,state.pos,state.xymnx,$
                                             state.size)  ; bottom
              'l': state.xymnx[0] = xgetx_plt(state.xcurs,state.pos,state.xymnx,$
                                             state.size)  ; left
              'r': state.xymnx[2] = xgetx_plt(state.xcurs,state.pos,state.xymnx,$
                                             state.size)  ; right
              't': state.xymnx[3] = xgety_plt(state.ycurs,state.pos,state.xymnx,$
                                             state.size)  ; top
              'Z': begin
                  state.xymnx[0] = xgetx_plt(state.xcurs,state.pos,state.xymnx,$
                                             state.size) - 20. ; left
                  state.xymnx[2] = state.xymnx[0]+40. ; right
              end
              'z': state.xymnx[1] = 0.0 ; Set ymin to 0.0
              'N': state.xymnx[3] = 1.1 ; Set ymax to 1.1
              's': begin  ; Region
                  if(state.flg_reg EQ 0) then begin
                      state.tmpxy[1] = xgety_plt(state.ycurs,state.pos,state.xymnx,$
                                                 state.size) ; bottom
                      state.tmpxy[0] = xgetx_plt(state.xcurs,state.pos,state.xymnx,$
                                                 state.size) ; left
                      state.flg_reg = 1
                      WIDGET_CONTROL, state.base_id, set_uvalue = state, /no_copy
                      return
                  end else begin
                      state.tmpxy[2] = xgetx_plt(state.xcurs,state.pos,state.xymnx,$
                                                 state.size) ; right
                      state.tmpxy[3] = xgety_plt(state.ycurs,state.pos,state.xymnx,$
                                                 state.size) ; top
                      state.flg_reg = 0
                      tmpxy = state.xymnx
                      state.xymnx[0] = state.tmpxy[0] < state.tmpxy[2]
                      state.xymnx[2] = state.tmpxy[0] > state.tmpxy[2]
                      state.xymnx[1] = state.tmpxy[1] < state.tmpxy[3]
                      state.xymnx[3] = state.tmpxy[1] > state.tmpxy[3]
                  endelse
              end
              'w': state.xymnx = state.svxymnx ; Reset the screen
              'W': state.xymnx = state.svxymnx ; Reset the screen
              'q': begin
                  tmp1 = state.aper
                  state.aper[0] = tmp1[0] < tmp1[1]
                  state.aper[1] = tmp1[0] > tmp1[1]
                  *state.pnt_aper = state.aper
                  widget_control, ev.top, /destroy
                  return
              end
              else:  ; Nothing
          endcase
      end
  endcase

  WIDGET_CONTROL, state.base_id, set_uvalue = state, /no_copy
; Update Plot
  xsetaper_UpdatePlot, ev.top
end
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;
;  Plot
;;;;;;;;;;;;;;;;;;;;


pro xsetaper_UpdatePlot, base_id
  
; State

  widget_control, base_id, get_uvalue=state, /no_copy

; Plot Data

  widget_control, state.draw_id, get_value=wind
  wset, wind

  plot, state.xdat, state.ydat, psym=state.psym, $
    position=state.pos, xrange=[state.xymnx[0],state.xymnx[2]], $
    yrange=[state.xymnx[1],state.xymnx[3]], xstyle=1, ystyle=1, $
    title=state.title, $
    background=fsc_color('white'), $
    color=fsc_color('blue'), $
    xcharsize=1.5
    ycharsize=1.5
      
; Apertures

    if state.flg_aper MOD 2 EQ 1 then begin
        oplot, [state.aper[0], state.aper[0]], [state.xymnx[1],state.xymnx[3]], $
          color=fsc_color('green')
    endif
    if state.flg_aper MOD 4 GT 2 then begin
        oplot, [state.aper[1], state.aper[1]], [state.xymnx[1],state.xymnx[3]], $
          color=fsc_color('green')
    endif

; Return State

  widget_control, base_id, set_uvalue=state, /no_copy

end

;;;;;;;;;;;;;;;;;;;;
;  Reset
;;;;;;;;;;;;;;;;;;;;

pro xsetaper_Reset, base_id

; State

  widget_control, base_id, get_uvalue=state, /no_copy
;
  state.ntot = n_elements(state.ydat)

; Plotting
  state.xymnx = state.svxymnx

;
  widget_control, base_id, set_uvalue=state, /no_copy

end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

function x_setaper, img, XSIZE=xsize, YSIZE=ysize, CLINE=cline

;
  if  N_params() LT 1  then begin 
    print,'Syntax - ' + $
             'aper = x_setaper(img, XSIZE=,YSIZE=, CLINE=) [V1.0])'
    return, -1
  endif 

;  Optional Keywords

  if not keyword_set( XSIZE ) then    xsize = 1000
  if not keyword_set( YSIZE ) then    ysize = 600
  
  sz = size(img)
  if sz[0] EQ 2 then begin
      if not keyword_set( CLINE ) then cline = sz[1]/2
      ydat = djs_median(img[cline-10:cline+10, *], 1)
  endif

;    Pointer for Aperture
      aper = fltarr(2)
      pnt_aper = PTR_NEW( aper )


;    STATE
  state = { ydat: ydat, $
            xdat: findgen(n_elements(ydat)), $
            ntot: n_elements(ydat), $
            pos: [0.1,0.1,0.95,0.95], $ ; Plotting
            svxymnx: [0., $     ; xmin
                      min(ydat)-0.01*abs(max(ydat)-min(ydat)), $ ; ymin
                      float(n_elements(ydat)-1), $ ; xmax
                      max(ydat)+0.01*abs(max(ydat)-min(ydat))], $ ; ymax
            xymnx: fltarr(4), $
            tmpxy: fltarr(4), $
            pnt_aper: pnt_aper, $
            aper: fltarr(2), $
            flg_aper: 0, $
            flg_reg: 0, $
            psym: 10, $
            title: '', $
            size: intarr(2), $
            xcurs: 0.0, $
            ycurs: 0.0, $
            base_id: 0L, $      ; Widgets
            lblordr_id: 0L, $
            draw_base_id: 0L, $
            draw_id: 0L, $
            text_id: 0L, $
            funclist_id: 0L, $
            error_msg_id: 0L $
          }
  
;    Title
  if keyword_set( TITLE ) then state.title = title

;    WIDGET
  base = WIDGET_BASE( title = 'x_setaper', /column)
  state.base_id = base
  
;      Toolbar
          
  toolbar = WIDGET_BASE( state.base_id, /row, /frame, /base_align_center,$
                           /align_center)
;        Version 
;  strlbl = strarr(10)
;  strlbl = ['x_setaper', ' ', 'Ver 1.0']
;  verslabel = WIDGET_TEXT(toolbar, value=strlbl, xsize=15, ysize=3)

;        Help
;  strhelp = strarr(50)
;  strhelp = ['   Help Menu   ',$
;             'l -- Set Left Window',$
;             'r -- Set Right Window',$
;             'b -- Set Bottom Window',$
;             't -- Set Top Window',$
;             'w -- Reset screen',$
;             'q -- Quit and save'$
;            ]
;  help_text_id = widget_text(toolbar, value=strhelp, xsize=30, ysize=4,$
;                                 /scroll)

;      Drawing
  state.size[0] = xsize
  state.size[1] = ysize
  state.draw_base_id = widget_base(base, /column, /base_align_left, $
                                   uvalue='DRAW_BASE', frame=2, $
                                   /tracking_events)

  state.draw_id = widget_draw(state.draw_base_id, xsize=state.size[0], $
                              ysize=state.size[1], /frame, $
                              /button_events, /motion_events, uvalue='DRAW')
;      Text
  state.text_id = widget_text(base, $
                              /all_events, $
                              scr_xsize = 1, $
                              scr_ysize = 1, $
                              units = 0, $
                              uvalue = 'TEXT', $
                              value = '')
;      Done
;  done = WIDGET_BUTTON(toolbar, value='Done',uvalue='DONE', /align_right)
  
; Realize
  WIDGET_CONTROL, base, /realize
  
; Color Table 
  loadct, 0, /silent
  
; Update
  WIDGET_CONTROL, base, set_uvalue = state, /no_copy
  xsetaper_Reset, base
  xsetaper_UpdatePlot, base

; Send to the xmanager
  xmanager, 'x_setaper', base
  aper = *pnt_aper
  PTR_FREE, pnt_aper
  return, round(aper)
end
