;+ 
; NAME:
; x_splot   
;     Version 1.1
;
; PURPOSE:
;    Plots any array interactively
;
; CALLING SEQUENCE:
;   
;   x_splot, [xdat], ydat, XSIZE=, YSIZE=, TITLE=, YTWO=
;
; INPUTS:
;   [xdat]     - x values (optional)
;   ydat       - Values 
;
; RETURNS:
;
; OUTPUTS:
;
; OPTIONAL KEYWORDS:
;   xsize      - Draw window xsize (pixels)
;   ysize      - Draw window ysize (pixels)
;   TITLE
;   YTWO
;
; OPTIONAL OUTPUTS:
;
; COMMENTS:
;
; EXAMPLES:
;   x_splot, y
;
;
; PROCEDURES/FUNCTIONS CALLED:
;  XGETX_PLT
;  XGETY_PLT
;  XGETXPIX_PLT
;  XGETYPIX_PLT
;
; REVISION HISTORY:
;   17-Nov-2001 Written by JXP (modified from x1dfit)
;   24-Nov-2001 Added ytwo
;-
;------------------------------------------------------------------------------

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro x_splot_initcomm

common x_splot_common, xdat, ydat, y2

end

;;;;
; Events
;;;;

pro x_splot_event, ev

  WIDGET_CONTROL, ev.top, get_uvalue = state, /no_copy
  WIDGET_CONTROL, ev.id, get_uvalue = uval

  case uval of
      'ERRORB' : widget_control, state.error_msg_id, set_value=''
      'DRAW_BASE' : begin
          if ev.enter EQ 0 then begin ; Turn off keyboard
              widget_control, state.text_id, sensitive = 0
          endif
          WIDGET_CONTROL, state.base_id, set_uvalue = state, /no_copy
          return
      end
      'DRAW' : begin
          widget_control, state.text_id, /sensitive, /input_focus ; Turn on keyb
          case ev.type of
              0 : begin ; Button press
                  case ev.press of
                      1 : begin
                          if(state.flg_reg EQ 0) then begin
                              state.tmpxy[1] = xgety_plt(state.ycurs,state.pos,$
                                                         state.xymnx,$
                                                         state.size) ; bottom
                              state.tmpxy[0] = xgetx_plt(state.xcurs,state.pos,$
                                                         state.xymnx,$
                                                         state.size) ; left
                              state.flg_reg = 1
                              WIDGET_CONTROL, state.base_id, set_uvalue = $
                                state, /no_copy
                              return
                          end else begin
                              state.tmpxy[2] = xgetx_plt(state.xcurs,state.pos,$
                                                         state.xymnx,$
                                                         state.size) ; right
                              state.tmpxy[3] = xgety_plt(state.ycurs,state.pos,$
                                                         state.xymnx,$
                                                         state.size) ; top
                              state.flg_reg = 0
                              tmpxy = state.xymnx
                              state.xymnx[0] = state.tmpxy[0] < state.tmpxy[2]
                              state.xymnx[2] = state.tmpxy[0] > state.tmpxy[2]
                              state.xymnx[1] = state.tmpxy[1] < state.tmpxy[3]
                              state.xymnx[3] = state.tmpxy[1] > state.tmpxy[3]
                          endelse
                      end
                      4 : 
                      else :
                  endcase
              end
              1 : ; Button release
              2 : begin ; Motion event
                  state.xcurs = ev.x
                  state.ycurs = ev.y
                  WIDGET_CONTROL, state.base_id, set_uvalue = state, /no_copy
                  return
              end
          endcase
      end
      'TEXT' : begin
          eventch = string(ev.ch)
          if (state.flg_reg EQ 1 AND eventch NE 's') then begin
              widget_control, state.error_msg_id, $
                set_value='Expecting another s !!'
              WIDGET_CONTROL, state.base_id, set_uvalue = state, /no_copy
              return
          endif
          case eventch of
              'b': state.xymnx[1] = xgety_plt(state.ycurs,state.pos,state.xymnx,$
                                             state.size)  ; bottom
              'z': state.xymnx[1] = 0.0
              'l': state.xymnx[0] = xgetx_plt(state.xcurs,state.pos,state.xymnx,$
                                             state.size)  ; left
              'r': state.xymnx[2] = xgetx_plt(state.xcurs,state.pos,state.xymnx,$
                                             state.size)  ; right
              't': state.xymnx[3] = xgety_plt(state.ycurs,state.pos,state.xymnx,$
                                             state.size)  ; top
              'N': state.xymnx[3] = 1.1
              'h': state.psym = 10
              's': begin  ; Region
                  if(state.flg_reg EQ 0) then begin
                      state.tmpxy[1] = xgety_plt(state.ycurs,state.pos,state.xymnx,$
                                                 state.size) ; bottom
                      state.tmpxy[0] = xgetx_plt(state.xcurs,state.pos,state.xymnx,$
                                                 state.size) ; left
                      state.flg_reg = 1
                      WIDGET_CONTROL, state.base_id, set_uvalue = state, /no_copy
                      return
                  end else begin
                      state.tmpxy[2] = xgetx_plt(state.xcurs,state.pos,state.xymnx,$
                                                 state.size) ; right
                      state.tmpxy[3] = xgety_plt(state.ycurs,state.pos,state.xymnx,$
                                                 state.size) ; top
                      state.flg_reg = 0
                      tmpxy = state.xymnx
                      state.xymnx[0] = state.tmpxy[0] < state.tmpxy[2]
                      state.xymnx[2] = state.tmpxy[0] > state.tmpxy[2]
                      state.xymnx[1] = state.tmpxy[1] < state.tmpxy[3]
                      state.xymnx[3] = state.tmpxy[1] > state.tmpxy[3]
                  endelse
              end
              'w': state.xymnx = state.svxymnx ; Reset the screen
              'W': state.xymnx = state.svxymnx ; Reset the screen
              'q': begin
                  widget_control, ev.top, /destroy
                  return
              end
              else:  ; Nothing
          endcase
      end
      'DONE' : begin
          widget_control, ev.top, /destroy
          return
      end
  endcase

  WIDGET_CONTROL, state.base_id, set_uvalue = state, /no_copy
; Update Plot
  xsplot_UpdatePlot, ev.top
end
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;
;  Plot
;;;;;;;;;;;;;;;;;;;;


pro xsplot_UpdatePlot, base_id
  
common x_splot_common

; State

  widget_control, base_id, get_uvalue=state, /no_copy

; Plot Data

  widget_control, state.draw_id, get_value=wind
  wset, wind

  color = getcolor(/load)
  plot, xdat, ydat, psym=state.psym, $
    position=state.pos, xrange=[state.xymnx[0],state.xymnx[2]], $
    yrange=[state.xymnx[1],state.xymnx[3]], xstyle=1, ystyle=1, $
    title=state.title, background=color.white, $
    xcharsize=1.5, $
    ycharsize=1.5, $
    color=color.black

; y2

  if state.flg_y2 then oplot, xdat, y2, color=color.red
      
; Return State

  widget_control, base_id, set_uvalue=state, /no_copy

end

;;;;;;;;;;;;;;;;;;;;
;  Reset
;;;;;;;;;;;;;;;;;;;;

pro xsplot_Reset, base_id

common x_splot_common

; State

  widget_control, base_id, get_uvalue=state, /no_copy

;
  state.ntot = n_elements(ydat)

; Plotting

  state.xymnx = state.svxymnx

;

  widget_control, base_id, set_uvalue=state, /no_copy

end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro x_splot, xin, yin, XSIZE=xsize, YSIZE=ysize, TITLE=title, YTWO=ytwo

common x_splot_common
;
  if  N_params() LT 1  then begin 
    print,'Syntax - ' + $
             'x_splot, [xdat], ydat, YTWO=ytwo, XSIZE=, YSIZE=, TITLE= (V1.1))'
    return
  endif 

; Set xdat, ydat

  x_splot_initcomm

  if keyword_set( yin ) then begin
      if n_elements(xin) NE n_elements(yin) then begin
          print, 'x_splot: Wrong array sizes!'
          return
      endif
      xdat = xin
      ydat = yin
  endif else begin
      xdat = findgen( n_elements(xin) )
      ydat = xin
  endelse

  if keyword_set( YTWO ) then y2 = ytwo
  
;  Optional Keywords

  if not keyword_set( XSIZE ) then    xsize = 1000
  if not keyword_set( YSIZE ) then    ysize = 600

;    STATE
  state = { ntot: n_elements(ydat), $
            reg: fltarr(100,2), $
            flg_y2: 0, $
            flg_reg: 0, $
            pos: [0.1,0.1,0.95,0.95], $ ; Plotting
            svxymnx: [ min(xdat)-0.01*abs(max(xdat)-min(xdat)), $ ; xmin
                      min(ydat)-0.01*abs(max(ydat)-min(ydat)), $ ; ymin
                      max(xdat)+0.01*abs(max(xdat)-min(xdat)), $ ; xmax
                      max(ydat)+0.01*abs(max(ydat)-min(ydat))], $  ; ymax
            xymnx: fltarr(4), $
            tmpxy: fltarr(4), $
            psym: 0, $
            title: '', $
            size: intarr(2), $
            xcurs: 0.0, $
            ycurs: 0.0, $
            base_id: 0L, $      ; Widgets
            lblordr_id: 0L, $
            draw_base_id: 0L, $
            draw_id: 0L, $
            text_id: 0L, $
            funclist_id: 0L, $
            error_msg_id: 0L $
          }
  
; Flag

  if keyword_set( YTWO ) then state.flg_y2 = 1

; Set xvxymnx[0,2]

  state.svxymnx[0] = min(xdat)
  state.svxymnx[2] = max(xdat)

;    Title
  if keyword_set( TITLE ) then state.title = title

;    WIDGET
  base = WIDGET_BASE( title = 'x_splot', /column)
  state.base_id = base
  
;      Toolbar
          
  toolbar = WIDGET_BASE( state.base_id, /row, /frame, /base_align_center,$
                           /align_center)
;        Version 
;  strlbl = strarr(10)
;  strlbl = ['x_splot', ' ', 'Ver 1.0']
;  verslabel = WIDGET_TEXT(toolbar, value=strlbl, xsize=15, ysize=3)

;        Help
;  strhelp = strarr(50)
;  strhelp = ['   Help Menu   ',$
;             'l -- Set Left Window',$
;             'r -- Set Right Window',$
;             'b -- Set Bottom Window',$
;             't -- Set Top Window',$
;             'w -- Reset screen',$
;             'q -- Quit and save'$
;            ]
;  help_text_id = widget_text(toolbar, value=strhelp, xsize=30, ysize=4,$
;                                 /scroll)

;      Drawing
  state.size[0] = xsize
  state.size[1] = ysize
  state.draw_base_id = widget_base(base, /column, /base_align_left, $
                                   uvalue='DRAW_BASE', frame=2, $
                                   /tracking_events)

  state.draw_id = widget_draw(state.draw_base_id, xsize=state.size[0], $
                              ysize=state.size[1], /frame, $
                              /button_events, /motion_events, uvalue='DRAW')
;      Text
  state.text_id = widget_text(base, $
                              /all_events, $
                              scr_xsize = 1, $
                              scr_ysize = 1, $
                              units = 0, $
                              uvalue = 'TEXT', $
                              value = '')
;      Done
;  done = WIDGET_BUTTON(toolbar, value='Done',uvalue='DONE', /align_right)
  
; Realize
  WIDGET_CONTROL, base, /realize
  
; Color Table 
  
; Update
  WIDGET_CONTROL, base, set_uvalue = state, /no_copy
  xsplot_Reset, base
  xsplot_UpdatePlot, base

; Send to the xmanager
  xmanager, 'x_splot', base, /no_block

return
end
